;-------------------------------------------------------------------------------
;spi - interrupt slave-routine
;NEW: Decide if there's incoming NET-,or HIF-Data...
;NEW:
;NET-Data: store to comNETBUF in segments (stack)
;-------------------------------------------------------------------------------

_SPIslave:
    push    r0
    push    r1
    push    r5
    push    r6
    push    r7
    push    r10
    push    r11
    push    r12
    push    r13
    push    r14
    push    r15
	push	r16	;r16 on stack
	push	r17	;r17 on stack
    push    xl
    push    xh
    push    yl
    push    yh
    push    zl
    push    zh
	in	r5,sreg

;----------------------------------------------------------
;receive HIF-data NEW
;-> check at start if HIF or NET-data will be received
;----------------------------------------------------------
    ;--------------------
    ;load data from spi 
    in  xl,spdr
	;--------------------
    lds	r17,comFLAG
    ;........................
    ;error?
    ;set on:
    ;- identifier wrong
    ;cleared when:
    ;_ifSEL = 1 & timINT
    ;........................
    sbrs    r17,spiBUSYerr
    rjmp    _SPIslaveA         ;no error - goon...
    clr r16
    sts spiDATctIST,r16
    rjmp    _SPIslaveBK         ;no further action



_SPIslaveA:
	;jetzt byte counter laden
	lds	r16,spiDATctIST	
	tst	r16	;identifier byte empfangen ?
	brne	_RECspi02CHKx
    ;-------------------
    ;check start-byte
    ;a) HIF-Data 0x2d
    ;b) NET-Data 0xd4
    ;-------------------
    cpi xl,0x2d
    breq   _RECspiHIFstart
    cpi xl,0xd4
    breq    _RECspiNETstart  
    ;invalid data
	sbr	r17,1<<spiBUSYerr	    
   	sts	comFLAG,r17
    rjmp    _SPIslaveBK         ;no further action

	_RECspi02CHKx:
    jmp 	_RECspi02CHK
;------------------------
;start HIF-data receiving
;------------------------
_RECspiHIFstart:
	;comBUF startadresse
	ldi	zh,high(comBUF)
	ldi	zl,low(comBUF)
	;comBUF sichern
	sts	comBUFadrH,zh
	sts	comBUFadrL,zl
	;&jetzt busy setzen
	cbr	r17,1<<spiBUSYerr	    ;no error
    sbr r17,1<<spiBUSYhif       ;busy with hif-data receiving
   	sts	comFLAG,r17
    ;cursorroutine disabled 
    ;till data receiving
	lds	r17,curMODflTE
	cbr	r17,1<<BOXcursor ;cursorroutine disable
	cbr	r17,1<<BOXactiv
	sts	curMODflTE,r17
    ;--------------------
    ;<spiDATctIST> +1
    ;--------------------
    lds	r16,spiDATctIST		
    inc r16
    sts spiDATctIST,r16
    ;block bus
     ;set _ifSEL low from pullup
    cbi ddrb,_ifSEL
    cbi portb,_ifSEL
    nop
    sbi ddrb,_ifSEL
    cbi portb,_ifSEL
 
    ;
    rjmp    _SPIslaveBK         ;no further action



;------------------------
;start NET-data receiving
;------------------------
_RECspiNETstart:
	;&jetzt busy setzen
	sbr	r17,1<<spiBUSYnet
    cbr	r17,1<<spiBUSYerr	    ;no error	
   	sts	comFLAG,r17
    ;update startadress of frame <comNETpointS>
    lds zl,comNETpointL
    lds zh,comNETpointH
    sts comNETpointSL,zl
    sts comNETpointSH,zh
    ;clear carry
    lds     r17,comNETstate
    cbr     r17,1<<comNETcarry
    sts     comNETstate,r17
    ;--------------------
    ;<spiDATctIST> +1
    ;--------------------
    lds	r16,spiDATctIST		
    inc r16
    sts spiDATctIST,r16
    ;preset 
    ldi r16,0xff
    sts spiDATctSOLL,r16
    ;block bus
    ;set _ifSEL low from pullup
    cbi ddrb,_ifSEL
    cbi portb,_ifSEL
    nop
    sbi ddrb,_ifSEL
    cbi portb,_ifSEL
    ;freeze...
    lds r16,comNETstackFL
    sbr r16,1<<STACKrebuild
    sts comNETstackFL,r16
    ;
    rjmp    _SPIslaveBK         ;no further action
 

;-------------------------
;HIF/NET-DATA ?
;-------------------------
_RECspi02CHK:
   ;check if it's hif-data
    sbrc    r17,spiBUSYhif
    rjmp    _RECspi03HIF        ;goon with hif-data
    ;check if it's hif-data
    sbrc    r17,spiBUSYnet
    rjmp    _RECspi03NET        ;goon with hif-data
    ;invalid state
    ;-> error    
	sbr	r17,1<<spiBUSYerr	    ;no error
   	sts	comFLAG,r17
    rjmp    _SPIslaveBK         ;no further action

 



;------------------------------------------------------------------------------------
;NET-DATA RECEIVING FROM ConcetMX1.xx
;------------------------------------------------------------------------------------
_RECspi03NET:
        lds zl,comNETpointL
        lds zh,comNETpointH
        st  z,xl                ;net data to buffer
        cpi r16,6               ;the sixth byte contains the frame-size
        brne  _RECspi03NETgo
        ;calculate range
        sts spiDATctSOLL,xl     ;the frame size !
        
;---------------------------
;check range
;---------------------------
_RECspi03NETgo:
	    lds	r17,spiDATctSOLL
        cp  r16,r17
        brlo     _RECspiBACK
        ;frame complete
       	;data counter = 0
    	clr	r16
	    sts	spiDATctIST,r16
 	    ;&jetzt free setzen
	    lds	r17,comFLAG
	    cbr	r17,1<<spiBUSYnet	
   	    sts	comFLAG,r17
        rjmp  _RECspiUPDATE                 ;now update stack


;----------------------------
;now update storage pointer
;----------------------------      
_RECspiBACK:
        ;byte counter +1
        inc r16
        sts	spiDATctIST,r16   
 _RECspiBACKz:         
        ;pointer at last storage adress
        ; > limit?
        lds xl,comNETpointL
        lds xh,comNETpointH
        cpi  xl,low(comNETbufferEND)
        ldi r16,high(comNETbufferEND)
        cpc xh,r16
        breq    TOstart
        ;set to next storageadress
        adiw    xh:xl,1
        rjmp    _RECspiRETI
        ;set bufferstartadress &also buffer carry !
        TOstart:
        lds     xl,comNETstate
        sbr     xl,1<<comNETcarry
        sts     comNETstate,xl

        ldi     xl,low(comNETbuffer)
        ldi     xh,high(comNETbuffer)
        _RECspiRETI:
        sts     comNETpointL,xl
        sts     comNETpointH,xh
        rjmp	_SPIslaveBK		;...& back

	








;------------------------------------------------------------------------------------
;HIF-DATA RECEIVING FROM ConcetMX1.xx
;------------------------------------------------------------------------------------

;-------------------------
;HIF-DATA
;<spiDATctIST> =1 ? 
;-------------------------
_RECspi03HIF:
    ;---------------------
    ;save data to buffer
    ;---------------------
	lds	zh,comBUFadrH
	lds	zl,comBUFadrL
	st	z+,xl           ;spi data to buffer
	;comBUF sichern
	sts	comBUFadrH,zh
	sts	comBUFadrL,zl
    ;---------------------
    ;check IST counter
    ;---------------------
    cpi r16,1
    brne   _RECspi03range ;check range
    ;calculate size
	;erstes byte - hier para variable prfen
	sbrc	xl,parG	    ;noch weiter parameter angehngt?
	rjmp	_SPI04sub	;ja!!
	ldi	r17,6		    ;nein == 6 bytes
	sts	spiDATctSOLL,r17
   ;NEW for remote
    sts REMsize,r17
	rjmp	_RECspi03range
	;
	;parameter angehngt...
_SPI04sub:
	andi	xl,0b00001111	;nur anzahl der parameter 
	ldi	zl,4			;pro parameter 4 bytes
	ldi	zh,6			;6 bytes 
	mul	zl,xl		    ;anzahl *4
	add	zh,r0			;6bytes+ xParameter(*4)
	mov	r17,zh	    	;xBytes gesamt
	sts	spiDATctSOLL,r17
    ;NEW for remote
    sts REMsize,r17
	jmp	_RECspi03range

;-------------------------
;check end of frame !
;-------------------------
_RECspi03range:
	lds	r17,spiDATctSOLL
	;range?
	cp	r16,r17		;ist<soll ?
	brlo	_SPI05sub	
	;data counter = 0
	clr	r16
	sts	spiDATctIST,r16
    ;free bus   
    cbi ddrb,_ifSEL
    sbi portb,_ifSEL

    ;........................
    ;check if remote is activ
    ;NOTE:an remote client
    ;never displays the data
    ;on the local screen ! 
    ;........................
    lds r17,REMif
    sbrs    r17,REMon
    rjmp    _SPI06sub           ;ready & no remote
    ;slave?
    sbrc    r17,REMstate
    rjmp    _SPI06sub            ;ready & no slave remote

    ;........................
    ;Yes, it's a remoted
    ;client !
    ;........................
    ;remote & slave
    ;stateSEND
    ldi r17,10
    sts STATEsend,r17
    ;NEW for remote
    ldi r17,REMdisDAT           ;display data on display (slave => master)
    sts REMtype,r17
    rjmp    _SPIslaveBK         ;no further action
  
   


    ;........................
	;not reached the end
    ;...still waiting for data
    ;........................
_SPI05sub:
	inc	r16
	sts	spiDATctIST,r16
  ;cursorroutine disabled 
    ;till data receiving
	lds	r17,curMODflTE
	cbr	r17,1<<BOXcursor ;cursorroutine disable
	cbr	r17,1<<BOXactiv
	sts	curMODflTE,r17
    rjmp    _SPIslaveBK         ;no further action


    ;.........................
	;ready received as non
    ;remoted system/client 
    ;...data will be displayed
    ;on the screen in order
    ;.........................
_SPI06sub:
	;>= fertig
    lds r16,comFLAG
	cbr	r16,1<<spiBUSYhif
	sbr	r16,1<<newHIFdata
	sts	comFLAG,r16
    ;cursorroutine disabled 
    ;till data receiving
	lds	r17,curMODflTE
	cbr	r17,1<<BOXcursor ;cursorroutine disable
	cbr	r17,1<<BOXactiv
	sts	curMODflTE,r17
    ;back...
    rjmp    _SPIslaveBK         ;no further action
 




;......................................................
;back from svr
;......................................................
_SPIslaveBK:
  

	out	sreg,r5
    pop zh
    pop zl
    pop yh
    pop yl
    pop xh
    pop xl
	pop	r17	;restore r17
	pop	r16	;restore r16
    pop r15
    pop r14
    pop r13
    pop r12
    pop r11
    pop r10
    pop r7
    pop r6
	pop	r5	;restore r5
    pop r1
    pop r0
	reti





;---------------------------------------------------------------------------------------------
;NEW - now stack also a kind of fifo... takes max. 40s time !
;---------------------------------------------------------------------------------------------
;<comNETstackCNT> = count of valid dataframes. the start-,endadresses are stored on stacksegments
;<comNETstackSTA> = related startpoint of the stack....
_RECspiUPDATE:
;OK.. now start with the first stack item
clr    zl
sts     comNETsegCNT,zl         ;will be increased by destroy/valid/shift
sts     comNETstackCNT,zl       ;will be increased by destroy/valid
sts     comNETstackFL,zl        ;clear flags
sts     comNETstackPOINT,zl     ;will be increased by destroy/valid/shift

;...................................................
;Ohh.. there are <1> OR more valid items
;stored in stacksegments...
;MEANS: check if the actual data already
;has destroyed an old frame...
;...................................................
;NOW: compare actual start-,endadresses with
;the existing stack items from the lowest to
;the highest stackposition...
;...................................................
;NOTE that a dataframe sometimes will be stored 
;with a buffer carry (buffer reached the endadress
;and is going on at buffer startadress !)
;...................................................  
_RECspiUPDATEloop:
    ;load frame start/endadress in segment
    ;adressed by <comNETstackPOINT>
lds    zl,comNETstackPOINT
ldi    zh,4         
mul    zl,zh                    ;*4            
mov    zl,r0
clr    zh
subi   zl,low(-comNETstack)
sbci   zh,high(-comNETstack)
sts    comNETstPOINTH,zh
sts    comNETstPOINTL,zl
;... MSB of startadress in <zh/zl> & <comNETstPOINT>




;.............................................................
;LOAD start/endadress from stacksegment
;& buffercarry flag !
call   _LDstack
;.............................................................

;.............................................................
;check for empty stack segment
lds     xh,comNETstackFL
sbrs    xh,STACKempty
rjmp    _RECspiUPDATEnotEMPTY
jmp     _DESTROYseg             ;segment empty - store data OR shift stack left

;.............................................................
;OK... lets go !
;FIRST check if ACTUAL Frame is stored with a buffer carry?
;NOTE: carry set/clr by receive loop !
;.............................................................
_RECspiUPDATEnotEMPTY:
lds     xl,comNETstate
sbrs    xl,comNETcarry
rjmp    _RECupdNOCARRYa         ;a) actual segment stored without carry!
rjmp    _RECupdNOCARRYb         ;b) actual segment stored with carry !

;.......................................................................
;CHECK a) actual segment no carry 
;.......................................................................
_RECupdNOCARRYa:        
lds     xh,comNETstackFL
sbrc    xh,STACKupCARRY
rjmp    _RECupdNOCARRYc         ;c) stack segment stored with carry !
rjmp    _RECupdNOCARRYd         ;d) stack segment stored without carry !

;.......................................................................
;CHECK a) actual segment carry  & stack? 
;.......................................................................
_RECupdNOCARRYb:
lds     xh,comNETstackFL
sbrc    xh,STACKupCARRY
rjmp    _DESTROYseg             ;DESTROY -  actual segment with carry & stack segment stored with carry !
rjmp    _RECupdNOCARRYd         ;d) actual segment with carry & stack segment stored without carry !


            ;........................................................................
            ;actual NO carry & stacked carry
            ;actual startadress >= stacked startadress -> destroy
            ;OR
            ;actual startadress <= stacked endadress   -> destroy
            ;OR
            ;actual endadress   >= stacked startadress -> destroy
            ;OR
            ;actual endadress   <= stacked endadress   -> destroy
            ;ELSE 
            ;frame still VALID !
            ;........................................................................
            _RECupdNOCARRYc:
            ;actual
            lds r10,comNETpointL           ;LSB of actual endadress
            lds r11,comNETpointH           ;MSB
            lds r12,comNETpointSL          ;LSB of actual startadress
            lds r13,comNETpointSH          ;MSB
            ;stacked
            lds r14,comNETskITedL          ;LSB of stacked endadress
            lds r15,comNETskITedH          ;MSB
            lds r6,comNETskITstL          ;LSB of stacked startadress
            lds r7,comNETskITstH          ;MSB
            ;compare actual startadress >= stacked startadress
            cp  r12,r6
            cpc r13,r7
            brsh    _DESTROYseg
            ;compare actual startadress <= stacked endadress
            cp  r12,r14
            cpc r13,r15
            breq    _DESTROYseg
            brlo    _DESTROYseg
            ;compare actual endadress   >= stacked startadress
            cp  r10,r6
            cpc r11,r7
            brsh    _DESTROYseg
            ;compare actual endadress   <= stacked endadress
            cp  r10,r14
            cpc r11,r15
            breq    _DESTROYseg
            brlo    _DESTROYseg
            rjmp    _VALIDseg



                    ;........................................................................
                    ;actual & stacked NO carry
                    ;actual startadress >= stacked startadress & <= stacked endadress
                    ;OR 
                    ;actual endadress   >= stacked startadress & <= stacked endadress
                    ;THEN 
                    ;frame DESTROYED !
                    ;ELSE 
                    ;frame still VALID !
                    ;........................................................................
                    _RECupdNOCARRYd:
                    ;actual
                    lds r10,comNETpointL           ;LSB of actual endadress
                    lds r11,comNETpointH           ;MSB
                    lds r12,comNETpointSL          ;LSB of actual startadress
                    lds r13,comNETpointSH          ;MSB
                    ;stacked
                    lds r14,comNETskITedL          ;LSB of stacked endadress
                    lds r15,comNETskITedH          ;MSB
                    lds r6,comNETskITstL          ;LSB of stacked startadress
                    lds r7,comNETskITstH          ;MSB
                    ;compare actual startadress >= stacked startadress
                    cp  r12,r6
                    cpc r13,r7
                    brsh    _CHKstartRANGE
                    ;no - perhaps end adress in stacksegment ?
                    rjmp    _CHKendRANGEa

                    ;...............................................
                    ;actual startadress >= stacked startadress
                    ;compare actual startadress <= stacked endadress
                    ;...............................................
                    _CHKstartRANGE:
                    cp  r12,r14
                    cpc r13,r15
                    breq    _DESTROYseg             ;startadress in stacked range
                    brlo    _DESTROYseg             ;startadress in stacked range

                    ;------------------------------------------------
                    ;LAST checks:
                    ;actual endadress >= stacked startadress
                    ;-------------------------------------------------
                    _CHKendRANGEa:
                    cp  r10,r6
                    cpc r11,r7
                    brsh    _CHKendRANGEb
                    rjmp    _VALIDseg

                    ;------------------------------------------------
                    ;FINAL check:
                    ;actual endadress <= stacked endadress
                    ;-------------------------------------------------
                    _CHKendRANGEb:
                    cp  r10,r14
                    cpc r11,r15
                    breq    _DESTROYseg             ;endadress in stacked range
                    brlo    _DESTROYseg             ;endadress in stacked range
                    rjmp    _VALIDseg









;...........................................................
;invalid segment !
;A) set new start-,endadress to the stack
;clears the segment by:
;* storing new start/endpoints to stack
;* incrementing the temporary stack pointer
;* goon searching in stack
;=> return with corrected: <comNETstackCNTte>
;                          <comNETstackCNT>
;...........................................................
_RECspiUPDATEloopx:
    jmp _RECspiUPDATEloop

_DESTROYseg:
    lds     zh,comNETstackFL
    sbrc    zh,STACKupdate          
    rjmp    _DESTROYsegA            ;stack already updated !
    ;stack not updated !
    sbr     zh,1<<STACKupdate 
    sts     comNETstackFL,zh
    ;store new start-,/endadress
    call    STOREstack
    ;...................................
    ;increase temporary stackpointer +1
    lds    zh,comNETstackPOINT
    inc zh
    sts comNETstackPOINT,zh
    ;...................................

    ;...................................
    ;store also as a activ segment
    lds    zh,comNETstackCNT
    inc zh
    sts comNETstackCNT,zh
    ;...................................

    ;...................................
    ;increase segment counter +1
    lds zh,comNETsegCNT
    inc zh
    sts comNETsegCNT,zh
    ;...................................

 
    ;back in loop
    jmp _RECspiUPDATEloop



    ;.......................................
    ;one stack already updated with
    ;actual data!
    ;= destroy further segment containing
    ;  the actual adress range !
    ;& shift the stack one position left...
    ;.......................................
_DESTROYsegA:
    rjmp    _SHIFTstack







;............................................................
;stores the segment by:
;* incrementing the actual stack sizecounter
;* exit - nomore searching...
;............................................................
_VALIDseg:
    ;...................................
    ;increase temporary stackpointer +1
    lds    zh,comNETstackPOINT
    inc zh
    sts comNETstackPOINT,zh
    ;...................................

    ;...................................
    ;store also as a activ segment
    lds    zh,comNETstackCNT
    inc zh
    sts comNETstackCNT,zh
    ;...................................

    ;...................................
    ;increase segment counter +1
    lds zh,comNETsegCNT
    inc zh
    sts comNETsegCNT,zh
    ;...................................

    ;check range
    cpi zh,10
    brsh  _VALIDsegB              ;>=10 clear the pointers up from  (comNETstackCNT*4) + <comNETstack>
    ;back in loop
    jmp _RECspiUPDATEloop

   ;ready 
_VALIDsegB:
   ;free bus
    cbi ddrb,_ifSEL
    sbi portb,_ifSEL
    rjmp    _RECspiBACKz            ;in order updating <comNETpoint> to next startadress !
   






;...........................................................
;selected stack segment is empty 
;A) store to actual position
;B) or shift left from actual position
;Condition: new data still stored to 
;a stacksegment...
;shift all left -1 segment
;...........................................................
_SHIFTstack:
    ;.................................
    ;first increase segment counter
    lds    r16,comNETsegCNT
    inc    r16
    sts comNETsegCNT,r16
    ;.................................
    ;check range
    cpi r16,10
    brsh  _SHIFTstackB              ;ready
    rjmp   _SHIFTstackC             ;>=10 shift

   ;ready 
_SHIFTstackB:
   ;free bus
    cbi ddrb,_ifSEL
    sbi portb,_ifSEL
    rjmp    _RECspiBACKz            ;in order updating <comNETpoint> to next startadress !
   
 
    
    ;set pointer to buffer-start/endadress
_SHIFTstackC:
    lds zl,comNETstPOINTL           ;LSB of actual stacksegment adress
    lds zh,comNETstPOINTH           ;MSB
    movw    yh:yl,zh:zl
    adiw    zh:zl,4

_SHIFTstackAloop:
    ld      r10,z+
    st      y+,r10
    ld      r10,z+
    st      y+,r10
    ld      r10,z+
    st      y+,r10
    ld      r10,z+
    st      y+,r10
    inc r16
    cpi r16,10
    brne    _SHIFTstackAloop
    ;back in loop
    sbiw    zh:zl,4
    clr r16
    st  z+,r16
    st  z+,r16
    st  z+,r16
    st  z,r16
    jmp _RECspiUPDATEloop


























;---------------------------------------------------
;SUB:load frame start-,endadress from stacksegment X
;& check if the frame is stored with a buffer carry
;---------------------------------------------------
_LDstack:
;set pointer to buffer-start/endadress
lds zl,comNETstPOINTL           ;LSB of actual stacksegment adress
lds zh,comNETstPOINTH           ;MSB
;load segment-startadress MSB from stack
ld      r15,z+               ;msb first
;store to var
sts     comNETskITstH,r15
;load segment-startadress LSB from stack
ld      r15,z+               ;lsb
;store to var
sts     comNETskITstL,r15
;load segment-endadress MSB from stack
ld      r15,z+               ;msb first
;store to var
sts     comNETskITedH,r15
;load segment-endadress LSB from stack
ld      r15,z                ;lsb
;store to var
sts     comNETskITedL,r15
;
;check if stored with a buffer carry !
;endadress < startadresss = carry
;endadress > startadress  = no carry 
lds     r16,comNETstackFL

lds     zl,comNETskITedL
lds     zh,comNETskITedH
lds     xl,comNETskITstL
lds     xh,comNETskITstH
;compare end <=> startadress
cp      zl,xl
cpc     zh,xh
brsh    _LDstackNOCARRY     ;end >= start NO carry
sbr     r16,1<<STACKupCARRY
rjmp    _LDstackret
_LDstackNOCARRY:
cbr     r16,1<<STACKupCARRY
_LDstackret:
cbr     r16,1<<STACKempty
;check for empty segments
lds     zl,comNETskITedL
lds     zh,comNETskITedH
lds     xl,comNETskITstL
lds     xh,comNETskITstH
cp      zl,xl
cpc     zh,xh
brne    _LDstackretB
sbr     r16,1<<STACKempty
_LDstackretB:
sts  comNETstackFL,r16   
ret
;---------------------------------------------------


;--------------------------------------
;SUB:store frame to stack...
;--------------------------------------
STOREstack:
;set pointer to buffer-start/endadress
lds r10,comNETpointL           ;LSB of actual endadress
lds r11,comNETpointH           ;MSB
lds r12,comNETpointSL          ;LSB of actual startadress
lds r13,comNETpointSH          ;MSB
;store segment-startadress to stack
lds     xl,comNETstPOINTL
lds     xh,comNETstPOINTH
st      x+,r13               ;msb first
st      x+,r12
;store segment-endadress to stack
st      x+,r11               ;msb first
st      x+,r10

ret





























