;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;nIRQ based receive/transmit fifo data buffer - Service Routine
;when: 
;- fifo has received amount of preprogrammed bit (8)
;- tx-register is ready to receive the next byte
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
nIRQ_SVR:
  	;save sreg
    push    r0
    push    r1
	push	r4
    push    r5
    push    r16
    push    r17
    push    r23
    push    zl
    push    zh
    push    yl
    push    yh
	in	r4,sreg
    
    ;....................
    ;nIRQ timeout counter
    ;preload
    ;....................
    ldi r23,rfmRECoffMF
    sts rfmRECcntH,r23
   	;....................
	;receive OR transmit
	;rfm-data 
	;....................
	lds	    r23,comFLAG
    sbr     r23,1<<RFMbusy	        ;spi with rfm busy - cleared by slave_init !
    sts     comFLAG,r23


	lds	r23,COMTXstate
	sbrs	r23,RFMbufFILL
	rjmp	nIRQ_SVRreceive		    ;=0 no tx-action <=> receive data
	sbrs	r23,RFMtxSTART
	rjmp	nIRQ_SVRcheckPAS		;no further action during tx-buffer fill...
	sbrs	r23,RFMtxRUN
	rjmp	nIRQ_SVRcheckPAS		;no further action during startsequence...
	sbrc	r23,RFMtxSTOP
	rjmp	nIRQ_SVRcheckPAS        ;_B		;perhaps one more nIRQ from rfm12 - means don't care about...


;.................................................................................................
;transmit data to
;rfm...
;.................................................................................................
    ;------------------
	;set led on
	cbi	portb,rfmNET
    ;------------------
	;***************************
	;A) check sent frame-size
	;***************************
	lds		r23,TRANSbyte
	lds		r17,FRAME_siz
	cp		r23,r17
	brsh    nIRQ_SVRtxREADY	    ;all frame-bytes are sent


    ;**************************
    ;MANCHESTER CODER
    ;**************************
    ;load data
	lds	yl,TXbufPOINTl
	lds	yh,TXbufPOINTh
    ld  r16,y               ;tx data in r16
    ;check state
	lds	r17,MANstate		;flag reg holen
	sbrs	r17,MANnibble	;1=zweites nibble
	rjmp	_serACTIV1man	;0=erstes nibble
	;++ zweites nibble ++
	andi	r16,0b11110000	;nibbles freigeben
	mov		zl,r16
	call	_encodeMAN	;encoden ! (ergebnis yl)
	mov		r16,yl		;higher nibble
	;flag lschen
	cbr		r17,1<<MANnibble
	sts		MANstate,r17

	;.......................................
    ;byte counter +1							
 	inc		r23
	sts		TRANSbyte,r23
	;adress pointer +1
    lds	yl,TXbufPOINTl
	lds	yh,TXbufPOINTh
    adiw  yh:yl,1
    sts	TXbufPOINTl,yl
	sts	TXbufPOINTh,yh
	rjmp	_serACTIVmanTX
    ;.......................................
	
    ;.......................................
	;first nibble
    ;.......................................
_serACTIV1man:
	swap	r16				;nibbles tauschen
	andi	r16,0b11110000	;getauschte nibbles freigeben
	mov		zl,r16
	call	_encodeMAN	;encoden ! (ergebnis yl)
	mov		r16,yl		;lower nibble
	;flag setzen
	sbr		r17,1<<MANnibble
	sts		MANstate,r17

	;***************************
	;I) write data to buffer & tx
	;***************************
_serACTIVmanTX:
	;lade MSB
	ldi	zh,high(_TXdata)		
	mov	zl,r16				;data from tx-buffer
	sts	toMSB,zh
	sts	toLSB,zl
	call rfm12_initSPI
	;set tx/rx-state =0
	lds	r23,TRANS_state
	cbr	r23,1<<nRFirqEV		;nIRQ Event
	sts	TRANS_state,r23
	rjmp	nIRQ_SVRbk

 


    ;dummy byte
nIRQ_SVRtxREADY:
	ldi		zh,high(_TXprae)
	ldi		zl,low(_TXprae)
	sts		toMSB,zh
	sts		toLSB,zl
	call	rfm12_initSPI		;instruction to rfm12
    ;stop
	lds	r23,COMTXstate
	sbr	r23,1<<RFMtxSTOP		;tx-stop instruction will be executed next time...
	sts	COMTXstate,r23
	;set tx/rx-state =0
	lds	r23,TRANS_state
	cbr	r23,1<<nRFirqEV		;nIRQ Event
	sts	TRANS_state,r23
	rjmp	nIRQ_SVRbk





;.................................................................................................
;passive nIRQ-detection A
;.................................................................................................
nIRQ_SVRcheckPAS:
	;goon transmitting...
	;set tx/rx-state =1
	lds	r23,TRANS_state
	sbr	r23,1<<nRFirqEV		;nIRQ Event
	sts	TRANS_state,r23
	rjmp	nIRQ_SVRbk


;.................................................................................................
;passive nIRQ-detection B
;.................................................................................................
nIRQ_SVRcheckPAS_B:
	;goon transmitting...
	;set tx/rx-state =0
	lds	r23,TRANS_state
	cbr	r23,1<<nRFirqEV		;nIRQ Event
	sts	TRANS_state,r23
	rjmp	nIRQ_SVRbk







;........................................................................................................................
;receive data from 
;rfm...
;a)first byte of a frame:   - start activ receiving (nRF_RXrun)=0 & (fifoNEW)=0
;                           - start passiv receiving (nRF_RXrun)=0 & (fifoNEW)=1
;b)further bytes in frame   - continue receiving (nRF_RXrun)=1 & (fifoNEW)=0
;                           - passiv receiving  (nRF_RXrun)=1 & (nRFov)=1 -> fifo will be resetted
;c)last byte received       - stop receiving (nRF_RXrun)=1 & (nRFov)=0 -> instructed by #timINT after <rfmRECoff> cycles
;                             also fifo will be resetted & spi goes to slave mode. (fifoNEW)=1 shows the <_rfmCOM> new data
;NOTE: rfm in receiver mode - but perhaps not in spi master mode
;.........................................................................................................................
nIRQ_SVRreceive:
    lds	r23,TRANS_state
    ;A) FIRST check spi system-activity
    sbic    pinb,_IFsel
    rjmp    nIRQ_SVRreceiveX          ;not activ - set spi to master
    ;what kind of communication is runnin'  
	sbrc r23,nRF_RXrun		            
    rjmp   nIRQ_SVRreceiveRUN          ;rfm's already receiving  
    ;spi with somethinh else busy
    ;but rfm has a byte to take !
    ;this byte is not valid !
	;reset fifo next time !
  	cbr	r23,1<<nRFirqEV		    ;nIRQ Event &data still in buffer
    sbr r23,1<<nRF_RXreq        ;reset next time NOTE: perhaps in slave mode !	 
  	sts	TRANS_state,r23
	rjmp	nIRQ_SVRbk




    ;...........................
    ;The first byte !
    ;bus free & new data in rfm
    ;...........................
nIRQ_SVRreceiveX:

    ;...........................
    ;set spi as a master device
    ;...........................
    call    _SPImasterRFMINIT
    ;...........................
    ;buffer enabled ?
    ;...........................
    lds		zl,fifoVAR
    sbrs		zl,fifoNEW
    rjmp    nIRQ_SVRreceiveXgo    
 
 
 
                    ;......................................
                    ;rfm-buffer not enabled
                    ;reset ...
                    ;......................................
                    ;A) perhaps there occured another fifo
                    ;   interrupt = clear it!
                    ;B) get state & fifo data
                    ;......................................
                    rjmp    nIRQ_SVRbkREADYres
                   ;--------------------------------------

   




    ;................................
    ;spi is already in master-mode
    ;& data will be taken from rfm
    ;................................
nIRQ_SVRreceiveXgo:
    ;yes !
    ldi zl,0xff
    sts rfmFRAMEsoll,zl
    ldi zl,1
    sts rfmFRAMEist,zl                  ;on byte is received
     
	clr	zl
	sts	rfQUAL,zl
    ;NEW 
    ;reload fifo START-pointer
    ;with actual IN-pointer
    lds zl,fifoIN
    sts fifoSTART,zl
    ;flags  
	sbr	r23,1<<nRF_RXrun	            ;activ receivermode
    cbr r23,1<<nRF_RXreq                ;no more data waiting...
 


 


;----------------------------------------------------------
;read buffer in order of reading status
;- data to buffer
;- update quality
;----------------------------------------------------------
    ;-----------------------
    ;buffer free or loading 
    ;-----------------------
nIRQ_SVRreceiveRUN:
   	cbr	r23,1<<nRFirqEV
	sts	TRANS_state,r23
    ;-----------------------
    ;update off-counter
    ;compare & execute
    ;by 220s interrupt !
    ;-----------------------
    ldi zl,rfmRECoff
    sts rfmRECcntL,zl 
    ;-----------------------
    ;get state & fifo data
    call    rfm12_rxSPIdata
    ;-----------------------
  	lds	r5,fromLSB              ;contains fifo-data
 	lds	zl,stateMSB             ;contains MSBstate (ffov...)





                    ;-------------------------------------
                    ;check for fifo overflow
                    ;-------------------------------------
                    sbrs    zl,ffov
                    rjmp    nIRQ_SVRreceiveYgoon
                    ;--------------------------------------
                    ;data is invalid...
                    ;......................................
                    ;A) perhaps there occured another fifo
                    ;interrupt = clear it!
                    ;B) get state & fifo data
                    ;......................................
                    rjmp    nIRQ_SVRbkREADYres
                    ;--------------------------------------






;..........................................................
;MANCHESTER DECODER
;..........................................................
nIRQ_SVRreceiveYgoon:
            ;**************************************************************
			;MANCHESTER DECODER
	        ;-> start MANnibble=0
    		;**************************************************************
			lds	r16,MANstate		
			;flag reg holen
			sbrc	r16,MANnibble	;0=erstes nibble
			rjmp	_serACTIVrx1man	;1=zweites nibble
			;then decode lower nibble
			mov	yl,r5
			call	_decodeMAN	;ergebnis (yl) decodieren
			swap	zl
			;ergebnis temporr speichern
			sts		RECmanTE,zl
			;flag f. zweites nibble setzen
			sbr		r16,1<<MANnibble
			sts		MANstate,r16
			
            ;.....................................
            ;...don't store the first nibble
            rjmp    nIRQ_SVRbk
			;..................................... 

            ;.....................................
			;then decode higher nibble
            ;.....................................
	_serACTIVrx1man:
			mov		yl,r5			;higher nibble
			call	_decodeMAN	;ergebnis (yl) decodieren
			lds		r5,RECmanTE
			or		r5,zl
			;flag f. erstes  nibble setzen
			cbr		r16,1<<MANnibble
			sts		MANstate,r16

    ;-------------------------
    ;check quality
    ;-------------------------
	lds	zl,stateLSB             ;contains LSBstate (quality ....)
	;check quality
	sbrs	zl,dqd
	rjmp	nIRQ_SVRreceiveYgoQUAL
    ;good quality !
	lds	zl,rfQUAL
	inc	zl                      ;update quality
	sts	rfQUAL,zl

    ;goon
nIRQ_SVRreceiveYgoQUAL:
    ;FIFO-buffer adressing
	lds		zl,fifoIN
	mov		r23,zl
	clr		zh			
	subi		zl,low(-fifoSEGrx)
	sbci		zh,high(-fifoSEGrx)
	st		z,r5		    ;store received data to buffer
	inc		r23
	sts		fifoIN,r23	    ;store new counter value <=> points to the next empty memory-space






    ;----------------------------------------------------------------------------------------------
    ;check & calculate framesize
    ;----------------------------------------------------------------------------------------------
    lds zl,rfmFRAMEist
    cpi zl,1
    brne  nIRQ_SVRreceiveGOa
    ;actual databyte is instruction
    sts rfmFRAMEteins,r5          ;received instruction
    inc zl                        ;framesize ist +1
    sts rfmFRAMEist,zl          
    rjmp    nIRQ_SVRbk            ;back...

    ;-----------------------------
    ;FIRST CHECK
    ;IST =2 ? // 
    ;-----------------------------
;far jump
nIRQ_SVRreceiveGObX:
    jmp nIRQ_SVRreceiveGOb

nIRQ_SVRreceiveGOa:
      cpi zl,2
      brne  nIRQ_SVRreceiveGObX
      lds r23,rfmFRAMEteins

    ;-------------------------------------  
    ;check for scan
    cpi r23,1           ;== 4 bytes fix
    brne nIRQ_SVRreceiveA
    ldi zh,4
    sts rfmFRAMEsoll,zh
    jmp nIRQ_SVRreceiveCOMPARE

    ;-------------------------------------
    ;check for alive
nIRQ_SVRreceiveA:
    cpi r23,7            ;== 6 bytes fix + (2*direct) INFO: count of direct are stored on byte 2
    brne nIRQ_SVRreceiveB
    mov zl,r5           ;actual byte contains count of direct
    ldi zh,2
    mul zl,zh
    ldi zh,6
    add zh,r0
    sts rfmFRAMEsoll,zh
    jmp nIRQ_SVRreceiveCOMPARE

    ;------------------------------------
    ;check for kill
nIRQ_SVRreceiveB:
    cpi r23,8            ;== 6 bytes fix + (2*direct) INFO: count of direct are stored on byte 2
    brne nIRQ_SVRreceiveC
    mov zl,r5           ;actual byte contains count of direct
    ldi zh,2
    mul zl,zh
    ldi zh,6
    add zh,r0
    sts rfmFRAMEsoll,zh
   jmp nIRQ_SVRreceiveCOMPARE

    ;------------------------------------
    ;check for scanned ack
nIRQ_SVRreceiveC:
    cpi r23,50           ;== 6 bytes fix + (2*meshed) INFO: count of meshed are stored on byte 2
    brne nIRQ_SVRreceiveD
    mov zl,r5           ;actual byte contains count of direct
    ldi zh,2
    mul zl,zh
    ldi zh,6
    add zh,r0
    sts rfmFRAMEsoll,zh
   jmp nIRQ_SVRreceiveCOMPARE

    ;-------------------------------------
    ;check for kill/alive
nIRQ_SVRreceiveD:
    cpi r23,51           ;== 4 bytes fix
    brne nIRQ_SVRreceiveE
    ldi zh,4
    sts rfmFRAMEsoll,zh
  jmp nIRQ_SVRreceiveCOMPARE

    ;-------------------------------------
    ;check for simple ack
nIRQ_SVRreceiveE:
    cpi r23,52           ;== 5 bytes fix
    brne nIRQ_SVRreceiveF
    ldi zh,5
    sts rfmFRAMEsoll,zh
  jmp nIRQ_SVRreceiveCOMPARE


    ;-------------------------------------
    ;check for simple ack
nIRQ_SVRreceiveF:
    cpi r23,53           ;== 5 bytes fix
    brne nIRQ_SVRreceiveG
    ldi zh,5
    sts rfmFRAMEsoll,zh
  jmp nIRQ_SVRreceiveCOMPARE

    ;-------------------------------------
    ;check for simple ack
nIRQ_SVRreceiveG:
    cpi r23,54           ;== 5 bytes fix
    brne nIRQ_SVRreceiveCOMPARE
    ldi zh,5
    sts rfmFRAMEsoll,zh
  jmp nIRQ_SVRreceiveCOMPARE











    ;--------------------------------
    ;SECOND CHECK
    ;IST =9 ?
    ;--------------------------------
nIRQ_SVRreceiveGOb:
     cpi zl,9
      brne  nIRQ_SVRreceiveGOc
      lds r23,rfmFRAMEteins


    ;check for emb.  to direct
    cpi r23,2           ; == 4 bytes + emb frame INFO: emb. framesize ist stored on byte 9
    brne nIRQ_SVRreceiveAA
    ldi zh,4
    add zh,r5
    sts rfmFRAMEsoll,zh
   ;------------------
   jmp nIRQ_SVRreceiveCOMPARE

  ;check for emb. as broadcast
nIRQ_SVRreceiveAA:
    cpi r23,6           ; == 4 bytes + emb frame INFO: emb. framesize ist stored on byte 9
    brne nIRQ_SVRreceiveCOMPARE
     ldi zh,4
    add zh,r5
    sts rfmFRAMEsoll,zh
   jmp nIRQ_SVRreceiveCOMPARE

 



    ;--------------------------------
    ;THIRD CHECK
    ;IST =10 ?
    ;--------------------------------
nIRQ_SVRreceiveGOc:
     cpi zl,10
      brne  nIRQ_SVRreceiveCOMPARE
      lds r23,rfmFRAMEteins  


    ;check for emb. to mesh via direct
    cpi r23,4           ; == 5 bytes + emb frame INFO: emb. framesize ist stored on byte 10
    brne nIRQ_SVRreceiveAAA
     ldi zh,5
    add zh,r5
    sts rfmFRAMEsoll,zh
   jmp nIRQ_SVRreceiveCOMPARE

   ;check for emb. to mesh 
nIRQ_SVRreceiveAAA:
    cpi r23,5           ; == 5 bytes + emb frame INFO: emb. framesize ist stored on byte 10
    brne nIRQ_SVRreceiveCOMPARE
     ldi zh,5
    add zh,r5
    sts rfmFRAMEsoll,zh
   jmp nIRQ_SVRreceiveCOMPARE

 

;-------------------------------
;COMPARE...
;-------------------------------
nIRQ_SVRreceiveCOMPARE:
     lds zh,rfmFRAMEsoll
    lds zl,rfmFRAMEist
    cp  zl,zh
    brsh    nIRQ_SVRbkREADY
    inc zl
    sts rfmFRAMEist,zl
    rjmp  nIRQ_SVRbk  

nIRQ_SVRbkREADY:
    lds		r23,fifoVAR
	sbr		r23,1<<fifoNEW
;    cbr     r23,1<<fifoTIOUT	    ;clear timeout // wait for requested data OK!
	sts		fifoVAR,r23



    lds zl,fifoSTART                ;points to the first stored byte
    ;store to output start-point
    sts fifoOUT,zl
    ;fifoIN-fifoSTART=fifoSEG
    ;now calculate segment size
    lds r23,fifoIN 
    sub r23,zl
    brcs    nIRQ_SVRnegRES          ;fifoSTART > than fifoIN
    ;range OK
    sts fifoSEG,r23                 ;== real size
    rjmp  nIRQ_SVRbkREADYres
    
     ;range NOT OK
nIRQ_SVRnegRES:
    ldi zh,0xff
    sub zh,zl
    lds r23,fifoIN
    add r23,zh
    inc r23                         ;+1 == real size
    sts fifoSEG,r23




nIRQ_SVRbkREADYres:
    ;.......................................
    ;NEW: disable receiver till
    ;data is taken by rfmRXlayer !
    ;THEN: decide if answer by rfmTXlayer
    ;      or further enable the receiver!
    ;.......................................
    lds r23,TRANS_state
   	cbr	r23,1<<nRFirqEV
	cbr	r23,1<<nRF_RXrun		;receive loop off
    cbr r23,1<<nRF_RXreq      
 	sts	TRANS_state,r23
    ;......................................
    ;A) perhaps there occured another fifo
    ;interrupt = clear it!
    ;B) get state & fifo data
    ;......................................
    call    rfm12_FIFOresdata
    sbi     eifr,intf0
    ;& spi slave...
    call    _SPIslaveINITb
  






	;..................
	;RETURN
	;..................
nIRQ_SVRbk:
	out	sreg,r4
	;restore regs
    pop yh
    pop yl
    pop zh
    pop zl
    pop r23
    pop r17
    pop r16
    pop r5
    pop r4
    pop r1
    pop r0
	reti




