
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;rfm12 transceiver control
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;NOTE:
;The following instruction are available:
;
;1) Prepare for data-tranmission to the wireless network
;   => <rfm12_TXinit>
;
;2) Complete data-tranmission to the wireless network,
;   disable transmitter unit and enable the receiver
;   => <rfm12_TXoff>
;
;3) Prepare data-reception from the wireless network,
;   => <rfm12_RXinit>

;NEW: Also frequency hopping now supported - set channel 0...20 !

;*****************************************************************
;2) Basic TX-Init Commands to rfm12 
;SPI not busy already checked !
;*****************************************************************
rfm12_TXinit:
    ;clear transceiver on flag
    lds yl,TRANS_state
    cbr yl,1<<nRF_RXreq          ;kill receive next time...
    sts TRANS_state,yl
    call    _SPImasterRFMINITb          ;master & block bus
 
	clr	yl
	clr	yh
	;start
rfm12_TXinitLOOP:
	rcall	rfm12_TXtableLD	;get instr. constant
	sts		toMSB,zh
	sts		toLSB,zl
	rcall	rfm12_initSPI		;instruction to rfm12
	inc		yl
	cpi		yl,4                 ;3
	brsh		rfm12_TXinitLOOPI		
	rjmp		rfm12_TXinitLOOP
rfm12_TXinitLOOPI:
	;now load channel !
	rcall	rfm12_TXCHtableLD	;get instr. constant
	sts		toMSB,zh
	sts		toLSB,zl
	rcall	rfm12_initSPI		;instruction to rfm12
	inc		yl
	;>8 
	;now enable tx-section !
	rcall	rfm12_TXtableLD	;get instr. constant
	sts		toMSB,zh
	sts		toLSB,zl
	rcall	rfm12_initSPI		;instruction to rfm12
	;now wait for nIRQ
	rcall	rfm12_WT		;wait tx-buffer ready



	;load further 0xaa
	ldi		zh,high(_TXprae)
	ldi		zl,low(_TXprae)
	sts		toMSB,zh
	sts		toLSB,zl
	rcall	rfm12_initSPI		;instruction to rfm12
	;now wait for nIRQ
	rcall	rfm12_WT		;wait tx-buffer ready



	;send fifo sync-byte 0x2d
	ldi		zh,high(_FIFOsynca)
	ldi		zl,low(_FIFOsynca)
	sts		toMSB,zh
	sts		toLSB,zl
	rcall	rfm12_initSPI		;instruction to rfm12
	;now wait for nIRQ
	rcall	rfm12_WT		;wait tx-buffer ready

	;send fifo sync-byte 0xd4
	ldi		zh,high(_FIFOsyncb)
	ldi		zl,low(_FIFOsyncb)
	sts		toMSB,zh
	sts		toLSB,zl
	rcall	rfm12_initSPI		;instruction to rfm12
	;now wait for nIRQ
	rcall	rfm12_WT		;wait tx-buffer ready
	ret
	;-------------------------------
	;NOTE: 
	;transmit further data-bytes (0xb8xx)
	;after transmission (don't forget
	;the dummy-byte !) disable tx
	;by 0x8209 !!!	
	;-------------------------------
 


;+++++++++++++++++++++++++++
;table load sub
;+++++++++++++++++++++++++++
rfm12_TXtableLD:
 	ldi	zh,high(_TXini_tblII)
 	ldi	zl,low(_TXini_tblII)
 	add	zl,yl ;+pointer
 	adc	zh,yh 
 	lsl	zh
 	lsl	zl	;
 	brcc	rfm12_TXinitnoCARRY
 	sbr	zh,1<<0 ;bit0 msb high
rfm12_TXinitnoCARRY:
 	lpm
 	mov	r2,r0  ; lade parameter-adresse-low 8-bit nibble 
 	sbr	zl,1<<0 ;
 	lpm	
 	mov	zh,r0  ; lade parameter-adresse-high 8-bit nibble
	mov	zl,r2
	ret


;+++++++++++++++++++++++++++
;channel load sub
;+++++++++++++++++++++++++++
rfm12_TXCHtableLD:
	clr	xh
	lds	xl,TXch				;load channel
 	ldi	zh,high(_FREQUhoptbl)
 	ldi	zl,low(_FREQUhoptbl)
 	add	zl,xl ;+pointer
 	adc	zh,xh 
 	lsl	zh
 	lsl	zl	;
 	brcc	rfm12_TXCHinitnoCARRY
 	sbr	zh,1<<0 ;bit0 msb high
rfm12_TXCHinitnoCARRY:
 	lpm
 	mov	r2,r0  ; lade parameter-adresse-low 8-bit nibble 
 	sbr	zl,1<<0 ;
 	lpm	
 	mov	zh,r0  ; lade parameter-adresse-high 8-bit nibble
	mov	zl,r2
	ret







  






;********************************************************************
;3) Basic RX-Init Commands to rfm12 
;********************************************************************
rfm12_RXinit:
	;test
	lds	yl,TRANS_state
	cbr	yl,1<<nRFirqEV
    cbr yl,1<<nRF_RXreq          ;
 	sts	TRANS_state,yl
    ;
	clr	yl
	clr	yh
	;start
rfm12_RXinitLOOP:
	rcall	rfm12_RXtableLD	;get instr. constant
	sts		toMSB,zh
	sts		toLSB,zl
	rcall	rfm12_initSPI		;instruction to rfm12
	inc		yl
	cpi		yl,2			;2 max instr. 
	breq		rfm12_RXchannelLOOP
	cpi		yl,9            ;5
	brlo		rfm12_RXinitLOOP
	ret



	;load channel
rfm12_RXchannelLOOP:
	rcall	rfm12_RXCHtableLD
	sts		toMSB,zh
	sts		toLSB,zl
	rcall	rfm12_initSPI		;instruction to rfm12
	inc		yl
	rjmp		rfm12_RXinitLOOP

	;--------------------------------------------
	;NOTE: note for nIRQ after return
	;valid fifo data now can be read by 0xb000 or
	;in order of reading status 0x0000!
	;after reading fifo send a 0xca81 (reset fifo)
	;followed by a 0xca83 (enable fifo)
	;--------------------------------------------


;+++++++++++++++++++++++++++++
;table load sub
;+++++++++++++++++++++++++++++
rfm12_RXtableLD:
 	ldi	zh,high(_RXini_tblII)
 	ldi	zl,low(_RXini_tblII)
 	add	zl,yl ;+pointer
 	adc	zh,yh 
 	lsl	zh
 	lsl	zl	;
 	brcc	rfm12_RXinitnoCARRY
 	sbr	zh,1<<0 ;bit0 msb high
rfm12_RXinitnoCARRY:
 	lpm
 	mov	r2,r0  ; lade parameter-adresse-low 8-bit nibble 
 	sbr	zl,1<<0 ;
 	lpm	
 	mov	zh,r0  ; lade parameter-adresse-high 8-bit nibble
	mov	zl,r2
	ret


;+++++++++++++++++++++++++++
;channel load sub
;+++++++++++++++++++++++++++
rfm12_RXCHtableLD:
	clr	xh
	lds	xl,RXch				;load channel
 	ldi	zh,high(_FREQUhoptbl)
 	ldi	zl,low(_FREQUhoptbl)
 	add	zl,xl ;+pointer
 	adc	zh,xh 
 	lsl	zh
 	lsl	zl	;
 	brcc	rfm12_RXCHinitnoCARRY
 	sbr	zh,1<<0 ;bit0 msb high
rfm12_RXCHinitnoCARRY:
 	lpm
 	mov	r2,r0  ; lade parameter-adresse-low 8-bit nibble 
 	sbr	zl,1<<0 ;
 	lpm	
 	mov	zh,r0  ; lade parameter-adresse-high 8-bit nibble
	mov	zl,r2
	ret




;*******************************************************************
;4) RX-Command
;Receiver off
;*******************************************************************
rfm12_RXoff:
    ;load instruction
	ldi		zh,high(_RECEIVEoffC)
	ldi		zl,low(_RECEIVEoffC)
	sts		toMSB,zh
	sts		toLSB,zl
	rcall	rfm12_initSPI		;instruction to rfm12
   ;load instruction
	ldi		zh,high(_RECEIVEoffM)
	ldi		zl,low(_RECEIVEoffM)
	sts		toMSB,zh
	sts		toLSB,zl
	rcall	rfm12_initSPI		;instruction to rfm12
    ret





;********************************************************************
;5) TX-Ready followed by receiver init Command to rfm12 
;********************************************************************
rfm12_TXoff:
	rcall	rfm12_WT	;wait tx-buffer ready
	ldi		zh,high(TXMANAGEMENT)
	ldi		zl,low(TXMANAGEMENT)            ;8219
	sts		toMSB,zh
	sts		toLSB,zl
	rcall	rfm12_initSPI		;instruction to rfm12
	;enable receive-mode
	rcall	rfm12_RXinit
    ;now enable spi slave mode
    ;perhaps system communication
    ;will goon..
    call    _SPIslaveINIT   ;including _ifSEL=input
    ;timer on...
	lds	zl,timerCONTR
	sbr	zl,1<<TIMrun		;starts the timer
	sts	timerCONTR,zl
    ;now reset data request-timeout
	lds		zl,fifoVAR
    cbr     zl,1<<fifoTIOUT	    ;clear timeout // wait for requested data OK!
	sts		fifoVAR,zl
  


    ;check remote
    lds zl,REMif
    sbrs    zl,REMon        
    rjmp   rfm12_TXoffnorem 
    ;long time
    ;default... for min 15ms
    ldi zl,low(REQtiOUTkREM)    ;310*220s
    sts REQtiOUTl,zl
    ldi zl,high(REQtiOUTkREM)    ;310*220s
    sts REQtiOUTh,zl
	ret
	;-------------------------------
	;NOTE:
	;now the transceiver is ready 
	;to receive the next frame
	;-------------------------------


rfm12_TXoffnorem:
    ;default... for min 15ms
    ldi zl,low(REQtiOUTk)    ;310*220s
    sts REQtiOUTl,zl
    ldi zl,high(REQtiOUTk)    ;310*220s
    sts REQtiOUTh,zl
	ret
	;-------------------------------
	;NOTE:
	;now the transceiver is ready 
	;to receive the next frame
	;-------------------------------




;********************************************************************
;6) TX-Ready Command to rfm12 // spi still master mode
;********************************************************************
rfm12_TXoffII:
	rcall	rfm12_WT	;wait tx-buffer ready
	;load instruction
	ldi		zh,high(TXMANAGEMENT)
	ldi		zl,low(TXMANAGEMENT)            ;8219
	sts		toMSB,zh
	sts		toLSB,zl
	rcall	rfm12_initSPI		;instruction to rfm12
	;enable receive-mode
    ret





;********************************************************************
;5) TX-Ready followed by receiver init Command to rfm12 
;********************************************************************
rfm12_TXoffIII:
	;now wait for nIRQ
	rcall	rfm12_WT	;wait tx-buffer ready
	;load instruction
	ldi		zh,high(TXMANAGEMENT)
	ldi		zl,low(TXMANAGEMENT)            ;8219
	sts		toMSB,zh
	sts		toLSB,zl
	rcall	rfm12_initSPI		;instruction to rfm12
    ;timer on...
	lds	zl,timerCONTR
	sbr	zl,1<<TIMrun		;starts the timer
	sts	timerCONTR,zl
    ;now disable data request-timeout
	lds		zl,fifoVAR
    cbr     zl,1<<fifoTIOUT	    ;clear timeout // wait for requested data OK!
	sts		fifoVAR,zl



    ;check remote
    lds zl,REMif
    sbrs    zl,REMon        
    rjmp   rfm12_TXoffIIInorem 
    ;long time
    ;default... for min 15ms
    ldi zl,low(REQtiOUTkREM)    ;310*220s
    sts REQtiOUTl,zl
    ldi zl,high(REQtiOUTkREM)    ;310*220s
    sts REQtiOUTh,zl
	ret
	;-------------------------------
	;NOTE:
	;now the transceiver is ready 
	;to receive the next frame
	;-------------------------------


rfm12_TXoffIIInorem:
    ;default... for min 15ms
    ldi zl,low(REQtiOUTk)    ;310*220s
    sts REQtiOUTl,zl
    ldi zl,high(REQtiOUTk)    ;310*220s
    sts REQtiOUTh,zl
	ret
	;-------------------------------
	;NOTE:
	;now the transceiver is ready 
	;to receive the next frame
	;-------------------------------



























;..............................................................
;rfm12 Communication
;..............................................................

;----------------------------------------------------------
;read rfm // needs r16,zl,zh
;----------------------------------------------------------
rfm12_rxSPIdata:
	;lade _READstate
	ldi	zh,high(_READstate)	
	ldi	zl,low(_READstate)
	sts	toMSB,zh
	sts	toLSB,zl
	call rfm12_initSPI
	lds	zh,fromMSB
	lds	zl,fromLSB
    sts stateMSB,zh
    sts stateLSB,zl

	ldi	zh,high(_READfifo)	
	ldi	zl,low(_READfifo)
	sts	toMSB,zh
	sts	toLSB,zl
	call rfm12_initSPI
	lds	zh,fromMSB
	lds	zl,fromLSB
	ret

;----------------------------------------------------------
;fifo reset // needs r16,zl,zh
;----------------------------------------------------------
rfm12_FIFOresdata:
	;lade _READstate
	ldi	zh,high(_READstate)	
	ldi	zl,low(_READstate)
	sts	toMSB,zh
	sts	toLSB,zl
	call rfm12_initSPI
	lds	zh,fromMSB
	lds	zl,fromLSB
rfm12_FIFOresdatab:
    ;fifo reset
	ldi	zh,high(_FIFOres)	
	ldi	zl,low(_FIFOres)
	sts	toMSB,zh
	sts	toLSB,zl
	call rfm12_initSPI
	lds	zh,fromMSB
	lds	zl,fromLSB
    ;fifo enabled
	ldi	zh,high(_FIFOena)	
	ldi	zl,low(_FIFOena)
	sts	toMSB,zh
	sts	toLSB,zl
	call rfm12_initSPI
	lds	zh,fromMSB
	lds	zl,fromLSB
	ret






;-----------------------------------------------------------------------------------
;calculate received checksum 
;- data in rfm_buffer
;- return calculated checksum in rxCKsumBUILT
;-----------------------------------------------------------------------------------
rfm12_rxCHKsum:
	lds	zl,FRAME_siz
	subi	zl,1
	clr	zh
	subi	zl,low(-rfm_buffer)
	sbci	zh,high(-rfm_buffer)
	ld	xl,z				;get received checksum
	sts	rxCKsumREC,xl
	ldi	xh,high(rfm_buffer)
	ldi	xl,low(rfm_buffer)
	ldi	zh,0xaa
	clr	r17
	;now calculate & compare
rfm12_rxCHKsumLOOP:
	lds	r16,FRAME_siz
	subi	r16,1
	cp	r17,r16
	brsh	rfm12_rxCHKsumOK	;checksum built
	ld	zl,x+
	eor	zh,zl
	sts	rxCKsumBUILT,zh
	inc	r17
	rjmp	rfm12_rxCHKsumLOOP
	;
rfm12_rxCHKsumOK:
	ret


;-------------------------------------------------------------------------------------
;build signal quality factor
;- uses <rfQUAL>&<FRAME_siz> 
;- return with the factor in rfm_buffer (+4)
;- n' the factor in <Qfactor>
;- clears the repeat counter <TXrepCNT>
;-------------------------------------------------------------------------------------
rfm12_Qfactor:
	lds	zl,rfQUAL					;quality counter <= FRAME_siz
	;(100*rfQUAL)
	ldi	r16,100
	mul	zl,r16					;result r1/r0
	mov	zl,r0
	mov	zh,r1
	;/FRAME_siz
	lds	r17,FRAME_siz				;FRAME_siz
	ldi	r16,10
rfm12_RX003scanningLOOPQUALa:
	mul	r17,r16					;*10/*20/*30/....
	;< (100*rfQUAL)?
	cp	r0,zl
	cpc	r1,zh
	brsh	rfm12_RX003scanningLOOPQUALb	;too big
	ldi	r18,10
	add	r16,r18					;+10 
	rjmp	rfm12_RX003scanningLOOPQUALa
	;
rfm12_RX003scanningLOOPQUALb:
	subi	r16,10
rfm12_RX003scanningLOOPQUALc:
	mul	r17,r16					;
	;< (100*rfQUAL)?
	cp	r0,zl
	cpc	r1,zh
	brsh	rfm12_RX003scanningLOOPQUALd	;ready
	ldi	r18,1
	add	r16,r18					;+1
	rjmp	rfm12_RX003scanningLOOPQUALc
rfm12_RX003scanningLOOPQUALd:
	subi	r16,1
	ldi	xl,4
	clr	xh
	subi	xl,low(-rfm_buffer)
	sbci	xh,high(-rfm_buffer)
	st	x,r16					;store quality factor
	sts	Qfactor,r16
	;clear repeat counter !
	clr	r16
	sts	TXrepCNT,r16
	ret










;**********************************************************************************
;Manchester Encoder for wireless conceptXM1 link
;**********************************************************************************
;type: subroutine <call/ret>
;-> byte to encode:  zl
;-> loop cnt: zh
;-> encoded (8bit): yl
;cycles: .....
;**********************************************************************************

	;*************************************
	;start conditions - call entry
	;*************************************
_encodeMAN:
	clr	yl		;16bit encoded LOW
	ldi	zh,4	;8	;loop cnt


	;*************************************
	;encoder loop
	;*************************************
_encodeMANloop:
	;loop fertig ?
	tst	zh
	breq	_encodeMANloopBK
	dec	zh
	rol	zl			;rotate byte left (carry to bit 0 / bit7 to carry)
	brcc _HLedge	;bit7 =0
	rjmp _LHedge	;bit7 =1

	;*************************************
	;encoder loop ready - return f. call
	;*************************************
_encodeMANloopBK:
	ret




	;*********************************************************
	;carry im quellbyte =0  <=> High/Low-Edge im encoded byte 
	;*********************************************************
_HLedge:
	sec		
	;LSB nach links
	rol	yl
	;jetzt clock LOW
	clc
	;LSB nach links
	rol	yl
	rjmp	_encodeMANloop


	;*********************************************************
	;carry im quellbyte =1  <=> Low/High-Edge im encoded byte 
	;*********************************************************
_LHedge:
	;clock LOW
	clc		
	;LSB nach links
	rol	yl
	;jetzt clock HIGH
	sec		
	;LSB nach links
	rol	yl
	rjmp	_encodeMANloop














;**********************************************************************************
;Manchester Decoder for wireless MultiNET0.x link
;**********************************************************************************
;type: subroutine <call/ret>
;-> byte to decode:  zl
;-> loop cnt: zh
;-> coded (8bit): yl
;cycles: .....
;**********************************************************************************


	;*************************************
	;start conditions - call entry
	;*************************************
_decodeMAN:
	clr		zl		;8bit decoded 
	ldi		zh,4	;loop cnt



	;*************************************
	;decoder loop
	;*************************************
_decodeMANloop:
	;loop fertig ?
	tst		zh
	breq	_decodeMANloopBK
	dec		zh
	;start test edge?
;shift LSB right+carry
	lsr		yl
	brcs	_LHedgeD		;Low.....L/H edge?
	rjmp	_HLedgeD		;High....H/L edge?



	;****************************************************************
	;H/L edge?
	;****************************************************************
	;MSB right
_HLedgeD:
	;LSB right
	lsr		yl			;shift LSB right 
	brcs	_DECzero	;H/L edge erkannt !	
	;eigentlich ist das
	;ein fehlerhafter 
	;zustand !!!
	rjmp	_DECone

	;decoder byte aktualisieren
	;H/L edge == 0
_DECzero:
	;shift decoded byte right
	lsr	zl
	cbr	zl,1<<7

	rjmp	_decodeMANloop	
	




	;****************************************************************
	;L/H edge?
	;****************************************************************
	;MSB right
_LHedgeD:
	;LSB right
	lsr		yl			;shift LSB right 
	brcc	_DECone		;L/H edge erkannt !	
	;eigentlich ist das
	;ein fehlerhafter 
	;zustand !!!
	rjmp	_DECzero	
	;decoder byte aktualisieren
	;L/H edge == 1
_DECone:
	;shift decoded byte right
	lsr	zl
	sbr	zl,1<<7

	rjmp	_decodeMANloop	
	
		

	;*************************************
	;decoder loop ready - return f. call
	;*************************************
_decodeMANloopBK:
	ret





