;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;Dynamic transceiver mode
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++







;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;rfm12_000run - check for transceiver action (HIGHEST PRIORITY)
;
;STATEsend          = contains the transmit function
;STATEreq           = contains the requested receive function
;fifoVAR(fifoTIOUT) = will be set if no requested data is received (after timeout)
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
rfm12_000run: 
    ;check transceiver state...
    lds r16,hifGLOB
    sbrs    r16,rfmENA
    rjmp    rfm12_003run
    ;------------------------------------------------
    ;WAITING FOR REQUESTED DATA ?
	lds	r16,STATEreq	
	tst	r16
	breq	rfm12_000runCKa     ;no requested answer !	
    ;YES ANSWER REQUESTD!
    lds	r16,fifoVAR
	sbrs	r16,fifoTIOUT	    ;OR timeout
	rjmp	rfm12_000runREC	    ;check for received frame...
	clr	r16
	sts	STATEreq,r16
 	rjmp	rfm12_000runBACK	;go back - next call checks for repeated transmitting 
rfm12_000runREC:
	call	rfm12_010run		;perhaps data in fifo
    rjmp	rfm12_000runBACK	;go back - next call checks for repeated transmitting 


    ;NO REQUESTED ANSWER
rfm12_000runCKa:
    ;.........................................................................
    ;TEST
    ;clear timer
	lds	zl,timerCONTR
	cbr	zl,1<<TIMrun		;disables the timer
	sts	timerCONTR,zl
    ;now reset data request-timeout
	lds		zl,fifoVAR
    cbr     zl,1<<fifoTIOUT	    ;clear timeout // wait for requested data OK!
	sts		fifoVAR,zl
    ;..........................................................................

    ;.. CHECK for new received frame
	call	rfm12_010run
    ;...CHECK for new transmit instruction 
	lds	r16,STATEsend		
	tst	r16
	breq	rfm12_003run		;no action == check for new NET DATA
    ;STATEsend >0
	call	rfm12_100run
    rjmp	rfm12_000runBACK	;go back - next call checks for repeated transmitting 

  












;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;rfm12_003run = check for actual data on COMnetSTACK
;COMTXstate(BUFsecure)  = disables/enables the stack access
;comNETstackSTA         = stack pointer
;RFMtxbuffer            = stacked data will be stored there
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

rfm12_NOvalidFRAMEx:
    jmp rfm12_NOvalidFRAME


rfm12_003run:
       ;first check if there's any valid frame on stack
        lds     r16,comNETstackCNT
        tst     r16
        breq    rfm12_NOvalidFRAMEx
        ;>0 then check if <COMTXstate> BUFsecure is set
        ;NOTE: if this flag is set the <RFMtxbuffer> content
        ;is already used for rfm-communication and should not 
        ;be edited...
        lds     r17,COMTXstate
        sbrc    r17,BUFsecure       ;SHOULD BE CLEARED BY DYNAMIC_TX !!!!
        rjmp    rfm12_NOvalidFRAME        ;there's a valid frame on stack - but the communication buffer is in secure mode
        ;
        lds     r17,comNETstackFL
        sbrc    r17,STACKrebuild
        rjmp    rfm12_NOvalidFRAME        ;there's a valid frame on stack - but the communication buffer is in secure mode

        ; temporary no interrupts allowed
        ;fill condition passed
        lds     r16,comNETstackCNT
        lds    zl,comNETstackSTA        ;related stack startpointer
        cp  zl,r16
        breq   rfm12_003runGOa          ;related = activ segments
        brlo   rfm12_003runGOa          ;related < activ segments
        ;related > activ segments
        clr zl    

rfm12_003runGOa:
        mov r16,zl
        inc r16
        sts comNETstackSTA,r16
         ;a) adress the stack
        ldi    zh,4         
        mul    zl,zh                    ;*4            
        mov    zl,r0
        clr    zh
        subi   zl,low(-comNETstack)
        sbci   zh,high(-comNETstack)
        ;load startadress
        ld     xh,z+
        ld     xl,z+
        ;load endadress
        ld     yh,z+
        ld     yl,z+
        ;clear stack segment
        sbiw   zh:zl,4
        clr     r16
        st  z+,r16
        st  z+,r16
        st  z+,r16
        st  z,r16
        
        
        ;b) copy frame to communication buffer
        ;startadress
        ldi zl,low(RFMtxbuffer)
        ldi zh,high(RFMtxbuffer)
       ;endadress < startadress = carry
        cp      yl,xl
        cpc     yh,xh
        breq    rfm12_NOvalidFRAME      ;equal not valid
        brsh    rfm12_VALIDframeCOPYb   ;endadress > startadress == without carry

   
    ;with buffer carry!
    rfm12_VALIDframeCOPYa:
                ld  r16,x
                st  z+,r16
                ;check range
                cpi xl,low(comNETbufferEND)
                ldi r17,high(comNETbufferEND)
                cpc xh,r17
                breq    rfm12_VALIDframeCOPYaX
                adiw    xh:xl,1
                rjmp    rfm12_VALIDframeCOPYa
                ;buffer carry detected !
                ;set to startadress
     rfm12_VALIDframeCOPYaX:  
                ldi     xl,low(comNETbuffer)
                ldi     xh,high(comNETbuffer)
                rjmp    rfm12_VALIDframeCOPYb
     
     
              


   ;without buffer carry !
   rfm12_VALIDframeCOPYb:
                ld  r16,x
                st  z+,r16
                ;check range
                cp  xl,yl
                cpc xh,yh
                breq    rfm12_VALIDframeCOPYready ; fraem complete copied !
                adiw    xh:xl,1
                rjmp    rfm12_VALIDframeCOPYb






   ;valid frame copied-action !
   rfm12_VALIDframeCOPYready:
        lds     r17,COMTXstate
        sbr     r17,1<<BUFsecure            ;SHOULD BE CLEARED BY DYNAMIC_TX !!!!
        sts     COMTXstate,r17
       ; sei
        rjmp	rfm12_003runRXcom		    ;yes - actually completed to segment 




;--------------------------------------------------------------------------------
;CHECK FOR RECEIVED RS485 DATA
;RS485state(RS485from)          =   set if data is received by uart(rs485)
;comNETstate(comNETnewDATtx)    =   set to transmit data to the system
;--------------------------------------------------------------------------------
rfm12_NOvalidFRAME:
    ;check state
    lds     r17,RS485state
    sbrs    r17,RS485from
    rjmp    rfm12_NOvalidFRAMEstate
    ;load data to buffer
    ldi     zl,5     
    clr     zh
    subi    zl,low(-rs485NETbuffer)
    sbci    zh,high(-rs485NETbuffer)
    ld      r10,z       ;load frame-size
    clr     r11
    sbiw    zh:zl,5
 	;set startadress
	ldi	xl,low(TX_buffer)
	ldi	xh,high(TX_buffer)
rfm12_NOvalidFRAMEloop:
    ld  r12,z+
    st  x+,r12
    inc r11
    cp  r11,r10
    brsh  rfm12_NOvalidFRAMEready
    rjmp   rfm12_NOvalidFRAMEloop
    ;ready loaded
rfm12_NOvalidFRAMEready:
    cbr     r17,1<<RS485from
    sts     RS485state,r17
    ;set system transfer flag
	lds	r17,comNETstate
	sbr	r17,1<<comNETnewDATtx
	sts	comNETstate,r17
	rjmp	rfm12_000runBACK			;back in loop...


    





;------------------------------------------
;CHECK IF ALIVE-PIN IS LOW
;------------------------------------------
rfm12_NOvalidFRAMEstate:
	sbis	pind,STalive
	rjmp	rfm12_003runALIVE
	rjmp	rfm12_000runBACKjp			;back in loop...	
	;alive
rfm12_003runALIVE:
	ldi	r16,7
	sts	STATEsend,r16
	;scan=1
	ldi	r16,1
	sts	HscanCNT,r16
	sts	Thost_adr,r16
	rjmp	rfm12_000runBACK			;back in loop...
    ;............................
	;no action - back
	;............................
rfm12_000runBACKjp:
	rjmp	rfm12_000runBACK			;back in loop...






;--------------------------------------------------------------------------------
;CHECK & DECODE RECEIVED SYSTEM DATA
;--------------------------------------------------------------------------------
	;..........................................
	;spi complete received data
	;NOTE: comNETBUF is used - now check data 
	;from system. Could be also data for a
	;meshed host !
	;data format <=> NEW serCOM-standard
	;(further info at conceptXM2 documentation)
	;..........................................
rfm12_003runRXcom:
    ldi zl,low(RFMtxbuffer)
    ldi zh,high(RFMtxbuffer)
   	ld	r16,z			;load serCOM mode instruction
	;0= invalid
	tst	r16
	brne	rfm12_003runRXcomSET	;>0
	;-------------------------------
	;INVALID!
	;-------------------------------
    jmp rfm12_000runERRORBACK




;........................................................................
;True?
;a) rfm12 receives the IFstate-setup instruction
;........................................................................

rfm12_003runRXcomSET:
	sts	serCOMmi,r16			;store to var
	sbrs	r16,if_setup		;1=setup
	rjmp	rfm12_003runRXcomA
    ;.............................
	;check type
    ;contains the IFstate-matrix !
    ;.............................
    adiw    zh:zl,6
	ld	r16,z			;load serCOM mode instruction
	;0=invalid
	tst	r16
	brne	rfm12_003runRXcomSETa
	;-------------------------------
	;INVALID!
	;-------------------------------
  jmp rfm12_000runERRORBACK



	


;...................................
;1=scan mode
;=> clears the whole tableSHAD!
;...................................
rfm12_003runRXcomSETa:	
	cpi	r16,1
	brne	rfm12_003runRXcomSETb
	sts	STATEsend,r16
	;sys msg
	call	_SYSmsgD
	call	_sys03IF
	;scan=1
	ldi	r16,1
	sts	HscanCNT,r16
	sts	Thost_adr,r16
    ;clear shadow host table
    clr r16
	;startadresses
	ldi	zl,low(host_tableSHAD)
	ldi	zh,high(host_tableSHAD)
	sts	HtablePOSl,zl
	sts	HtablePOSh,zh
rfm12_003runRXcomSETaLOOP:
    st  z,r16
   	;range ?
	cpi	zl,low(host_tableSHADrange)
	ldi	xh,high(host_tableSHADrange)
	cpc	zh,xh
	brsh	rfm12_003runRXcomSETaGO
    adiw    zh:zl,1
    rjmp    rfm12_003runRXcomSETaLOOP
rfm12_003runRXcomSETaGO:
	rjmp	rfm12_000runBACK			;back in loop...



        ;...................................
        ;2=set new host ident
        ;- store the new host ID 
        ;- followed by a lha change meassge 
        ;...................................
        rfm12_003runRXcomSETb:	
        	cpi	r16,2
        	brne	rfm12_003runRXcomSETc
            ;load OLDShost_adr
            lds r22,Shost_adr
            sts OLDShost_adr,r22
            ;load new host adress
            ldi zl,low(RFMtxbuffer)
            ldi zh,high(RFMtxbuffer)
            adiw    zh:zl,7
            ld  r22,z
            sts  Shost_adr,r22
	        ldi	zh,high(host_adress_e2p)
	        ldi	zl,low(host_adress_e2p)
	        call	_WRe2big
            ;transmit change
            ldi r16,9
            sts	STATEsend,r16
                        ;................
                        ;LHA change text
                        ;................
                        call    _SYSmsgJ
                    	call	_sys03IF
                        ;start host
	                    ldi	r16,1
                        sts	HscanCNT,r16
                        sts	Thost_adr,r16
                    	rjmp	rfm12_000runBACK			;back in loop...
        
                       






                        ;...................................
                        ;3=rf-transceiver off // rs485 off
                        ;...................................
                        rfm12_003runRXcomSETc:	
                        	cpi	r16,3
                        	brne	rfm12_003runRXcomSETd
                          	;---------------------
                        	;check spi activity
                        	;---------------------
                        rfm12_003runRXcomSETcLOOP:
                            sbis    pinb,_ifSEL
                        	rjmp	rfm12_003runRXcomSETcLOOP			;spi still busy with SPI
                            ;check 
                            lds r16,hifGLOB
                            sbrs    r16,rfmENA
                            rjmp    rfm12_003runRXcomSETcBK
                            cbr     r16,1<<rfmENA
                            sts     hifGLOB,r16                          
                            ;---------------------
                            ;disable Interrupts
                            ;---------------------
                            cli 
                            ;---------------------
                            ;free = spi master
                            ;---------------------
                            call    _SPImasterRFMINITb          ;master & block bus
                            ;---------------------
                            ;receiver off
                            ;---------------------
                            call    rfm12_RXoff
                            ;---------------------
                            ;spi slave
                            ;---------------------
                            call    _SPIslaveINITb
                            ;--------------------------
                            ;clear old receive request
                            ;--------------------------
                            clr   r16
                         	sts	TRANS_state,r16
                            ;---------------------------
                            ;clear int0 request
                            ;---------------------------
                            sbi     eifr,intf0 
                            ;---------------------------
                            ;-now receiver is off
                            ;-spi is slave
                            ;---------------------------
                             ;global on...
                            sei
                    	 rfm12_003runRXcomSETcBK:
                            lds r16,RS485state
                            sbrs    r16,RS485ena
                            rjmp   rfm12_003runRXcomSETccBK 
                            cbr r16,1<<RS485ena
                            sts RS485state,r16
                            ;uart off
                            clr r16
                            sts ucsr0b,r16
                          

                        
                          rfm12_003runRXcomSETccBK:
                            ;enable buffer
                            lds     r17,COMTXstate
                            cbr     r17,1<<BUFsecure           
                            sts     COMTXstate,r17                           
                            rjmp	rfm12_000runBACK			;back in loop...








                                            ;...................................
                                            ;4=rf-transceiver on
                                            ;...................................
                                         rfm12_003runRXcomSETd:	
                                            cpi	r16,4
                                            brne	rfm12_003runRXcomSETf
                                            ;---------------------
                                            ;check spi activity
                                            ;---------------------
                                         rfm12_003runRXcomSETdLOOP:
                                            sbis    pinb,_ifSEL
                                            rjmp	rfm12_003runRXcomSETdLOOP			;spi still busy with SPI
                                            ;check
                                            lds r16,hifGLOB
                                            sbrc    r16,rfmENA
                                            rjmp    rfm12_003runRXcomSETdBK
                                            sbr     r16,1<<rfmENA
                                            sts     hifGLOB,r16                                                                     
                                            ;---------------------
                                            ;disable Interrupts
                                            ;---------------------
                                            cli 
                                            ;---------------------
                                            ;free = spi master
                                            ;---------------------
                                            call    _SPImasterRFMINITb          ;master & block bus
                                            ;---------------------
                                            ;receiver on
                                            ;---------------------
                                            call    rfm12_RXinit
                                            ;---------------------
                                            ;spi slave
                                            ;---------------------
                                            call    _SPIslaveINITb
                                            sei
                                       rfm12_003runRXcomSETdBK:
                                            ;enable buffer
                                            lds     r17,COMTXstate
                                            cbr     r17,1<<BUFsecure           
                                            sts     COMTXstate,r17
                                            rjmp	rfm12_000runBACK			;back in loop...



;...................................
;6=soft-uart (rs485) on
;...................................
rfm12_003runRXcomSETf:	
	cpi	r16,6
	brne	rfm12_003runRXcomSETg
    ;enable
    lds r16,RS485state
    sbrc    r16,RS485ena
    rjmp   rfm12_003runRXcomSETfBK 
    sbr r16,1<<RS485ena
    sts RS485state,r16
    ;uart receive
    clr r16
    sbr r16,1<<rxcie0
    sbr r16,1<<rxen0
    sbr r16,1<<txen0
    sts ucsr0b,r16
    ;direction receive
    cbi portd,uartDIR
    ;
rfm12_003runRXcomSETfBK:
    ;enable buffer
    lds     r17,COMTXstate
    cbr     r17,1<<BUFsecure           
    sts     COMTXstate,r17

	rjmp	rfm12_000runBACK			;back in loop...




                    ;...................................
                    ;7=alive message
                    ;...................................
                    rfm12_003runRXcomSETg:	
                    	cpi	r16,7
                    	brne	rfm12_003runRXcomSETh
                        sts	STATEsend,r16
                        ;................
                        ;alive text
                        ;................
                        call    _SYSmsgG
                    	call	_sys03IF
                        ;start host
	                    ldi	r16,1
                        sts	HscanCNT,r16
                        sts	Thost_adr,r16
                    	rjmp	rfm12_000runBACK			;back in loop...



                                    ;...................................
                                    ;8=kill message
                                    ;...................................
                                    rfm12_003runRXcomSETh:	
                                    	cpi	r16,8
                                    	brne	rfm12_003runRXcomSETj
                                    	sts	STATEsend,r16
                                        ;.................
                                        ;kill text
                                        ;.................
                                        call    _SYSmsgH
                                    	call	_sys03IF
                                        ;
                                    	;start host
                                    	ldi	r16,1
                                    	sts	HscanCNT,r16
                                    	sts	Thost_adr,r16
                                    	rjmp	rfm12_000runBACK			;back in loop...

          
;...................................
;10= master-remote on message
;...................................
;jmp
rfm12_003runRXcomSETkX:
    jmp rfm12_003runRXcomSETk
        ;
rfm12_003runRXcomSETj:	
	cpi	r16,10
	brne	rfm12_003runRXcomSETkX
    ;first check if the slave adress
    ;is valid (=in host_table)
    ldi zl,low(RFMtxbuffer)
    ldi zh,high(RFMtxbuffer)
    adiw    zh:zl,1
    ld  r16,z               ;load adress
	sts	Thost_adr,r16
    ;save slave
    sts REMslave,r16
	;...............................
	;now check if it's a direct
	;OR a meshed host...
	;shadow ram !
	call	scan_160HOtable
	;...............................
	lds	r16,DIRhost	
	tst	r16
	breq	rfm12_003runRXcomSETjFAIL   ;no direct host with this adress found
    ;save as a master
    lds r16,Shost_adr
    sts REMmaster,r16
    ;adress ok!
    ;set local remote flags
    clr r16
    sbr r16,1<<REMon        ;remote on
    sbr r16,1<<REMstate     ;master
    sts REMif,r16
    ;transmit(10)
    ldi r16,10
	sts	STATEsend,r16
   ;type
    ldi r16,REMinstr
    sts REMtype,r16                         ;instruction without data
    ;size
    ldi r16,8
    sts REMsize,r16
    ;wait in order for display data
	lds	r16,comFLAG
	cbr	r16,1<<spiBUSYhif	                ;1=spi hif-data ready transmitted 
	cbr	r16,1<<newHIFtx
    sbr r16,1<<waitHIFdata                  ;-> wait for concept-data... cleared after interpreting !
	sts	comFLAG,r16
    rjmp	rfm12_000runBACK			    ;back in loop...
    ;
    ;adress NOT ok!
rfm12_003runRXcomSETjFAIL:
    sbis    pinb,_ifSEL
    rjmp	rfm12_003runRXcomSETjFAIL			;spi still busy with SPI
    ;free
    call    _SPImasterINIT
    ;set transfer
    cbi	portb,_SS   
    ;clear remote flags
    nop
    nop
    nop
    clr r16
    sts REMif,r16
	sts	Thost_adr,r16
    sts REMmaster,r16
  
    ;save slave
    sts REMslave,r16
    ;A)tx ident
    ldi xl,0xd4
	call	_sndPARAspiDO
    ;B)mi: setup/to core
    ldi xl,0b01001000          
  	call	_sndPARAspiDO
    ;C)4xdummy
    ldi xl,0xaa
 	call	_sndPARAspiDO
  	call	_sndPARAspiDO
  	call	_sndPARAspiDO 
   	call	_sndPARAspiDO
    ;D)size=7
    ldi xl,7                  
  	call	_sndPARAspiDO
    ;E)type=11 (off)
    ldi xl,11
 	call	_sndPARAspiDO
    nop
    nop
    nop
    ;slave mode
    call    _SPIslaveINIT
    ;enable buffer
    lds     r17,COMTXstate
    cbr     r17,1<<BUFsecure           
    sts     COMTXstate,r17
 

    rjmp	rfm12_000runBACK			;back in loop...
  




;...................................
;11= from slave-remote off message
;...................................
rfm12_003runRXcomSETk:
	cpi	r16,11
	brne	rfm12_003runRXcomSETl
    ;transmit(10)
    ldi r16,10
	sts	STATEsend,r16
    ;type
    ldi r16,REMinstr
    sts REMtype,r16                         ;instruction without data
    ;size
    ldi r16,8
    sts REMsize,r16
    rjmp	rfm12_000runBACK			;back in loop...




;...................................
;invalid
;...................................
rfm12_003runRXcomSETl:
  jmp rfm12_000runERRORBACK







;........................................................................
;NO SETUP INSTRUCTION !
;
;rfm12 receives broadcast instruction
;or a direkt Transfer instruction?
;from the master system
;........................................................................
rfm12_003runRXcomA:
    ;check if rs485 is enabled
    lds     r16,RS485state
    sbrs    r16,RS485ena
    rjmp    rfm12_003runRXcomAX
    ;enabled
    ;build frame...
    ldi     zl,low(RFMtxbuffer)
    ldi     zh,high(RFMtxbuffer)
    ldi     r17,0xff
    clr     r18
    ldi     xl,0xaa
rfm12_003runRXcomAck:
    sbrc	r16,RS485wait           ;will be reset in timINT !
    rjmp    rfm12_003runRXcomAck
    ;direction transmit
    sbi     portd,uartDIR
    ;transmit to rs 485
rfm12_003runRXcomAloop:
    cpi     r18,5
    breq    rfm12_003runRXcomAloopFRAMESIZ
    ld      r2,z+  
    eor     xl,r2
    sts     udr0,r2   ;transmit
    rjmp    rfm12_003runRXcomAwt
    ;new frame size
rfm12_003runRXcomAloopFRAMESIZ:
    ld      r2,z+  
    inc     r2        ;+1
    eor     xl,r2
    sts     udr0,r2   ;transmit
    ;
rfm12_003runRXcomAwt:
    ;check state
    lds     r16,ucsr0a
    sbrs    r16,udre0
    rjmp    rfm12_003runRXcomAwt
    ;cnt+1
    inc     r18
    cp  r18,r17
    brne   rfm12_003runRXcomAloop 
    ;ready
    ;transmit ck sum...
    sts     udr0,xl   ;transmit
rfm12_003runRXcomAwtB:
    ;check state complete shifted
    lds     r16,ucsr0a
    sbrs    r16,txc0
    rjmp    rfm12_003runRXcomAwtB
    ;direction receive
    cbi     portd,uartDIR
    rjmp    rfm12_003runRXcomAX






    ;--------------------------------
    ;RFM enabled ?
    ;-------------------------------- 
rfm12_003runRXcomAX:
  ;check transceiver state...
    lds r16,hifGLOB
    sbrs    r16,rfmENA
    rjmp    rfm12_000runERRORBACK
   	;broadcast? 
    lds     r16,serCOMmi
	sbrs	r16,b_cast
	rjmp	rfm12_003runRXcomAYES	;0=broadcast /no broadcast allowed

;*****************************************
;NO BROADCAST MESSAGE !
;b) read target LAB-target-host & search
;by scanning the (host_table)-contents
;NOTE: The target adress could be a
;- this host adress (used for rfm12 setup)
;- a direct host adress
;- a meshed host adress
;*****************************************

    ldi     zl,low(RFMtxbuffer)
    ldi     zh,high(RFMtxbuffer)
    adiw    zh:zl,1
	ld	r17,z				;load target adress
	sts	Thost_adr,r17

	;...............................
	;now check if it's a direct
	;OR a meshed host...
	;shadow ram !
	rcall	scan_160HOtable
	;...............................
	lds	r16,DIRhost	
	tst	r16
	brne	rfm12_003runRXcomB
	;-------------------------------
	;INVALID - HOST NOT AVAILABLE!
	;now enable the int1 interrupt
	;further low-level edge will
	;execute the receive service!
	;-------------------------------
  jmp rfm12_000runERRORBACK



;..............................
;DIRECT HOST FOUND !
;send direct OR meshed ?
;..............................
rfm12_003runRXcomB:
	lds	r17,T2host_adr	
	tst	r17
	brne	rfm12_003runRXcomC		;meshed !
;.............................................................................
;DIRECT HOST
;rfm12 transmit as a ,embedded-data'
;message to a DIRECT host
;TRANSFER INSTRUCTION = 2
;REQUESTED INSTRUCTION = 52
;format: <mode-instr.(2)><rfm-source-adr><rfm-target-adr>[EMB.DATA]<ck-sum>
;frame-size: (4xrfm12)+ EMB.DATA-size 
;.............................................................................
	;send state
	ldi	r16,2
	sts	STATEsend,r16
	rjmp	rfm12_000runBACK			;back in loop...


;.........................................
;TRANSFER DATA TO A MESHED HOST
;rfm12 transmit as a ,embedded-data'
;message to a MESHED host
;TRANSFER INSTRUCTION = 4
;REQUESTED INSTRUCTION = 55
;.........................................
rfm12_003runRXcomC:
	ldi	r16,4
	sts	STATEsend,r16
	rjmp	rfm12_000runBACK			;back in loop...





;.........................................
;BROADCAST
;rfm12 transmit embedded data to all 
;available direct hosts as a kind of
;broadcast message
;TRANSFER INSTRUCTION = 6
;REQUESTED INSTRUCTION = 52
;.........................................
rfm12_003runRXcomAYES:
	;..........................................
	;first have a look for direct hosts
	;=> stored in the shadow-segment !
	;returns: count of direct hosts <CNTd>
	;		direct hosts in [TE_garbage_seg]
	;..........................................
	rcall	scan_180HOtable
	;.......................
	;check if there's any 
	;direct host scannend!
	;.......................
	lds	r16,HOcnt
	tst	r16
	breq	rfm12_003runRXcomAYESkill	;no direct host - no broadcast !
	ldi	r16,6   
	sts	STATEsend,r16
	rjmp	rfm12_000runBACK			;back in loop...

rfm12_003runRXcomAYESkill:
	;-------------------------------
	;INVALID!
	;-------------------------------
  jmp rfm12_000runERRORBACK






;--------------------------------------------------------------------------------------------------------
;RECEIVE ERROR FRAME & BACK !
;_-------------------------------------------------------------------------------------------------------
rfm12_000runERRORBACK:
    lds     r17,COMTXstate
    cbr     r17,1<<BUFsecure           
    sts     COMTXstate,r17

   	rjmp	rfm12_000runBACK			;back in loop...



;------------------------------------------------------------------------------------------------------
;back in loop
;------------------------------------------------------------------------------------------------------

rfm12_000runBACK:
	ret















