;----------------------------------------------------------------------------------------------------------------
;Receive primary & secondary messages
;A) primary:
;1=  scan (transmit in order:50)
;2=  embedded data as a direct host (transmit in order:52)
;4=  embedded data to a meshed host - first step receive as the direct host (transmit in order:5)
;5=  embedded data from the direct received as the meshed-host (transmit in order:53)
;6=  embedded data to all available direct hosts (transmit in order:52)
;7=  alive message from a host (transmit in order:51)
;8=  kill message from a host (transmit in order:51)
;9=  lha change message (transmit in order:51)
;10= remote message (transmit in order:55)
;
;B)Secondary:
;50= receive available direct-host (incl.quality) by the calling host
;51= simple acknowledge received by the calling host (in order of a transmitted:7/8)
;52= simple acknowledge received by the calling host (in order of a transmitted:2/6)
;53= mesh-acknowledge received by the calling direct-host (in order of a transmitted:5)-> transmit:54 !
;54= mesh-acknowledge from the direct received by the calling host (in order of a transmitted:4) 
;55= remote acknowledge (in order of a transmitted:10)
;----------------------------------------------------------------------------------------------------------------
;
;.................................................................................................................
;RECEIVE THE INSTRUCTION-BYTE FROM RFM12
;* store received byte to (rfm_buffer)
;
;RESULTS: rfm_instruction 		<=> recorded rfm_instruction
;		STATErec				<=> recorded rfm_instruction	
;		rfQUAL				<=> data quality good counter
;.................................................................................................................
    ;............................
    ;copy & decode received data
    ;............................
rfm12_010run:  
    ;data in buffer ?
    lds r16,fifoVAR
	sbrs r16,fifoNEW
	rjmp  _errRX            ;state== 0 & no data == go back (receiver enabled ?)

    
    ;-------------------------
    ;NOTE: receiver curcuit 
    ;is still disabled !
    ;-------------------------
    ;data in buffer
    ;copy to rfm_buffer
    ;-------------------------
    ldi	zh,high(rfm_buffer)
	ldi	zl,low(rfm_buffer)
    ;load real segment size
    lds r16,fifoSEG
    ;load real fifo-segment startadress
    clr yh
	lds	yl,fifoOUT		;segmnet size out counter
    mov r18,yl          ;use for increment adressing
 	subi	yl,low(-fifoSEGrx)
    sbci    yh,high(-fifoSEGrx)


rfm12_010runGOloop:
    ld  r17,y
    st  z+,r17
    ;increment start pointer
    inc r18
    mov yl,r18
    clr yh
	subi	yl,low(-fifoSEGrx)
    sbci    yh,high(-fifoSEGrx)
    ;decrement segment size
    dec r16
    tst r16
    breq rfm12_010runGOloopREADY        ;ready loaded   
    rjmp  rfm12_010runGOloop
      
	;.................................................................
	;BUFFER- data load enable buffer load
	;.................................................................
rfm12_010runGOloopREADY:
    lds r16,fifoVAR
	cbr	r16,1<<fifoNEW
	sts	fifoVAR,r16
    ;load instruction 
    ldi	zh,high(rfm_buffer)
	ldi	zl,low(rfm_buffer)
    ld  r16,z
	sts	rfm12_instr,r16		;store rfm12 instruction to var

    
    ;..................................................................
    ;system waits for requested data !?
    ;..................................................................
    lds r18,STATEreq
    tst r18
    breq  rfm12_010runA     ;no data requested == decode all
    ;any data requested !
    cp  r18,r16
    breq  rfm12_010runA     ;requested data received = decode it
    ;not the requested data !
    ;=> perhaps a runnin' primary
    ;tx interrupted?
    ;=> therefore test the STATEsend
    ;for primary instruction...
    lds r18,STATEsend
    cpi r18,50
    brsh  rfm12_010runA     ;not relevant
    ;yes - interrupted by receiveing
    sts STATEsendTE,r18       
    jmp _errRX



	;..........................
	;receive state =1 ?
	;..........................
rfm12_010runA:
	cpi	r16,1
	brne	rfm12_010runB	
    ;.........................................................
 	st	z,r16				;store rfm12 instruction to buffer
    ;.........................................................
   	;store rx-state=1
	sts	STATErec,r16
	;store fixed framesize
	ldi	r16,4
	sts	FRAME_siz,r16	
    jmp    rfm12_011run




	;..........................
	;receive state =2 ?
	;..........................
rfm12_010runB:
	cpi	r16,2
	brne	rfm12_010runC 
    ;.........................................................
 	st	z,r16				;store rfm12 instruction to buffer
    ;.........................................................    			
	;store rx-state=2
	sts	STATErec,r16
    jmp    rfm12_012run




	;..........................
	;receive state =4 ?
	;..........................
rfm12_010runC:
	cpi	r16,4
	brne	rfm12_010runD
    
    ;.........................................................
  	st	z,r16				;store rfm12 instruction to buffer
    ;.........................................................    			
 	;store rx-state=4
	sts	STATErec,r16
  jmp    rfm12_013run

			







	;..........................
	;receive state =5 ?
	;..........................
rfm12_010runD:
	cpi	r16,5
	brne	rfm12_010runE

    ;.........................................................
    ;use rfm_buffer !
 	st	z,r16				;store rfm12 instruction to buffer
    ;.........................................................    			
	;store rx-state=5
	sts	STATErec,r16
  jmp    rfm12_014run
			




	;..........................
	;receive state =6 ?
	;broadcast massages will
	;be received by mode:2!
	;..........................
rfm12_010runE:
	cpi	r16,6
	brne	rfm12_010runF
    ;.........................................................
    ;use rfm_buffer !
 	st	z,r16				;store rfm12 instruction to buffer
    ;.........................................................    			    	
	ldi	r16,2		
	;store rx-state=2 !!!
	sts	STATErec,r16
  jmp    rfm12_012run






	;..........................
	;receive state =7/8 (ALIVE
	;OR KILL)?
	;..........................
rfm12_010runF:
	cpi	r16,7
	breq	rfm12_010runFA	
	cpi	r16,8
	breq	rfm12_010runFA	
	jmp	rfm12_010runG
rfm12_010runFA:	
    ;.........................................................
 	st	z,r16				;store rfm12 instruction to buffer
    ;.........................................................    			

	sts	STATErec,r16
	sts	STATErecTE,r16		
	;store fixed framesize
	;6x rfm12 frame
	ldi	r17,6
	sts	FRAME_siz,r17		
	  jmp    rfm12_017run

	;..........................
	;receive state =9 ?
    ;LHA change
	;..........................
rfm12_010runG:
	cpi	r16,9
	brne	rfm12_010runH	
    ;.........................................................
 	st	z,r16				;store rfm12 instruction to buffer
    ;.........................................................
   	;store rx-state=9
	sts	STATErec,r16
	sts	STATErecTE,r16		
	;store fixed framesize
	ldi	r16,5
	sts	FRAME_siz,r16	
    jmp    rfm12_019run

	;..........................
	;receive state =10 ?
    ;REMOTE
	;..........................
rfm12_010runH:
	cpi	r16,10
	brne	rfm12_010runI
     ;.........................................................
 	st	z,r16				;store rfm12 instruction to buffer
    ;.........................................................    			
	;store rx-state=2
	sts	STATErec,r16
    jmp    rfm12_020run

	;..........................
	;receive state =11 ?
    ;RESET
	;..........................
rfm12_010runI:
	cpi	r16,11
	brne	rfm12_010runJ
     ;.........................................................
 	st	z,r16				;store rfm12 instruction to buffer
    ;.........................................................    			
	;store rx-state=11
	sts	STATErec,r16
    jmp    rfm12_021run


	;..........................
	;receive state =50 ?
	;..........................
rfm12_010runJ:
	cpi	r16,50
	brne	rfm12_010runK
   ;.........................................................
  	st	z,r16				;store rfm12 instruction to buffer
    ;.........................................................    			
	;store rx-state=1
	sts	STATErec,r16
	;store fixed framesize
	;6x rfm12 frame
	ldi	r17,6
	sts	FRAME_siz,r17		
  jmp    rfm12_050run



	;..........................
	;receive state =51 ?
	;..........................
rfm12_010runK:
	cpi	r16,51
	brne	rfm12_010runL
  ;.........................................................
 	st	z,r16				;store rfm12 instruction to buffer
    ;.........................................................    			
	;store rx-state=1
	sts	STATErec,r16
	;store fixed framesize
	;4x rfm12 frame
	ldi	r17,4
	sts	FRAME_siz,r17		
  jmp    rfm12_051run




	;..........................
	;receive state =52..54 ?
	;..........................
rfm12_010runL:
	cpi	r16,55
	brsh	rfm12_010runM	            ;
	cpi	r16,52
	brlo	_errRX
    ;.........................................................
  	st	z,r16				;store rfm12 instruction to buffer
    ;.........................................................    			
    
	;store rx-state=1
	sts	STATErec,r16
	;store fixed framesize
	;5x rfm12 frame
	ldi	r17,5
	sts	FRAME_siz,r17		
  jmp    rfm12_052run



	;..........................
	;receive state =55 ?
	;..........................
rfm12_010runM:
	cpi	r16,55
	brne	_errRX  ;receiver is disabled...
  ;.........................................................
 	st	z,r16				;store rfm12 instruction to buffer
    ;.........................................................    			
	;store rx-state=1
	sts	STATErec,r16
	;store fixed framesize
	;4x rfm12 frame
	ldi	r17,5
	sts	FRAME_siz,r17		
    jmp    rfm12_053run






    ;..............................
    ;-  no data in buffer
    ;or false identifier
    ;..............................
_errRX:
    ;clear rx-state=0
	clr	r16
	sts	STATErec,r16
	ret




	;..............................
	;- receive state (<50)in range
    ;& invalid data
    ;- no tx requested !!!
    ;- enable tx buffer
    ;SPI still in mastermode !
    ;..............................
_errRX_DATA: 
  	;store rx-state=0
	clr	r16
	sts	STATErec,r16
	;clear framesize
	sts	FRAME_siz,r16		
	;clear byte-counter			
	sts	RECbyte,r16
    ;enable buffer
    sts	COMTXstate,r16	
	ret


	;...............................
	;- receive state (<50)in range
    ;  & valid data
    ;  & tx requested !!!
    ;- disable tx buffer
    ;SPI still in mastermode !
    ;...............................
_valRX_DATA: 
    ;store STATEsend 
    sts	STATEsend,r16
	;store rx-state=0
	clr	r16
	sts	STATErec,r16
	sts	STATEreq,r16
	;clear framesize
	sts	FRAME_siz,r16		
	;clear byte-counter			
	sts	RECbyte,r16
       ;...............................
        ;but buffer secure now activ !
        ;clear after acknowledge....
        sbr r16,1<<BUFsecure  
        sts	COMTXstate,r16	
        ;..............................
	ret





	;..............................
	;- receive state (>=50)in range
    ;& invalid data
    ;- no tx requested !!!
    ;- perhaps tx repeat
    ;SPI still in mastermode !
    ;..............................
_errRX_DATAb: 
  	;store rx-state=0
	clr	r16
	sts	STATErec,r16
	;clear framesize
	sts	FRAME_siz,r16		
	;clear byte-counter			
	sts	RECbyte,r16
    ;clear request state
	sts	STATEreq,r16
 	ret





	;..............................
	;- receive state in range
    ;  & valid data
    ;  & tx requested !!!
    ;SPI still in mastermode !
    ;..............................
_valRX_DATAb: 
    ;store STATEsend 
;    sts	STATEsend,r16
	;store rx-state=0
	clr	r16
	sts	STATErec,r16
	sts	STATEreq,r16
	;clear framesize
	sts	FRAME_siz,r16		
	;clear byte-counter			
	sts	RECbyte,r16
	;clr repeat
	sts	TXrepCNT,r16
       ;...............................
        ;but buffer secure now activ !
        ;clear after acknowledge....
        sbr r16,1<<BUFsecure  
        sts	COMTXstate,r16	
        ;..............................
	ret


	;..............................
	;- receive state in range
    ;  & valid data
    ;  & tx requested !!!
    ;SPI still in mastermode !
    ;..............................
_valRX_DATAc: 
    ;..............................
    ;note: fi there's a broadcast 
    ;message runnin' (STATEsend=6)
    ;then also check & decrement the 
    ;HOcnt-1 !
    ;..............................
    lds r16,STATEsend
    cpi r16,6
    breq  _valRX_DATAcBROAD         ;yes, it's a broadcast  
    ;no broadcast
	clr	r16
    sts	STATEsend,r16
	sts	STATErec,r16
	sts	STATEreq,r16
	;clear framesize
	sts	FRAME_siz,r16		
	;clear byte-counter			
	sts	RECbyte,r16
	;clr repeat
	sts	TXrepCNT,r16
    ;enable buffer
    sts	COMTXstate,r16	
	ret
                                            ;acknowledge, broadcast message received
                                    _valRX_DATAcBROAD:
                                            ;anymore hosts ?
                                            lds	r16,HOcnt				            ;<=1 ready !
                                            cpi	r16,2
                                            brsh	_valRX_DATAcBROADNXThost		;next host....
                                            ;<2
                                            ;don't decrement any more
                                            ;because the last host in
                                            ;table is reached !
                                        	clr	r16
                                            sts	STATEsend,r16
                                        	sts	STATErec,r16
                                        	sts	STATEreq,r16
                                        	;clear framesize
                                        	sts	FRAME_siz,r16		
                                        	;clear byte-counter			
                                        	sts	RECbyte,r16
                                        	;clr repeat
                                        	sts	TXrepCNT,r16
                                            ;enable buffer
                                            sts	COMTXstate,r16	
                                        	ret
  
                                          

                                            ;..........................
                                            ;set next hostpointer
                                            ;..........................
                                    _valRX_DATAcBROADNXThost:
                                            dec	r16
                                            sts	HOcnt,r16
                                        	;don't clear the STATEsend
                                            clr	r16
                                          	sts	STATErec,r16
                                        	sts	STATEreq,r16
                                        	;clear framesize
                                        	sts	FRAME_siz,r16		
                                        	;clear byte-counter			
                                        	sts	RECbyte,r16
                                        	;clr repeat
                                        	sts	TXrepCNT,r16
                                        	ret
            






;.................................................................................................................
;1= RECEIVE THE SCAN-INSTRUCTION FROM RFM12
;* store received data to (rfm_buffer)
;
;RESULTS: rfm_instruction 		<=> recorded rfm_instruction
;		STATErec				<=> recorded rfm_instruction	
;		rfQUAL				<=> data quality good counter
;		RECbyte (byte counter)	<=> byte counter
;STRUCTURE:
;<rfm12-instr.(1)>|<source-adress>|<target-adress>|<checksum 8bit>
;OK !
;.................................................................................................................
;jmp
rfm12_012runX:
	jmp	rfm12_012run

rfm12_011run:
    lds	r16,STATErec
	cpi	r16,1
	brne	rfm12_012runX		;>1 check next modul
   	;.........................
	;store the source adress #1
	;.........................
	clr	zh
	ldi	zl,1	
	subi	zl,low(-rfm_buffer)
	sbci	zh,high(-rfm_buffer)
	ld	r16,z+
	sts	Thost_adr,r16		            ;the new target adress for acknowledge !
	;.........................
	;check target (scan)adress#2
	;.........................
	ld	r16,z
	lds	r17,Shost_adr
	cp	r16,r17
	brne	rfm12_011runKILL			;adress not equal !

	;.........................
	;proof check sum
	;.........................
	call	rfm12_rxCHKsum	
	lds	r16,rxCKsumREC
	lds	r17,rxCKsumBUILT
	cp	r16,r17
	brne	rfm12_011runKILL			;ck-sum not equal !
	;.........................
	;data good
	;.........................
	cbi	portb,rfmNET				;led on
	;...................................
	;generate requested acknowledge
	;message (50)! 
	;contents all direct hosts in table
	;as meshed hosts
	;...................................
	;state send = 50
	ldi	r16,50  ;50
 	jmp _valRX_DATA                     ;tx 50 requested
	;.........................
	;no valid data received
	;<=> invalid !
	;.........................
rfm12_011runKILL:
    ;not valid data
  	jmp _errRX_DATA                     ;enable all....






;.................................................................................................................
;2= RECEIVE THE TRANSFER-INSTRUCTION FROM RFM12 - EMBEDDED DATA TO A DIRECT HOST
;6= RECEIVE THE TRANSFER-INSTRUCTION FROM RFM12 - EMBEDDED DATA TO A DIRECT HOST BY BROADCAST

;* store received data to (rfm_buffer)
;
;RESULTS: rfm_instruction 		<=> recorded rfm_instruction
;		STATErec				<=> recorded rfm_instruction	
;		rfQUAL				<=> data quality good counter
;		RECbyte (byte counter) 	<=> byte counter
;.................................................................................................................
;jmp
rfm12_013runX:
	jmp	rfm12_013run

rfm12_012runKILLz:
	jmp	rfm12_012runKILL


rfm12_012run:
    lds	r16,STATErec
	cpi	r16,2
	brne	rfm12_013runX		;>2 check next modul
	;.........................
	;check target (scan)adress
	;.........................
	clr	zh
	ldi	zl,1	
	subi	zl,low(-rfm_buffer)
	sbci	zh,high(-rfm_buffer)
	ld	r16,z+
	sts	Thost_adr,r16				;NOTE: This is the adress of the requesting host...
	ld	r16,z
	lds	r17,Shost_adr
	cp	r16,r17
	brne	rfm12_012runKILLz			;adress not equal !






    ;..........................................
    ;load & calculate complete frame size #8
    ;..........................................
	clr	yh
	ldi	yl,8
	subi	yl,low(-rfm_buffer)
	sbci	yh,high(-rfm_buffer)			;startpoint first embedded byte
    ld  r16,y
    ldi r17,4
    add r16,r17
    sts	FRAME_siz,r16	




	;.........................
	;proof check sum
	;.........................
	call	rfm12_rxCHKsum	
	lds	r16,rxCKsumREC
	lds	r17,rxCKsumBUILT
	cp	r16,r17
	brne	rfm12_012runKILLz			;ck-sum not equal !
	;.........................
	;data good
	;.........................
	cbi	portb,rfmNET				;led on
	;..............................
	;write system-data to TX_buffer 
	;& send to system
	;..............................
    ;load frame size #8
	clr	yh
	ldi	yl,8
	subi	yl,low(-rfm_buffer)
	sbci	yh,high(-rfm_buffer)			;startpoint first embedded byte
    ld  r18,y
	ldi	r17,1						;byte counter =1
	sts	RECbyte,r17
    ;startpoint#3
    sbiw    yh:yl,5
	ldi	zl,low(TX_buffer)
	ldi	zh,high(TX_buffer)				;startpoint TX_buffer

rfm12_012runLOOP:
	ld	r16,y+
	st	z+,r16
	cp	r17,r18
	brsh	rfm12_012runLOOPex				;limit reached
	inc	r17
	jmp	rfm12_012runLOOP				;store in loop
	;.............................
	;TX_buffer filled
	;uart start condition
	;.............................
rfm12_012runLOOPex:
	lds	r16,comNETstate
	sbr	r16,1<<comNETnewDATtx
	sts	comNETstate,r16
	;..............................
	;generate requested acknowledge
	;,ack' message (52)!
	;..............................
	ldi	r16,52
	jmp _valRX_DATA         ;tx requested
	;.........................
	;no valid data received
	;<=> invalid !
	;.........................
rfm12_012runKILL:
 	jmp _errRX_DATA         ;enable all






;.................................................................................................................
;4= RECEIVE THE TRANSFER-INSTRUCTION FROM RFM12 - TRANSFER DATA TO A MESHED HOST, BUT FIRST TO THIS DIRECT HOST
;* store received data to (rfm_buffer)
;Second step: TRANMIT THIS DATA-FRAME TO THE MESHED HOST (MODE:5)
;.................................................................................................................
;jmp
rfm12_014runX:
	jmp	rfm12_014run
rfm12_013run:
    lds	r16,STATErec
	cpi	r16,4
	brne	rfm12_014runX		;>3 check next modul
	;.........................
	;compare Dhost/Shost
	;.........................
	clr	zh
	ldi	zl,2	
	subi	zl,low(-rfm_buffer)
	sbci	zh,high(-rfm_buffer)
	ld	r16,z+
	lds	r17,Shost_adr
	cp	r16,r17
	brne	rfm12_013runKILL			;adress not equal !
	;.........................
	;proof check sum
	;.........................
	call	rfm12_rxCHKsum	
	lds	r16,rxCKsumREC
	lds	r17,rxCKsumBUILT
	cp	r16,r17
	brne	rfm12_013runKILL			;ck-sum not equal !
	;.........................
	;data good
	;.........................
	cbi	portb,rfmNET			
	;set transmit state
	ldi	r16,5 	;!!!
	jmp _valRX_DATA                     ;tx requested
	;.........................
	;no valid data received
	;<=> invalid !
	;.........................
rfm12_013runKILL:
	;not valid - next receiver on
 	jmp _errRX_DATA                     ;enable all






;.................................................................................................................
;5= RECEIVE THE TRANSFER-INSTRUCTION FROM RFM12 - TRANSFER DATA TO A MESHED HOST
;* store received data to (rfm_buffer/tx_buffer) and send to system (uart) 
;in order: transmit acknowledge to the direct host (mode=53)
;<rfm12-instr><sou-adr><D-tar-adr><M-tar-adr>[EMB-DATA]<ck-sum>
;.................................................................................................................
;jmp
rfm12_017runX:
	jmp	rfm12_017run

rfm12_014runKILLz:
	jmp	rfm12_014runKILL

rfm12_014run:
    lds	r16,STATErec
	cpi	r16,5
	brne	rfm12_017runX		;>2 check next modul
	;.........................
	;check target (scan)adress#3
	;.........................
	clr	zh
	ldi	zl,3	
	subi	zl,low(-rfm_buffer)
	sbci	zh,high(-rfm_buffer)
	ld	r16,z
	lds	r17,Shost_adr
	cp	r16,r17
	brne	rfm12_014runKILLz			;adress not equal !

	;........................
	;adresses for acknowledge#1
	;........................
    sbiw    zh:zl,2
	ld	r16,z+
	sts	T2host_adr,r16					;the calling hosts adress
	ld	r16,z
	sts	Thost_adr,r16					;the calling direct host adress

	;.........................
	;proof check sum
	;.........................
	call	rfm12_rxCHKsum	
	lds	r16,rxCKsumREC
	lds	r17,rxCKsumBUILT
	cp	r16,r17
	brne	rfm12_014runKILLz			;ck-sum not equal !
	;.........................
	;data good
	;.........................
	cbi	portb,rfmNET				;led on
	;..............................
	;write system-data to TX_buffer 
	;& send to system
	;..............................
    ;frame size#10
	clr	yh
	ldi	yl,10
	subi	yl,low(-rfm_buffer)
	sbci	yh,high(-rfm_buffer)			;startpoint first embedded byte
    ld  r18,y
	subi	r18,5
	sts	FRAME_siz,r18					;complete frame - rfm12 size
	ldi	r17,1						;byte counter =1
	sts	RECbyte,r17
    ;startpoint#4
    sbiw    yh:yl,6
	ldi	zl,low(TX_buffer)
	ldi	zh,high(TX_buffer)				;startpoint TX_buffer
rfm12_014runLOOP:
	ld	r16,y+
	st	z+,r16
	cp	r17,r18
	brsh	rfm12_014runLOOPex				;limit reached
	inc	r17
	jmp	rfm12_014runLOOP				;store in loop
	;.............................
	;TX_buffer filled
	;uart start condition
	;.............................
rfm12_014runLOOPex:
	lds	r16,comNETstate
	sbr	r16,1<<comNETnewDATtx
	sts	comNETstate,r16
	;..............................
	;generate requested acknowledge
	;,ack' message (53)!
	;..............................
	;set transmit state
	ldi	r16,53
	jmp _valRX_DATA                     ;tx requested
	;.........................
	;no valid data received
	;<=> invalid !
	;.........................
rfm12_014runKILL:
	;not valid - next receiver on
   	jmp _errRX_DATA                     ;enable all

















;.................................................................................................................
;7/8= RECEIVE THE TRANSFER-INSTRUCTIONS FROM RFM12 - ALIVE OR KILL (CONTAINS NO EMBEDDED DATA)
;* store received data to (rfm_buffer)
;
;RESULTS: rfm_instruction 		<=> recorded rfm_instruction
;		STATErec				<=> recorded rfm_instruction	
;		rfQUAL				<=> data quality good counter
;		RECbyte (byte counter) 	<=> byte counter
;the received alive-frame-structure:
;<mode-instr.>|<#direct hosts>|<host-adress>|<alive-adress>|<host-quality>|<n*mesh-host>|<n*mesh-host quality>|<checksum 8bit>
;scan-adress  = source adress of the ,alive'-host
;host-quality = by the receiver calculated quality
;.................................................................................................................
;jmp



rfm12_017run:
    lds	r16,STATErec
	cpi	r16,7
	breq	rfm12_017runW
	cpi	r16,8
	breq	rfm12_017runW
	jmp	rfm12_019run
rfm12_017runW:
    clr	yh
	ldi	yl,1
	subi	yl,low(-rfm_buffer)
	sbci	yh,high(-rfm_buffer)			;contains the count of direct hosts
	ld  zl,y+                    			;store to buffer
	ldi	zh,2
	mul	zl,zh
	lds	zh,FRAME_siz
	add	zh,r0				;frame-size = (count *2)+frame-size (old)
	sts	FRAME_siz,zh
	sts	tSEG_siz,zh			;for table scanner !!!!	

	;........................
	;2= check the adress...
	;........................
rfm12_017runB:
	ld	zl,y+
	lds	zh,Shost_adr
	cp	zh,zl
	breq	rfm12_017runC
	jmp	rfm12_017runKILL		;not ok!
	;........................
	;3=store the adress...
	;........................
rfm12_017runC:
	ld	zl,y
	sts	DIRhostREC,zl
	sts	Thost_adr,zl
	;**************************
	;J) all bytes are received
	;now calculate checksum
	;**************************
rfm12_017runALL:
	call	rfm12_rxCHKsum
	lds	zl,rxCKsumREC
	lds	zh,rxCKsumBUILT
	cp	zh,zl
	breq	rfm12_017runQUAL		;ok, now check quality
	jmp	rfm12_017runKILL		;not ok...
	;**************************
	;K) calculate the quality
	;factor in 1...100%
	;needs time !
	;min.90cycles /max.177 cycles
	;equal: max.17s (11MHz fclk)
	;**************************
rfm12_017runQUAL:
	;.........................
	;.........................
	;data good
	;.........................
	cbi	portb,rfmNET				;led on
	;	
	call	rfm12_Qfactor
	;...........................
	;store quality
	;...........................
	ldi	zh,high(rfm_buffer)
	ldi	zl,low(rfm_buffer)
	adiw	zh:zl,4
	lds	r16,Qfactor
	st	z,r16
	;transmit state=51 (simple ack)
	ldi	r16,51
	jmp _valRX_DATA                 ;tx requested
	;........................
	;invalid data..
	;........................
rfm12_017runKILL:
	;not valid- next receiver on
 	jmp _errRX_DATA                 ;enable all..










;.................................................................................................................
;9= RECEIVE THE TRANSFER-INSTRUCTIONS FROM RFM12 - LHAdress CHANGE (CONTAINS NO EMBEDDED DATA)
;* received data stored to (rfm_buffer)
;<mode-instr.(9)>|<Shost-adress>|<Thost-adress>|<OLDShost-adress>|<checksum 8bit>
;.................................................................................................................
rfm12_020runX:
    jmp rfm12_020run
rfm12_019run:
    lds	r16,STATErec
	cpi	r16,9
	brne	rfm12_020runX		
   	;.........................
	;store the source adress #1
    ;as the new target adress ! 
	;.........................
	clr	zh
	ldi	zl,1	
	subi	zl,low(-rfm_buffer)
	sbci	zh,high(-rfm_buffer)
	ld	r16,z+
	sts	Thost_adr,r16		            ;the new target adress for acknowledge !
	;.........................
	;check target (scan)adress#2
	;.........................
	ld	r16,z+
	lds	r17,Shost_adr
	cp	r16,r17
	brne	rfm12_019runKILL			;adress not equal !
    ;.........................
    ;load the hosts old adress
    ;.........................
	ld	r16,z+
	sts	OLDShost_adr,r16		            ;the new target adress for acknowledge !
	;.........................
	;proof check sum
	;.........................
	call	rfm12_rxCHKsum	
	lds	r16,rxCKsumREC
	lds	r17,rxCKsumBUILT
	cp	r16,r17
	brne	rfm12_019runKILL			;ck-sum not equal !
	;.........................
	;data good
	;.........................
	cbi	portb,rfmNET				;led on
	;...................................
	;generate requested acknowledge
	;message (51)! 
	;...................................
	;state send = 51
	ldi	r16,51  ;51
 	jmp _valRX_DATA                     ;tx 51 requested
	;.........................
	;no valid data received
	;<=> invalid !
	;.........................
rfm12_019runKILL:
    ;not valid data
  	jmp _errRX_DATA                     ;enable all....




;.................................................................................................................
;10= RECEIVE THE TRANSFER-INSTRUCTION FROM RFM12 - EMBEDDED DATA TO A DIRECT HOST
;Decide if it's a:
;- Intruction without data          (REMtype=REMinstr) + (REMsize=8)
;- Instruction with controller data (REMtype=REMconDAT)+ (REMsize= [spiDATctSOLL]+8) --- set by spiIRQ
;- Instruction with display data    (REMtype=REMdisDAT)+ (REMsize=14) --- set by spiTX
;
;format:
;<m-instr><LAB-source><LAB-target><Master-adress><Client-adress><REMtype><REMsize>[data]<ck-sum>
;                          |                      
;                       this host  
;NOTE: activ remote connection uses channel 15               
;.................................................................................................................
;jmp
rfm12_021runX:
	jmp	rfm12_021run
;-----------------------
rfm12_020runKILLz:
	jmp	rfm12_020runKILL
;-----------------------
rfm12_020runCLIENTx:
    jmp rfm12_020runCLIENT
;-----------------------
rfm12_020run:
    lds	r16,STATErec
	cpi	r16,10
	brne	rfm12_021runX		;>2 check next modul
  	;...........................
	;A)store the source adress #1
    ;as the new target adress ! 
    ;for ack needed !
	;...........................
	clr	zh
	ldi	zl,1	
	subi	zl,low(-rfm_buffer)
	sbci	zh,high(-rfm_buffer)
	ld	r16,z+
	sts	Thost_adr,r16		            ;the new target adress for acknowledge !
	;.........................
	;A1)check target adress
	;.........................
	ld	r16,z+
	lds	r17,Shost_adr
	cp	r16,r17
	brne	rfm12_020runKILLz			;adress not equal !
    ;.........................
    ;A2)this host the master or
    ;the client?
    ;If (0) then shutdown remote
    ;.........................
    ld      r22,z+                      ;load the masters adress
	sts	REMmaster,r22		            ;the master adress !
    cp      r22,r17
    brne    rfm12_020runCLIENTx          
    ;.........................
    ;A3)this host is the master !
    ;-> load & test client 
    ;adress =0 ? means shutdown !
    ;.........................
    ld      r22,z+                      ;load the slave adress
	sts	    REMslave,r22		           
    ;.........................
    ;A4) calculate the frame
    ;size...
    ;.........................
  	clr	zh
	ldi	zl,5
	subi	zl,low(-rfm_buffer)
	sbci	zh,high(-rfm_buffer)	
    ld  r16,z+
    sts REMtypeTE,r16                ;remote type loaded & temporary stored..
    ld  r16,z
    sts REMsizeTE,r16
    ldi r17,8
    add r16,r17
    sts	FRAME_siz,r16	
	;.........................
	;A4)check sum
	;.........................
	call	rfm12_rxCHKsum	
	lds	r16,rxCKsumREC
	lds	r17,rxCKsumBUILT
	cp	r16,r17
	brne	rfm12_020runKILLz			;ck-sum not equal !
	;.........................
	;A5)data good
	;.........................
	cbi	portb,rfmNET				;led on
    ;.........................
    ;A6) what kind of data 
    ;received the master
    ;- instruction from slave?
    ;- display data from slave?
    ;.........................
    lds r16,REMtypeTE
    cpi r16,REMinstr
    breq    rfm12_020masterINSTR        ;A)instruction WITHOUT data
    cpi r16,REMdisDAT
    breq   rfm12_020masterDATA          ;B)instruction WITH display-data
;    cpi r16,REMconDAT
;    breq    rfm12_020masterCON          ;C)instruction WITH control-data
    jmp     rfm12_020runKILL            ;invalid....

    
    ;.............................
    ;A7) Instruction from client
    ;-> always shutdown !
    ;.............................
 rfm12_020masterINSTR: 	
    ;.............................
    ;A8)clear local remote flags
    ;& in order the systems remote
    ;flags
    ;............................. 
    ;clear remote flags
;    clr  r16                ;
;    sts REMif,r16
    ;buffer start
    ldi	zl,low(TX_buffer)
	ldi	zh,high(TX_buffer)				;startpoint TX_buffer
    ;B)mi: setup/to core
    ldi r16,0b01001000          
  	st  z+,r16
    ;C)4xdummy
    ldi r16,0xaa
 	st  z+,r16
  	st  z+,r16
  	st  z+,r16
  	st  z+,r16
    ;D)size=7
    ldi r16,7                  
  	st  z+,r16
    ;E)type=11 (off)
    ldi r16,11
  	st  z,r16
    ;enable tranmit to system
 	lds	r16,comNETstate
	sbr	r16,1<<comNETnewDATtx
	sts	comNETstate,r16
    ;back
    jmp rfm12_020runLOOPex

    


    ;.............................
    ;A9) Display data from client
    ;-> data to <COMbuf>
    ;.............................
 rfm12_020masterDATA:
    ;NEW
    lds r16,comFLAG
	sbrs	r16,newHIFdata
    rjmp   rfm12_020masterDATAb 
    ;back
    jmp rfm12_020runKILL
rfm12_020masterDATAb:
	cbr	r16,1<<spiBUSYhif
	sbr	r16,1<<newHIFdata
;    cbr r16,1<<waitHIFdata          ;no more waiting - hif data received
	sts	comFLAG,r16
    ;load size
    lds r22,REMsizeTE
    ;startpoint rfm_buffer
    clr	zh
	ldi	zl,7
	subi	zl,low(-rfm_buffer)
	sbci	zh,high(-rfm_buffer)	
  	;comBUF startadresse
	ldi	yh,high(comBUF)
	ldi	yl,low(comBUF)
    ;copy in loop
rfm12_020masterDATAloop: 
    ld  r16,z+
    st  y+,r16
    dec r22
    tst r22
    breq   rfm12_020masterDATAready
    rjmp   rfm12_020masterDATAloop 
    ;data already in <COMbuf>
rfm12_020masterDATAready:   
    ;cursorroutine disabled 
    ;till data receiving
	lds	r17,curMODflTE
	cbr	r17,1<<BOXcursor ;cursorroutine disable
	cbr	r17,1<<BOXactiv
	sts	curMODflTE,r17
    ;back
    jmp rfm12_020runLOOPex

   
   
   
    




    ;............................
    ;C) is this host the client?
    ;............................
;.........................
;jmp
rfm12_020runKILLzz:
    jmp rfm12_020runKILL
;.........................
rfm12_020runCLIENT:
    ld      r22,z+                      ;load the clients adress
	sts	    REMslave,r22		           
    cp      r22,r17
    brne    rfm12_020runKILLzz            ;nothing of both- kill !
    ;.........................
    ;this host is the client !
    ;C1) calculate the frame
    ;size...
    ;.........................
 	clr	zh
	ldi	zl,5
	subi	zl,low(-rfm_buffer)
	sbci	zh,high(-rfm_buffer)	
    ld  r16,z+
    sts REMtypeTE,r16                ;remote type loaded & temporary stored..
    ld  r16,z
    sts REMsizeTE,r16
    ldi r17,8
    add r16,r17
    sts	FRAME_siz,r16	


	;.........................
	;C2)check sum
	;.........................
	call	rfm12_rxCHKsum	
	lds	r16,rxCKsumREC
	lds	r17,rxCKsumBUILT
	cp	r16,r17
	brne	rfm12_020runKILLzz			;ck-sum not equal !
	;.........................
	;C3)data good
	;.........................
	cbi	portb,rfmNET				;led on
    ;.........................
    ;C4) what kind of data 
    ;received the client
    ;- instruction from master?
    ;- control data from master?
    ;.........................
    lds r16,REMtypeTE
    cpi r16,REMinstr
    breq    rfm12_020clientINSTR        ;A)instruction WITHOUT data
;    cpi r16,REMdisDAT
;    breq   rfm12_020clientDATA          ;B)instruction WITH display-data
    cpi r16,REMconDAT
    breq    rfm12_020clientCON          ;C)instruction WITH control-data
    jmp     rfm12_020runKILL            ;invalid....

  
    ;.............................
    ;A7) Instruction from master
    ;-> set the if/sys as a slave
    ;.............................
 rfm12_020clientINSTR: 
  ;.................
    ;remote activ text
;    call    _SYSmsgK
;    call	_sys03IF
 
   ;buffer start
    ldi	zl,low(TX_buffer)
	ldi	zh,high(TX_buffer)				;startpoint TX_buffer
    ;B)mi: setup/to core
    ldi r16,0b01001000          
  	st  z+,r16
    ;C)4xdummy
    lds r16,REMmaster
 	st  z+,r16
    ldi r16,0xaa
  	st  z+,r16
  	st  z+,r16
  	st  z+,r16
    ;D)size=7
    ldi r16,7                  
  	st  z+,r16
    ;E)type=10 (on)
    ldi r16,10
  	st  z,r16
    ;enable tranmit to system
 	lds	r16,comNETstate
	sbr	r16,1<<comNETnewDATtx
	sts	comNETstate,r16
    ;set local flags
    clr r16
    sbr r16,1<<REMon        ;remote on
    cbr r16,1<<REMstate     ;slave
    sts REMif,r16 
    ;back
    jmp rfm12_020runLOOPex



    ;.............................
    ;A8) Control data from master
    ;-> store to var's
    ;.............................
 rfm12_020clientCON:
    ;NEW
	lds	r16,comFLAG
    sbrs r16,waitHIFdata        ;-> wait for concept-data... cleared after interpreting !
    rjmp rfm12_020clientCONb 
    sbrs r16,newHIFtx        ;-> wait for concept-data... cleared after interpreting !
    rjmp rfm12_020clientCONb 
    ;system busy...
    jmp rfm12_020runKILL


rfm12_020clientCONb:
    ;.................
    ;start transmitting to system
   	sbr	r16,1<<newHIFtx
    sbr r16,1<<waitHIFdata  ;-> wait for concept-data... cleared after interpreting !
	sts	comFLAG,r16


 	clr	zh
	ldi	zl,7
	subi	zl,low(-rfm_buffer)
	sbci	zh,high(-rfm_buffer)	
    ;.............................
    ;A9) store to var's
    ;.............................
    ;index
    ld  r22,z+
    sts curINDEXte,r22
    ;graph function
	ld 	r22,z+
    sts paraFUNCte,r22
    ;para position
	ld	r22,z+
    sts paraPOSte,r22
    ;parameter
	ld	r22,z+
    sts paraAte,r22
    ;para limit lower
	ld	r22,z+
    sts paraMINte,r22
    ;para limit upper
	ld	r22,z
    sts paraMAXte,r22
     ;set local flags
    clr r16
    sbr r16,1<<REMon        ;remote on
    cbr r16,1<<REMstate     ;slave
    sts REMif,r16 
;	call	_conceptSND   
    ;back
    jmp rfm12_020runLOOPex





rfm12_020runLOOPex:
	;..............................
	;generate requested acknowledge
	;,ack' message (55)!
	;..............................
	ldi	r16,55
	jmp _valRX_DATA         ;tx requested
	;.........................
	;no valid data received
	;<=> invalid !
	;.........................
rfm12_020runKILL:
 	jmp _errRX_DATA         ;enable all














;.................................................................................................................
;11= RECEIVE THE TRANSFER-INSTRUCTIONS FROM RFM12 - RESET IF&SYS
;* received data stored to (rfm_buffer)
;<mode-instr.(11)>|<Shost-adress>|<Thost-adress>|<checksum 8bit>
;.................................................................................................................
rfm12_050runX:
    jmp rfm12_050run
rfm12_021run:
    lds	r16,STATErec
	cpi	r16,11
	brne	rfm12_050runX		
   	;.........................
	;store the source adress #1
    ;as the new target adress ! 
	;.........................
	clr	zh
	ldi	zl,1	
	subi	zl,low(-rfm_buffer)
	sbci	zh,high(-rfm_buffer)
	ld	r16,z+
	sts	Thost_adr,r16		            ;the new target adress for acknowledge !
	;.........................
	;check target (scan)adress#2
	;.........................
	ld	r16,z+
	lds	r17,Shost_adr
	cp	r16,r17
	brne	rfm12_021runKILL			;adress not equal !
	;.........................
	;proof check sum
	;.........................
	call	rfm12_rxCHKsum	
	lds	r16,rxCKsumREC
	lds	r17,rxCKsumBUILT
	cp	r16,r17
	brne	rfm12_021runKILL			;ck-sum not equal !
	;.........................
	;data good
	;.........................
	cbi	portb,rfmNET				;led on
	;...................................
	;generate requested acknowledge
	;message (56)! 
	;...................................
	;state send = 56
	ldi	r16,56  ;56
 	jmp _valRX_DATA                     ;tx 56 requested
	;.........................
	;no valid data received
	;<=> invalid !
	;.........................
rfm12_021runKILL:
    ;not valid data
  	jmp _errRX_DATA                     ;enable all....













;.................................................................................................................
;50= RECEIVE THE (REQUESTED) TRANSFER-INSTRUCTION - ACKNOWLEDGE, INCLUDING HOSTTABLE AFTER SCANNING
;
;- time out counter included (if no data received)
;- data quality check included
;- repeated tx possibility included (jmp to E)
;OK!
;-----------------------------------------------------------------------------------------------------------------
;the received scan-frame-structure:
;<mode-instr.>|<#meshed hosts>|<host-adress>|<scanned-adress>|<host-quality>|<n*mesh-host>|<n*mesh-host quality>|<checksum 8bit>
;
;explaining the shortform:
;#meshed hosts			<=>	count of meshed host stored by the scanned host
;scanned-adress		<=>	adress of the scanned host
;host-quality			<=>	received data quality by this scan
;n*mesh-host			<=>  adress of a meshed host stored by the scanned host
;n*mesh-host quality	<=>	meshed host data quality, stored by the scanned host
;host-adress			<=>	this devices adress
;
;calculate the the frame-size(including checksum):
;base-size, without any meshed hosts = 6
;per existing mesh-host add 2
;
;for example:
;base size + 3 meshed hosts	=	6+ (2*3)
;				FRAME_siz	=	12
;-----------------------------------------------------------------------------------------------------------------
;jmp
rfm12_051runX:
	jmp	rfm12_051run
rfm12_050run:
	cpi	r16,50
	brne	rfm12_051runX		;>2 check next modul
    ;check requested
;	lds	r16,STATEreq            ;check requested state !!!!
;	cpi	r16,50
;	brne	rfm12_050runKILL

    clr	yh
	ldi	yl,1
	subi	yl,low(-rfm_buffer)
	sbci	yh,high(-rfm_buffer)			;contains the count of direct hosts
	ld	zl,y+
	ldi	zh,2
	mul	zl,zh
	lds	zh,FRAME_siz
	add	zh,r0				;frame-size = (count *2)+frame-size (old)
	sts	FRAME_siz,zh	
	;**************************
	;F) check host adress
	;should be the own LHA
	;**************************
	ld	zl,y
	lds	zh,Shost_adr
	cp	zl,zh
	breq	rfm12_050runALL
	jmp	rfm12_050runKILL	;not ok...
	;**************************
	;J) all bytes are received
	;now calculate checksum
	;**************************
rfm12_050runALL:
	call	rfm12_rxCHKsum
	lds	zl,rxCKsumREC
	lds	zh,rxCKsumBUILT
	cp	zh,zl
	breq	rfm12_050runQUAL		;ok, now check quality
	jmp	rfm12_050runKILL		;not ok...
	;**************************
	;K) calculate the quality
	;factor in 1...100%
	;needs time !
	;min.90cycles /max.177 cycles
	;equal: max.17s (11MHz fclk)
	;**************************
rfm12_050runQUAL:
	;.........................
	;data good
	;.........................
	cbi	portb,rfmNET				;led on
	;	
	call	rfm12_Qfactor
	;***************************
	;L) store hosts to table
	;***************************
	call	scan_000HOtable
	;***************************
	;M) prepare to scan the
	;next host...
	;**************************
	lds	r17,HscanCNT				;host counter - perhaps increased by tx layer
    lds r16,Thost_adr               ;scan adress...
    cp  r17,r16
    breq  rfm12_050runB             ;still not increased..
    jmp _valRX_DATAb                ;host answer received - scan next host

    ;not increased
rfm12_050runB:
	cpi	r17,255
	breq	rfm12_050runNXTHOSTEXIT		;all hosts are scanned & exit
	inc	r17
	sts	HscanCNT,r17
	jmp _valRX_DATAb                    ;host answer received - scan next host



	;...........................
	;ALL DONE !
	;clean up the host-table
	;...........................
rfm12_050runNXTHOSTEXIT:
	;..........................
	;start garbage collector
	;..........................
	call	scan_100HOtable			    ;start garbage collector 
	;........................
	;prepare for exit
	;........................
	;--------------------
	;a) display ready msg
	;--------------------
   ;b)network job ready display								
	call    _SYSmsgF
	;display header & rectangles
	call	_sys03IF

 	jmp _valRX_DATAc                    ;all hosts are scanned - enable all...

	;**************************
	;N) no/too little data rec.
	;repeat scan if necesarry
	;or scan the next host (M) 
	;************************** 
rfm12_050runKILL:
	jmp _errRX_DATAb                     ;tx repeat available - it's the decision of the tx layer !










;.................................................................................................................
;51= RECEIVE THE (REQUESTED) TRANSFER-INSTRUCTION - ACKNOWLEDGE AFTER TRANSMITTING ALIVE/KILL/LHA-CHANGE MESSAGE
;<rfm12-instr.><source-adr.><target-adr.><ck-sum>
;.................................................................................................................
;jmp
rfm12_052runX:
	jmp	rfm12_052run

rfm12_051runKILLz:
	jmp	rfm12_051runKILL

rfm12_051run:
	cpi	r16,51
	brne	rfm12_052runX		;>2 check next modul
    ;check requested
;	lds	r16,STATEreq            ;check requested state !!!!
;	cpi	r16,51
;	brne	rfm12_051runKILLz

	;.........................
	;check target (scan)adress
	;.........................
	clr	zh
	ldi	zl,2	
	subi	zl,low(-rfm_buffer)
	sbci	zh,high(-rfm_buffer)
	ld	r16,z
	lds	r17,Shost_adr
	cp	r16,r17
	brne	rfm12_051runKILLz			;adress not equal !
	;.........................
	;proof check sum
	;.........................
	call	rfm12_rxCHKsum
	lds	zl,rxCKsumREC
	lds	zh,rxCKsumBUILT
	cp	zh,zl
	breq	rfm12_051runQUAL		;ok, now check quality
	jmp	rfm12_051runKILL		;not ok...
	;**************************
	;K) calculate the quality
	;factor in 1...100%
	;needs time !
	;min.90cycles /max.177 cycles
	;equal: max.17s (11MHz fclk)
	;**************************
rfm12_051runQUAL:
	;.........................
	;data good
	;.........................
	cbi	portb,rfmNET				;led on
	;
	call	rfm12_Qfactor
	;***************************
	;M) prepare to scan the
	;next host...
	;**************************
	lds	r17,HscanCNT				;host counter - perhaps increased by tx layer
    lds r16,Thost_adr               ;scan adress...
    cp  r17,r16
    breq  rfm12_051runB             ;still not increased..
    jmp _valRX_DATAb                ;host answer received - scan next host
    ;not increased
rfm12_051runB:
	cpi	r17,255
	breq	rfm12_051runNXTHOSTEXIT		;all hosts are scanned & exit
	inc	r17
	sts	HscanCNT,r17
	jmp _valRX_DATAb                    ;host answer received - scan next host

	;...........................
	;ALL DONE !
	;clean up the host-table
	;...........................
rfm12_051runNXTHOSTEXIT:
 	jmp _valRX_DATAc                    ;all hosts are scanned - enable all...

	;**************************
	;N) no/too little data rec.
	;repeat scan if necesarry
	;or scan the next host (M) 
	;************************** 
rfm12_051runKILL:
	jmp _errRX_DATAb                     ;tx repeat available - it's the decision of the tx layer !





	





;.................................................................................................................
;52/53/54= RECEIVE THE (REQUESTED) ACKNOWLEDGE TRANSFER-INSTRUCTION(2/4/5/6) - ,SIMPLE' ACKNOWLEDGE AFTER TRANSMITTING
;TO A DIRECT HOST/MESHED HOST - ALSO DEFINED AS AN ,EMBEDDED' ACKNOWLEDGE 
;<rfm12-instr.><source-adr.><Dtarget-adr.><Mtarget-adr><ck-sum>
;.................................................................................................................
;jmp
rfm12_053runX:
	jmp	rfm12_053run
rfm12_052run:
	cpi	r16,52
	brsh	rfm12_052runZ	
	brne	rfm12_053runX		;<52...
rfm12_052runZ:
	cpi	r16,55
	brlo	rfm12_052runYgo		;52..54 OK
	jmp	rfm12_053runX		;>54...
rfm12_052runYgo:
	;.........................
	;receive acknowledege from
	;the direct-target host
	;=> compare 
	;source | Dtarget
	;mode:52
	;... a simple ack!
	;.........................
;	lds	r16,STATEreq            ;check requested state !!!!
	cpi	r16,52
	brne	rfm12_052runREADYb	
	clr	zh
	ldi	zl,2	
	subi	zl,low(-rfm_buffer)
	sbci	zh,high(-rfm_buffer)
	ld	r16,z
	lds	r17,Shost_adr
	cp	r16,r17
	brne	rfm12_052runKILLz			;adress not equal !
	jmp	rfm12_052runREADYck			;equal !

;jmp
rfm12_052runKILLz:
	jmp	rfm12_052runKILL


	;.........................
	;receive acknowledege from
	;the meshed-target host
	;=> compare 
	;source | Dtarget
	;mode:53
	;transmit in order mode:54
	;.........................
rfm12_052runREADYb:
	cpi	r16,53
	brne	rfm12_052runREADYc
	clr	zh
	ldi	zl,2	
	subi	zl,low(-rfm_buffer)
	sbci	zh,high(-rfm_buffer)
	ld	r16,z
	lds	r17,Shost_adr
	cp	r16,r17
	brne	rfm12_052runKILLz			;adress not equal !
	jmp	rfm12_052runREADYck			;equal !

	;.........................
	;receive acknowledege from
	;the direct-target host
	;=> compare 
	;source | Mtarget
	;mode:54
	;.........................
rfm12_052runREADYc:
	cpi	r16,54
	brne	rfm12_052runKILLz
	clr	zh
	ldi	zl,3	
	subi	zl,low(-rfm_buffer)
	sbci	zh,high(-rfm_buffer)
	ld	r16,z
	lds	r17,Shost_adr
	cp	r16,r17
	brne	rfm12_052runKILL			;adress not equal !
	jmp	rfm12_052runREADYck			;equal !

	;.........................
	;proof check sum
	;.........................
rfm12_052runREADYck:
	call	rfm12_rxCHKsum
	lds	zl,rxCKsumREC
	lds	zh,rxCKsumBUILT
	cp	zh,zl
	breq	rfm12_052runQUAL		;ok, now check quality
	jmp	rfm12_052runKILL		;not ok...
	;**************************
	;K) calculate the quality
	;factor in 1...100%
	;needs time !
	;min.90cycles /max.177 cycles
	;equal: max.17s (11MHz fclk)
	;**************************
rfm12_052runQUAL:
	;.........................
	;data good
	;.........................
	cbi	portb,rfmNET				;led on
	;
;	call	rfm12_Qfactor
	;***************************
	;NEW
	;only mode:53
	;further acknowledge to the
	;primary source-adress 
	;mode:54
	;... a kind of double ack !
	;***************************
	lds	r16,STATErec
	cpi	r16,53
	brne	rfm12_052runREADYckX
    ;...........................
    ;state = 53
    ;...........................
	ldi	r16,54
    jmp _valRX_DATA                     ;tx requested...
    ;...........................
	;state = 52/54
    ;...........................
rfm12_052runREADYckX:
 	;ready 
	jmp _valRX_DATAc                    ;- enable all...

;------------------------------
;perhaps repeated transmitting?
;------------------------------
rfm12_052runKILL:
	;........................
	;prepare for repeating
	;........................
	jmp _errRX_DATAb                     ;tx repeat available - it's the decision of the tx layer !




;.................................................................................................................
;55= RECEIVE THE (REQUESTED) ACKNOWLEDGE IN ORDER OF TRANSFER-INSTRUCTION(10)
;TO A DIRECT HOST - ALSO DEFINED AS AN ,REMOTE' ACKNOWLEDGE 
;<rfm12-instr.><source-adr.><Dtarget-adr.><REMtype><ck-sum>
;NOTE: activ remote connection uses channel 15
;.................................................................................................................
;jmp
rfm12_053runKILLx:
    jmp rfm12_053runKILL
rfm12_054runX:
	jmp	rfm12_054run
rfm12_053run:
	cpi	r16,55
	brne	rfm12_054runX		;<55...
	;.........................
	;receive acknowledege from
	;the direct-target host
	;=> compare 
	;source | Dtarget
	;.........................
	clr	zh
	ldi	zl,2	
	subi	zl,low(-rfm_buffer)
	sbci	zh,high(-rfm_buffer)
	ld	r16,z
	lds	r17,Shost_adr
	cp	r16,r17
	brne	rfm12_053runKILLx			;adress not equal !
	;.........................
	;proof check sum
	;.........................
	call	rfm12_rxCHKsum
	lds	zl,rxCKsumREC
	lds	zh,rxCKsumBUILT
	cp	zh,zl
	breq	rfm12_053runTYPE		;ok, now check quality
	jmp	rfm12_053runKILL		;not ok...
    ;.........................
    ;proof requested REMtype
    ;........................
rfm12_053runTYPE: 
	clr	zh
	ldi	zl,3	
	subi	zl,low(-rfm_buffer)
	sbci	zh,high(-rfm_buffer)
	ld	r16,z
    lds zl,REMtype
    cp  r16,zl
    breq  rfm12_053runOK
    ;not equal
    rjmp   rfm12_053runKILL           

	;**************************
    ;FRAME OK
	;**************************
rfm12_053runOK:
    cpi zl,REMinstr
    brne   rfm12_053runOKDOKI
    lds r16,REMif
    sbrc    r16,REMstate
    rjmp    rfm12_053runOKmstr      ;master 



    ;***************************
    ;slave & instr. = remOFF
    ;***************************
    ;led on
	cbi	portb,rfmNET
rfm12_053runOKwt:
    sbis    pinb,_ifSEL
	rjmp	rfm12_053runOKwt			;spi still busy with SPI
    ;block if_sel & rfm_master
    call    _SPImasterRFMINIT
    ;------------------------------------
    ;now client changes to default channel
	;rx/tx-channel=10
	ldi	r16,10
	sts	RXch,r16
	sts	TXch,r16
    ;------------------------------------			
	;enable receive-mode
	call	rfm12_RXinit
    ;now enable spi slave mode
    ;perhaps system communication
    ;will goon..
    call    _SPIslaveINIT   ;including _ifSEL=input
    ;slave
    clr r16
    sts REMif,r16
    ;b)remote job ready display								
	call    _SYSmsgL
	;display header & rectangles
	call	_sys03IF
    ;set temp.transmit state=0
    clr r16
   	sts	STATEsendTE,r16
  	jmp _valRX_DATAc                    ;- enable all...


    ;*******************************
    ;master & instr. = remote activ
    ;*******************************
rfm12_053runOKmstr:
    ;led on
	cbi	portb,rfmNET
 rfm12_053runOKwtB:
    sbis    pinb,_ifSEL
	rjmp	rfm12_053runOKwtB			;spi still busy with SPI
    ;block if_sel & rfm_master
    call    _SPImasterRFMINIT
    ;------------------------------------
    ;now master changes to remote channel
	;rx/tx-channel
	ldi	r16,15
	sts	RXch,r16
	sts	TXch,r16
    ;------------------------------------
	;enable receive-mode
	call	rfm12_RXinit
    ;now enable spi slave mode
    ;perhaps system communication
    ;will goon..
    call    _SPIslaveINIT   ;including _ifSEL=input
    ;slave
  	jmp _valRX_DATAc                    ;- enable all...

 
	;.........................
	;data good
	;.........................
rfm12_053runOKDOKI:
	cbi	portb,rfmNET				;led on
	jmp _valRX_DATAc                    ;- enable all...

;------------------------------
;perhaps repeated transmitting?
;------------------------------
rfm12_053runKILL:
	;........................
	;prepare for repeating
	;........................
	jmp _errRX_DATAb                     ;tx repeat available - it's the decision of the tx layer !










;.................................................................................................................
;EXIT
;.................................................................................................................
rfm12_054run:
  ;invalid
  jmp _errRX


