;----------------------------------------------------------------------------------------------------------------
;Transmit primary & secondary messages
;A) primary:
;1=  scan (requests:50)
;2=  embedded data to direct host (requests:52)
;4=  embedded data to meshed host - first step to the direct host (requests:54)
;5=  embedded data from the direct to the meshed-host (requests:53)
;6=  embedded data to all available direct hosts (requests:52)
;7=  alive message to the complete network (requests:51)
;8=  kill message to the complete network (requests:51)
;9=  lha change message to all direct hosts as a kind of broadcast (request:51)
;10= remote access message to a direct host (request:55)
;NOTE: database for primary instruction is always the <RFMtxbuffer>
;
;B)Secondary:
;50= available direct-host (incl.quality) to the calling host (in order of a received:1)
;51= simple acknowledge to the calling host (in order of a received:7/8)
;52= simple acknowledge to the calling host (in order of a received:2/6)
;53= mesh-acknowledge to the calling direct-host (in order of a received:5)
;54= mesh-acknowledge from the direct to the calling host (in order of a received:53)
;55= remote acknowledge to the direct host (in order of a received:10) 
;56= reset acknowledge to the direct host (in order of a received:11) 

;----------------------------------------------------------------------------------------------------------------
;NOTE:
;The actual version is now rebuild to a dynamic structure. The following flags are used:
;rfm-communication state flags
;COMTXstate		    =	0xe96		;com state
;RFMbufFILL		    =	0			;rfm buffer filled with data 
;RFMtxSTART		    =	1			;rfm buffer transmitting started
;RFMtxRUN			=	2			;rfm buffer transmitting runs
;RFMtxSTOP		    =	3			;tx finished - check further conditions...
;
;.................................................................................................................
;1 = TRANSMIT MODE ,PING THE NETWORK'
;- NO SYSTEM DEPENDENT INSTRUCTION
;- RUNS IN ORDER OF A POR
;- REQUESTS ACKNOWLEDGED DATA FROM TARGET HOST (MODE: 50)
;NEW: rebuild for dynamic process !
;.................................................................................................................. 
rfm12_110runX:
	jmp	rfm12_110run

rfm12_100runBACK:
    ret

;+++++++++++++++++++++++++++++++++++++++++++++++++++
;STATEsend >0 // check if transmission is runnin'
;OR rfm / spi is busy...
;+++++++++++++++++++++++++++++++++++++++++++++++++++
rfm12_100run:
    ;NEW
	lds	r16,COMTXstate
    sbrc   r16,RFMbufFILL
    rjmp    rfm12_100runOK              ;tx running
 	;---------------------
	;check spi activity
	;---------------------
    sbis    pinb,_ifSEL
	rjmp	rfm12_100runBACK			;spi still busy with SPI
    ;---------------------
    ;disable Interrupts
    ;---------------------
    cli 
    ;---------------------
    ;free = spi master
    ;---------------------
    call    _SPImasterRFMINITb          ;master & block bus
    ;---------------------
    ;receiver off
    ;---------------------
    call    rfm12_RXoff
    ;---------------------
    ;spi slave
    ;---------------------
    call    _SPIslaveINITb
    ;--------------------------
    ;clear old receive request
    ;--------------------------
    lds	r16,TRANS_state    
    cbr r16,1<<nRF_RXreq
 	sts	TRANS_state,r16
    ;---------------------------
    ;clear int0 request
    ;---------------------------
    sbi     eifr,intf0 
    ;---------------------------
    ;-now receiver is off
    ;-spi is slave
    ;---------------------------
    ;NOW load data to transmit
    ;after that start tranmitting
    ;---------------------------
    sei


;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;STATEsend >0  // transmit data
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

rfm12_100runOK:
    ;.....................    
    ;goon tx-job
    ;.....................
  
	lds	r16,STATEsend		
	cpi	r16,1
	brne	rfm12_110runX
	;...............................
	;check tx-state
	;...............................
	lds	r16,COMTXstate
	sbrs	r16,RFMbufFILL				
	rjmp	rfm12_100runFILL			;0=fill rfm buffer first...
	sbrs	r16,RFMtxSTART			
	rjmp	rfm12_100runSTART			;0=rfm buffer filled -> transmit startsequence
	sbrs	r16,RFMtxRUN				
	rjmp	rfm12_100runACTIV			;0=nIRQ-svr based transmitting -> nothing else to do now!
	sbrs	r16,RFMtxSTOP				
	rjmp	rfm12_100runACTIV			
	jmp	rfm12_100runLOOPex			;1=nIRQ-svr already transmitted the tx-off instruction -> check further conditions


;......................................
;A) copy data to rfm_buffer
;NOTE: receiver is off
;......................................
rfm12_100runFILL:
	sbr	r16,1<<RFMbufFILL
	sts	COMTXstate,r16
	;clear checksum
	ldi	r17,0xaa
	sts	txCKsum,r17
	;adresspointer rfm-buffer start
	ldi	yl,low(rfm_buffer)
	ldi	yh,high(rfm_buffer)
	;*******************
	;set values
	;A) rfm12 instruction
	;*******************
	ldi	r18,1
	sts	rfm12_instr,r18		;mode instruction =1 (scan the network)
	st	y+,r18			;store to tx-buffer
	eor	r17,r18			;exclusive or (ck-sum)
	;*******************
	;B) load source adress 
	;*******************
   	lds	r22,Shost_adr
	st	y+,r22			;store to tx-buffer
	eor	r17,r22			;exclusive or (ck-sum)
	;*******************
	;C) target adress
	;*******************
    lds r19,HscanCNT
	sts	Thost_adr,r19   ;will be check by rx layer if equal with the scanned !!!!
	st	y+,r19			;store to tx-buffer
	eor	r17,r19
	;*******************
	;D) store ck-sum
	;*******************
	st	y,r17			;store to tx-buffer
	sts	txCKsum,r17
	ret




;............................................
;B) init TX
;including preamble & sync !
;............................................
rfm12_100runSTART:
    ;spi free ?
    sbis    pinb,_ifSEL
	rjmp	rfm12_100runSTARTbk			;spi still busy with SPI
    ;free
 	sbr	r16,1<<RFMtxSTART
	sts	COMTXstate,r16
	call	rfm12_TXinit
	;.......................................
	;start is done !
	;-> tells th nIRQ-svr to do the further
	;   tx-job !
	;.......................................
	lds	r16,COMTXstate
	sbr	r16,1<<RFMtxRUN
	sts	COMTXstate,r16
 	;***************************
	;I) write data to buffer & tx
	;***************************
	;lade MSB
	ldi	yl,low(rfm_buffer)
	ldi	yh,high(rfm_buffer)
    ;manchester
    call    _TXmanSUB
 
	ldi	r17,4	;4
	sts	FRAME_siz,r17
    ldi r17,0
	sts	TRANSbyte,r17
	call rfm12_initSPI
    ;startsequence transmitted 
    jmp rfm12_100runOK
	
    ;*******************
    ;spi busy
    ;*******************
rfm12_100runSTARTbk:
	ret	
;............................................
;C) further tx will be done by nIRQfifo svr
;............................................
rfm12_100runACTIV:
	    jmp rfm12_100runOK


;.............................................
;D) tx is ready - check conditions 
;.............................................
rfm12_100runLOOPex:
	;.............................
	;check repeated tx
	;.............................
	lds	r17,TXrepCNT
	cpi	r17,TXrep					;max repeats
	breq	rfm12_100runNXTHOST
	inc	r17
	sts	TXrepCNT,r17
    ;.....................................
    ;rfm as a receiver & spi to slave
    ;.....................................
     call    rfm12_TXOFF
   ;.....................................
    ;data is sent to the scanning host
    ;now wait for the requested answer: 50
	;.....................................
	ldi	r16,50
    jmp _errTX_REQ
    
	;.....................................
    ;max scan repeats reached...
	;check for limit...
    ;OR scan next host after the
    ;last wait state for requested answer:50
	;......................................
rfm12_100runNXTHOST:
	clr	r17
	sts	TXrepCNT,r17
	lds	r17,HscanCNT				;host counter
	cpi	r17,255				        ;
	breq	rfm12_100runEX  		;all hosts are scanned & exit
	inc	r17
	sts	HscanCNT,r17                ;for the next tx - job !
   ;.....................................
    ;rfm as a receiver & spi to slave
    ;.....................................
    call    rfm12_TXOFF
    ;.....................................
    ;data is sent to the scanning host
    ;now wait for the requested answer: 50
	;.....................................
	ldi	r16,50
    jmp _errTX_REQ
  

	;........................
	;prepare for exit
	;........................
rfm12_100runEX:
    call    rfm12_TXOFFII

    ;.................
    ;host transfer text
    call    _SYSmsgI
    call	_sys03IF
    ;.................


    ;..........................
	;start garbage collector
	;..........................
	call	scan_100HOtable			;TEST 100HOtable  start garbage collector 
 
    ;....................................
    ;transfer host table to concept...
    ;- 10bytes payload
    ;- mi (mi3/mi6=1)
    ;- framesize=17
    ;- type=8
    ;....................................
    call    _SPImasterINIT
    call    scan_190HOtable

    ;.................
    ;network job ready 
	call    _SYSmsgF
	call	_sys03IF
    ;.................
    call    _SPImasterRFMINIT
	;enable receive-mode
	call	rfm12_RXinit
    ;enable slave mode
    call    _SPIslaveINIT   ;including _ifSEL=input
    jmp _exTX


 







;.................................................................................................................
;2 = TRANSMIT MODE ,EMBEDDED DATA TO A DIRECT HOST'
;- EMBEDDED DATA IS PRESENT IN comNETBUF STORED BY SPI !
;- BUILD COMPLETE FRAME, INCLUDING CHECK-SUM
;- TRANSMIT DATA 
;REQUESTED ACKNOWLEDGE INSTRUCTION = 52
;format: <mode-instr.(2)><rfm-source-adr><rfm-target-adr>[EMB.DATA]<ck-sum>
;frame-size: (4xrfm12)+ EMB.DATA-size 
;NEW: rebuild for dynamic process -NOW use STACK!
;tested:190807
;.................................................................................................................. 
rfm12_111runX:
	jmp	rfm12_111run


rfm12_110run:
	lds	r16,STATEsend		
	cpi	r16,2
	brne	rfm12_111runX
 	;...............................
	;check tx-state
	;...............................
	lds	r16,COMTXstate
	sbrs	r16,RFMbufFILL				
	rjmp	rfm12_110runFILL			;0=fill rfm buffer first...
	sbrs	r16,RFMtxSTART			
	rjmp	rfm12_110runSTART			;0=rfm buffer filled -> transmit startsequence
	sbrs	r16,RFMtxRUN				
	rjmp	rfm12_110runACTIV			;0=nIRQ-svr based transmitting -> nothing else to do now!
	sbrs	r16,RFMtxSTOP				
	rjmp	rfm12_110runACTIV			
	jmp	rfm12_110runLOOPex			;1=nIRQ-svr already transmitted the tx-off instruction -> check further conditions


;......................................
;A) copy data to rfm_buffer
;NOTE: receiver is off
;......................................
rfm12_110runFILL:
	sbr	r16,1<<RFMbufFILL
	sts	COMTXstate,r16
	;load adresses
	ldi	yl,low(rfm_buffer)
	ldi	yh,high(rfm_buffer)
	;clear checksum
	ldi	r17,0xaa
	sts	txCKsum,r17
	;*******************
	;set values
	;A) rfm12 instruction
	;*******************
	ldi	r18,2
	sts	rfm12_instr,r18	;mode instruction =1 (scan the network)
	st	y+,r18			;store to tx-buffer
	eor	r17,r18			;exclusive or (ck-sum)
	;*******************
	;B) load source adress 
	;*******************
	lds	r22,Shost_adr
	st	y+,r22			;store to tx-buffer
	eor	r17,r22			;exclusive or (ck-sum)
	;*******************
	;C) target adress
	;*******************
	;adresspointer communicationbuffer
    ldi zl,low(RFMtxbuffer)
    ldi zh,high(RFMtxbuffer)
	;to target-host
    adiw    zh:zl,1
	ld	r16,z			;load targetadress
	sts	Thost_adr,r16		;store to var 
	st	y+,r16
	eor	r17,r16			;xor check-sum
	;***********************
	;D) load emb. frame-size
	;& calculate complete size
	;***********************
	;adresspointer communicationbuffer
    ldi zl,low(RFMtxbuffer)
    ldi zh,high(RFMtxbuffer)
	;to embedded frame-size
    adiw    zh:zl,5
	ld	r16,z			;load embedded framesize
	ldi	r18,4
	add	r16,r18
	sts	FRAME_siz,r16		;store complete size to var 
	;**********************
	;E) set byte counter
	;there're already 
	;3 bytes stored in
	;rfm_buffer !
	;**********************
	ldi	r16,3
	sts	TRANSbyte,r16
	;adresspointer communicationbuffer
    ldi zl,low(RFMtxbuffer)
    ldi zh,high(RFMtxbuffer)
	lds	r20,FRAME_siz
	subi	r20,1			;stop storing after the last embedded byte
	;***********************
	;F) store embedded data
	;to rfm_buffer
	;***********************
rfm12_110runLOOPb:
	ld	r18,z+
	st	y+,r18
	eor	r17,r18			;xor check-sum
	inc	r16				;byte counter +1
	cp	r16,r20
	brsh	rfm12_110runLOOPready
	jmp	rfm12_110runLOOPb	;store in loop
rfm12_110runLOOPready:
	st	y+,r17			;store ck-sum
	;buffer filled & back
	ret

;............................................
;B) init TX to rfm & TX to RS485
;including preamble & sync !
;............................................
rfm12_110runSTART:
    sbis    pinb,_ifSEL
	rjmp	rfm12_110runSTARTbk			;spi still busy with SPI
   	;free...
	sbr	r16,1<<RFMtxSTART
	sts	COMTXstate,r16
	call	rfm12_TXinit
	;-------------------------
	;set transfer to 
	;rs485 bus...
	;check by 220s svr
	;start: if rs485 not busy
	;exit:  after tx the frame
	;-------------------------
	lds	r17,RS485state
	sbr	r17,1<<RS485to
	sts	RS485state,r17
	;.......................................
	;start is done !
	;-> tells th nIRQ-svr to do the further
	;   tx-job !
	;.......................................
	lds	r16,COMTXstate
	sbr	r16,1<<RFMtxRUN
	sts	COMTXstate,r16
 	;***************************
	;I) write data to buffer & tx
	;***************************
	;lade MSB
	ldi	yl,low(rfm_buffer)
	ldi	yh,high(rfm_buffer)
    ;manchester
    call    _TXmanSUB
    ;NOTE
    ldi r16,0
	sts	TRANSbyte,r16
	call rfm12_initSPI
	;startsequence transmitted & back
    jmp rfm12_110run        ;loop ...
    ;*******************
	;spi busy - ret
    ;*******************
rfm12_110runSTARTbk:
	ret	

;............................................
;C) further tx will be done by nIRQfifo svr
;............................................
rfm12_110runACTIV:
    jmp rfm12_110run        ;loop ...

;.............................................
;D) tx is ready - check conditions 
;.............................................
rfm12_110runLOOPex:
    ;tx-off & receive-mode
    call    rfm12_TXOFF
    ;check tx-repeat
	lds	r17,TXrepCNT  
	cpi	r17,TXrep					;max repeats
	breq	rfm12_110runKILL
    inc r17
    sts TXrepCNT,r17
     ;.....................................
    ;data is sent to the called host
    ;now wait for the requested answer: 52
	;.....................................
	ldi	r16,52
    jmp _errTX_REQ
  	
	;no acknowledge received & no tx repeat !...
rfm12_110runKILL:
    clr r17
    sts TXrepCNT,r17
    jmp _exTX







;.................................................................................................................
;4 = TRANSMIT MODE ,EMBEDDED DATA TO A MESHED HOST - FIRST TRANSMITTING TO THE DIRECT HOST
;- EMBEDDED DATA IS PRESENT IN comNETBUF STORED BY SPI !
;- BUILD COMPLETE FRAME, INCLUDING CHECK-SUM
;- TRANSMIT DATA 
;REQUESTED ACKNOWLEDGE INSTRUCTION = 54
;format: <rfm12-instr><sou-adr><D-tar-adr><M-tar-adr>[EMB-DATA]<ck-sum>
;frame-size: (5xrfm12)+ EMB.DATA-size 
;NEW: rebuild for dynamic process !
;.................................................................................................................. 

rfm12_112runX:
	jmp	rfm12_112run
rfm12_111runKILLx:
	jmp	rfm12_111runKILL

rfm12_111run:
	lds	r16,STATEsend	
	cpi	r16,4
   	brne	rfm12_112runX
    ;...............................
	;check tx-state
	;...............................
	lds	r16,COMTXstate
	sbrs	r16,RFMbufFILL				
	rjmp	rfm12_111runFILL			;0=fill rfm buffer first...
	sbrs	r16,RFMtxSTART			
	rjmp	rfm12_111runSTART			;0=rfm buffer filled -> transmit startsequence
	sbrs	r16,RFMtxRUN				
	rjmp	rfm12_111runACTIV			;0=nIRQ-svr based transmitting -> nothing else to do now!
	sbrs	r16,RFMtxSTOP				
	rjmp	rfm12_111runACTIV			
	jmp	rfm12_111runLOOPex			;1=nIRQ-svr already transmitted the tx-off instruction -> check further conditions


;......................................
;A) copy data to rfm_buffer
;NOTE: receiver is off
;......................................
rfm12_111runFILL:
	sbr	r16,1<<RFMbufFILL
	sts	COMTXstate,r16
	;load adresses
	ldi	yl,low(rfm_buffer)
	ldi	yh,high(rfm_buffer)
	;clear checksum
	ldi	r17,0xaa
	sts	txCKsum,r17
	;*******************
	;set values
	;A) rfm12 instruction
	;*******************
	ldi	r18,4
	sts	rfm12_instr,r18	;mode instruction =1 (scan the network)
	st	y+,r18			;store to tx-buffer
	eor	r17,r18			;exclusive or (ck-sum)
	;*******************
	;B) load source adress 
	;*******************
	lds	r22,Shost_adr
	st	y+,r22			;store to tx-buffer
	eor	r17,r22			;exclusive or (ck-sum)
	;*********************
	;C) store Dtar-adress
	;*********************
	lds	r22,DIRhost
	st	y+,r22			;store to tx-buffer
	eor	r17,r22			;exclusive or (ck-sum)
	;*********************
	;D) store Mtarget adress
	;*********************
	lds	r22,T2host_adr
	st	y+,r22			;store to tx-buffer
	eor	r17,r22			;exclusive or (ck-sum)

	;***********************
	;E) load emb. frame-size
	;& calculate complete size
	;***********************
	;adresspointer communicationbuffer
    ldi zl,low(RFMtxbuffer)
    ldi zh,high(RFMtxbuffer)
	;to embedded frame-size
    adiw    zh:zl,5
	ld	r16,z			;load embedded framesize
	ldi	r18,5
	add	r16,r18
	sts	FRAME_siz,r16		;store complete size to var 
	;**********************
	;E) set byte counter
	;there're already 
	;3 bytes stored in
	;rfm_buffer !
	;**********************
	ldi	r16,4
	sts	TRANSbyte,r16
 	;adresspointer communicationbuffer
    ldi zl,low(RFMtxbuffer)
    ldi zh,high(RFMtxbuffer)

    lds	r20,FRAME_siz
	subi	r20,1			;stop storing after the last embedded byte
	;***********************
	;F) store embedded data
	;to rfm_buffer
	;***********************
rfm12_111runLOOPb:
	ld	r18,z+
	st	y+,r18
	eor	r17,r18			;xor check-sum
	inc	r16				;byte counter +1
	cp	r16,r20
	brsh	rfm12_111runLOOPready
	jmp	rfm12_111runLOOPb	;store in loop
rfm12_111runLOOPready:
	st	y,r17			;store ck-sum
	ret




;............................................
;B) init TX
;including preamble & sync !
;............................................
rfm12_111runSTART:
    sbis    pinb,_ifSEL
	rjmp	rfm12_111runSTARTbk			;spi still busy with SPI
  	;free...
	sbr	r16,1<<RFMtxSTART
	sts	COMTXstate,r16
	call	rfm12_TXinit
	;.......................................
	;start is done !
	;-> tells th nIRQ-svr to do the further
	;   tx-job !
	;.......................................
	lds	r16,COMTXstate
	sbr	r16,1<<RFMtxRUN
	sts	COMTXstate,r16

 	;***************************
	;I) write data to buffer & tx
	;***************************
	;lade MSB
	ldi	yl,low(rfm_buffer)
	ldi	yh,high(rfm_buffer)
   ;manchester
    call    _TXmanSUB
 
    ldi r16,0
	sts	TRANSbyte,r16
	call rfm12_initSPI
	;startsequence transmitted
    jmp rfm12_111run
    ;*****************
    ;spi busy
    ;*****************
rfm12_111runSTARTbk:
	ret
;............................................
;C) further tx will be done by nIRQfifo svr
;............................................
rfm12_111runACTIV:
	   jmp rfm12_111run
;.............................................
;D) tx is ready - check conditions 
;.............................................
rfm12_111runLOOPex:
 ;tx-off & receive-mode
    call    rfm12_TXOFF
    ;check tx-repeat
	lds	r17,TXrepCNT
    inc r17
	cpi	r17,TXrep					;max repeats
	breq	rfm12_111runKILL
    sts TXrepCNT,r17
    ;repeat !
    ;.....................................
    ;data is sent to the called host
    ;now wait for the requested answer: 54
	;.....................................
	ldi	r16,54
    jmp _errTX_REQ
	
	;no acknowledge received...
rfm12_111runKILL:
    jmp _exTX





;.................................................................................................................
;5 = TRANSMIT MODE ,EMBEDDED DATA FROM THE DIRECT TO THE MESHED HOST - IN ORDER OF RECEIVE:4
;- EMBEDDED DATA IS PRESENT IN RFM_BUFFER 
;- TRANSMIT DATA AS RECEIVED
;REQUESTED ACKNOWLEDGE INSTRUCTION = 53
;format: <rfm12-instr><sou-adr><D-tar-adr><M-tar-adr>[EMB-DATA]<ck-sum>
;frame-size: (5xrfm12)+ EMB.DATA-size 
;NEW: rebuild for dynamic process !
;.................................................................................................................. 


rfm12_113runX:
	jmp	rfm12_113run
rfm12_112runKILLx:
	jmp	rfm12_112runKILL

rfm12_112run:
    lds  r16,STATEsend
	cpi	r16,5
	brne	rfm12_113runX
 	;...............................
	;check tx-state
	;...............................
	lds	r16,COMTXstate
	sbrs	r16,RFMbufFILL				
	rjmp	rfm12_112runFILL			;0=fill rfm buffer first...
	sbrs	r16,RFMtxSTART			
	rjmp	rfm12_112runSTART			;0=rfm buffer filled -> transmit startsequence
	sbrs	r16,RFMtxRUN				
	rjmp	rfm12_112runACTIV			;0=nIRQ-svr based transmitting -> nothing else to do now!
	sbrs	r16,RFMtxSTOP				
	rjmp	rfm12_112runACTIV			
	jmp	rfm12_112runLOOPex			;1=nIRQ-svr already transmitted the tx-off instruction -> check further conditions


;......................................
;A) copy data to rfm_buffer
;......................................
rfm12_112runFILL:
	sbr	r16,1<<RFMbufFILL
	sts	COMTXstate,r16
	ret


;............................................
;B) init TX
;including preamble & sync !
;............................................
rfm12_112runSTART:
    sbis    pinb,_ifSEL
	rjmp	rfm12_112runSTARTbk			;spi still busy with SPI
 	;free...
	sbr	r16,1<<RFMtxSTART
	sts	COMTXstate,r16
	call	rfm12_TXinit
	;.......................................
	;start is done !
	;-> tells th nIRQ-svr to do the further
	;   tx-job !
	;.......................................
	lds	r16,COMTXstate
	sbr	r16,1<<RFMtxRUN
	sts	COMTXstate,r16
 	;***************************
	;I) write data to buffer & tx
	;***************************
	;lade MSB
	ldi	yl,low(rfm_buffer)
	ldi	yh,high(rfm_buffer)
   ;manchester
    call    _TXmanSUB
  ldi r16,0
	sts	TRANSbyte,r16
	call rfm12_initSPI
	;startsequence transmitted
    jmp rfm12_112run
    ;******************
    ;spi bus busy
    ;******************
rfm12_112runSTARTbk:
	ret
;............................................
;C) further tx will be done by nIRQfifo svr
;............................................
rfm12_112runACTIV:
	jmp rfm12_112run

;.............................................
;D) tx is ready - check conditions 
;.............................................
rfm12_112runLOOPex:
     ;tx-off & receive-mode
    call    rfm12_TXOFF
    ;check tx-repeat
	lds	r17,TXrepCNT
    inc r17
	cpi	r17,TXrep					;max repeats
	breq	rfm12_112runKILL
    sts TXrepCNT,r17
    ;repeat !
    ;.....................................
    ;data is sent to the called host
    ;now wait for the requested answer: 53
	;.....................................
	ldi	r16,53
    jmp _errTX_REQ
	

	;............................
	;no acknowledge received...
	;............................
rfm12_112runKILL:
    jmp _exTX







;.................................................................................................................
;6 = TRANSMIT MODE ,EMBEDDED DATA TO A DIRECT HOSTS AS A BROADCAST MESSAGE'
;- EMBEDDED DATA IS PRESENT IN comNETBUF STORED BY SPI !
;- BUILD COMPLETE FRAME, INCLUDING CHECK-SUM
;- TRANSMIT DATA TO AVAILABLE DIRECT HOSTS IN SEQUENCE
;REQUESTED ACKNOWLEDGE INSTRUCTION = 52
;format: <mode-instr.(6)><rfm-source-adr><rfm-target-adr>[EMB.DATA]<ck-sum>
;frame-size: (4xrfm12)+ EMB.DATA-size 
;input: HOcnt (count of direct hosts)
;	   host_tableSHAD (the direct hosts are stored there temporary!)
;NEW: rebuild for dynamic process !
;.................................................................................................................. 
rfm12_117runX:
	jmp	rfm12_117run

rfm12_113run:
    lds r16,STATEsend
	cpi	r16,6
	brne	rfm12_117runX
	;...............................
	;check tx-state
	;...............................
	lds	r16,COMTXstate
	sbrs	r16,RFMbufFILL				
	rjmp	rfm12_113runFILL			;0=fill rfm buffer first...
	sbrs	r16,RFMtxSTART			
	rjmp	rfm12_113runSTART			;0=rfm buffer filled -> transmit startsequence
	sbrs	r16,RFMtxRUN				
	rjmp	rfm12_113runACTIV			;0=nIRQ-svr based transmitting -> nothing else to do now!
	sbrs	r16,RFMtxSTOP				
	rjmp	rfm12_113runACTIV			
	jmp	rfm12_113runLOOPex			;1=nIRQ-svr already transmitted the tx-off instruction -> check further conditions


;......................................
;A) copy data to rfm_buffer
;......................................
rfm12_113runFILL:
	sbr	r16,1<<RFMbufFILL
	sts	COMTXstate,r16	;check repeat counter
	;load adresses
	ldi	yl,low(rfm_buffer)
	ldi	yh,high(rfm_buffer)
	;clear checksum
	ldi	r17,0xaa
	sts	txCKsum,r17
	;*******************
	;set values
	;A) rfm12 instruction
	;*******************
	ldi	r18,6
	sts	rfm12_instr,r18	;mode instruction =6
	st	y+,r18			;store to tx-buffer
	eor	r17,r18			;exclusive or (ck-sum)
	;*******************
	;B) load source adress 
	;*******************
	lds	r22,Shost_adr
	st	y+,r22			;store to tx-buffer
	eor	r17,r22			;exclusive or (ck-sum)
	;*******************
	;C) target adress
	;*******************
	;adresspointer in table
	;[TE_garbage_seg]
	;to target-host
	clr	zh
	lds	zl,HOcnt	
	subi	zl,1			
	subi	zl,low(-TE_garbage_seg)
	sbci	zh,high(-TE_garbage_seg)
	ld	r16,z			;load targetadress
	sts	Thost_adr,r16		;store to var 
	st	y+,r16
	eor	r17,r16			;xor check-sum
	;***********************
	;D) load emb. frame-size
	;& calculate complete size
	;***********************
  	;adresspointer communicationbuffer
    ldi zl,low(RFMtxbuffer)
    ldi zh,high(RFMtxbuffer)
	;to embedded frame-size
    adiw    zh:zl,5
	ld	r16,z			;load embedded framesize
	ldi	r18,4
	add	r16,r18
	sts	FRAME_siz,r16		;store complete size to var 
	;**********************
	;E) set byte counter
	;there're already 
	;3 bytes stored in
	;rfm_buffer !
	;**********************
	ldi	r16,3
	sts	TRANSbyte,r16
 	;adresspointer communicationbuffer
    ldi zl,low(RFMtxbuffer)
    ldi zh,high(RFMtxbuffer)

   	lds	r20,FRAME_siz
	subi	r20,1			;stop storing after the last embedded byte
	;***********************
	;F) store embedded data
	;to rfm_buffer
	;***********************
rfm12_113runLOOPb:
	ld	r18,z+
	st	y+,r18
	eor	r17,r18			;xor check-sum
	inc	r16				;byte counter +1
	cp	r16,r20
	brsh	rfm12_113runLOOPready
	jmp	rfm12_113runLOOPb	;store in loop
rfm12_113runLOOPready:
	st	y,r17			;store ck-sum
	ret



;............................................
;B) init TX
;including preamble & sync !
;............................................
rfm12_113runSTART:
    sbis    pinb,_ifSEL
	rjmp	rfm12_113runSTARTbk			;spi still busy with SPI
 	;free...
	sbr	r16,1<<RFMtxSTART
	sts	COMTXstate,r16
	call	rfm12_TXinit
	;.......................................
	;start is done !
	;-> tells th nIRQ-svr to do the further
	;   tx-job !
	;.......................................
	lds	r16,COMTXstate
	sbr	r16,1<<RFMtxRUN
	sts	COMTXstate,r16
 	;***************************
	;I) write data to buffer & tx
	;***************************
	;lade MSB
	ldi	yl,low(rfm_buffer)
	ldi	yh,high(rfm_buffer)
   ;manchester
    call    _TXmanSUB
   ldi r16,0
	sts	TRANSbyte,r16
	call rfm12_initSPI
	;startsequence transmitted
    jmp rfm12_113run
    ;*******************
    ;spi busy
    ;*******************
rfm12_113runSTARTbk:
	ret
;............................................
;C) further tx will be done by nIRQfifo svr
;............................................
rfm12_113runACTIV:
	  jmp rfm12_113run

;.............................................
;D) tx is ready - check conditions 
;.............................................
rfm12_113runLOOPex:
    ;tx-off & receive-mode
    call    rfm12_TXOFF
    ;check tx-repeat
	lds	r17,TXrepCNT
    inc r17
	cpi	r17,TXrep					;max repeats
	breq	rfm12_113runKILL
    sts TXrepCNT,r17
    ;repeat !
    ;.....................................
    ;data is sent to the called host
    ;now wait for the requested answer: 52
	;.....................................
	ldi	r16,52
    jmp _errTX_REQ




	;............................
	;no acknowledge received...
	;now prepare for next host
	;transmitting...
	;............................
rfm12_113runKILL:
 	;set repeat
	clr	r16
	sts	TXrepCNT,r16
	;anymore hosts ?
	lds	r17,HOcnt				    ;<=1 exit !
	cpi	r17,2
	brsh	rfm12_113runNXThost		;next host....
	;no!
    jmp _exTX                       ;ready transmitted

	;..........................
	;set next hostpointer
	;..........................
rfm12_113runNXThost:
	dec	r17
	sts	HOcnt,r17
	;store rx-state=0
	clr	r16
	sts	STATErec,r16
	;clear framesize
	sts	FRAME_siz,r16		
	;clear byte-counter			
	sts	TRANSbyte,r16

	;clr requested(ack)
	sts	STATEreq,r16
	;clear rfm-states
    ;but buffer secure still activ !
    sbr r16,1<<BUFsecure  
	sts	COMTXstate,r16	
	ret








;..................................................................................................................
;7/8= TRANSMIT ALIVE MESSAGE (7) OR KILL MESSAGE
;the transmitting scan-frame-structure:
;<mode-instr.(7)>|<#meshed hosts>|<host-adress>|<ALIVE/KILL-adress>|*<host-quality>|<n*mesh-host>|<n*mesh-host quality>|<checksum 8bit>
;*don't care == will be calculated by receiver !!!
;explaining the shortform:
;#meshed hosts			<=>	count of direct hosts stored by the scanned host 		== this host
;ALIVE/KILL-adress		<=>	adress of this host 						            == this host
;host-quality			<=>	don't care, will be calculated by the receiver !
;n*mesh-host			<=>  adress of a meshed host stored by the scanned host 	== will be stored as meshed host by receiver
;n*mesh-host quality	<=>	meshed host data quality, stored by the scanned host	== will be stored as meshed hosts by receiver
;host-adress			<=>	the requesting device adress                            == is now the target adress !
;NEW: rebuild for dynamic process !
;..................................................................................................................
;input (Thost_adr) needed !
;input (Shost_adr) needed !
rfm12_119runX:
	jmp	rfm12_119run

rfm12_117run:
    lds r16,STATEsend
	cpi	r16,7
	brne	rfm12_117runb
	;*******************
	;set values (7)
	;A) rfm12 instruction
	;*******************
	sts	rfm12_instr,r16		;mode instruction =1 (scan the network)
	jmp	rfm12_117runSTARTa
rfm12_117runb:
	cpi	r16,8
	brne	rfm12_119runX
	;*******************
	;set values (8)
	;A) rfm12 instruction
	;*******************
	sts	rfm12_instr,r16		;mode instruction =1 (scan the network)
	;..........................
	;check the entry-condition
	;pointer = start rfm12_buffer
	;..........................
rfm12_117runSTARTa:
	;...............................
	;check tx-state
	;...............................
	lds	r16,COMTXstate
	sbrs	r16,RFMbufFILL				
	rjmp	rfm12_117runFILL			;0=fill rfm buffer first...
	sbrs	r16,RFMtxSTART			
	rjmp	rfm12_117runSTART			;0=rfm buffer filled -> transmit startsequence
	sbrs	r16,RFMtxRUN				
	rjmp	rfm12_117runACTIV			;0=nIRQ-svr based transmitting -> nothing else to do now!
	sbrs	r16,RFMtxSTOP				
	rjmp	rfm12_117runACTIV			
	jmp	rfm12_117runLOOPex			;1=nIRQ-svr already transmitted the tx-off instruction -> check further conditions

;......................................
;A) copy data to rfm_buffer
;......................................
rfm12_117runFILL:
	sbr	r16,1<<RFMbufFILL
	sts	COMTXstate,r16

	;clear checksum
	ldi	r17,0xaa
	sts	txCKsum,r17
	;adresspointer rfm-buffer
	lds	r18,rfm12_instr
	;adresspointer rfm-buffer start
	ldi	yl,low(rfm_buffer)
	ldi	yh,high(rfm_buffer)
	st	y,r18			;store identifier to tx-buffer
	eor	r17,r18			;exclusive or (ck-sum)
	adiw	yh:yl,2
	;*******************
	;B) store target adress 
	;*******************
    lds r22,HscanCNT
	sts	Thost_adr,r22   ;will be check by rx layer if equal with the scanned !!!!
	st	y+,r22			;store to tx-buffer
	eor	r17,r22			;exclusive or (ck-sum)
	;*******************
	;C) target adress
	;*******************
	lds	r19,Shost_adr
	st	y+,r19			;store to tx-buffer
	eor	r17,r19
	;******************
	;dummy
	;******************
	st	y+,r19			;store to tx-buffer
	eor	r17,r19

	;*******************
	;D) store ck-sum
	;*******************
	sts	txCKsum,r17
	;**********************
	;store pointer
	;points to the first 
	;direct adressposition
	;**********************
	sts	TXbufPOINTl,yl
	sts	TXbufPOINTh,yh
	;****************************
	;now search for direct 
	;hosts
	;* calculates the frame-size
    ;* stores up from <TXbufPOINT>
	;****************************
	call	scan_170HOtable
	ret




;............................................
;B) init TX
;including preamble & sync !
;............................................
rfm12_117runSTART:
    sbis    pinb,_ifSEL
	rjmp	rfm12_117runSTARTbk			;spi still busy with SPI 
 	;free...
	sbr	r16,1<<RFMtxSTART
	sts	COMTXstate,r16
	call	rfm12_TXinit
	;.......................................
	;start is done !
	;-> tells th nIRQ-svr to do the further
	;   tx-job !
	;.......................................
	lds	r16,COMTXstate
	sbr	r16,1<<RFMtxRUN
	sts	COMTXstate,r16
 	;***************************
	;I) write data to buffer & tx
	;***************************
	;lade MSB
	ldi	yl,low(rfm_buffer)
	ldi	yh,high(rfm_buffer)
   ;manchester
    call    _TXmanSUB
   ldi r16,0
	sts	TRANSbyte,r16
	call rfm12_initSPI
	;startsequence transmitted
    jmp rfm12_117run
    ;*******************
    ;spi busy
    ;*******************
rfm12_117runSTARTbk:
	ret

;............................................
;C) further tx will be done by nIRQfifo svr
;............................................
rfm12_117runACTIV:
	jmp rfm12_117run
	
;.............................................
;D) tx is ready - check conditions 
;.............................................
rfm12_117runLOOPex:
 	;.............................
	;check repeated tx
	;.............................
	lds	r17,TXrepCNT
	cpi	r17,TXrep					;max repeats
	breq	rfm12_117runNXTHOST
	inc	r17
	sts	TXrepCNT,r17
    ;.....................................
    ;data is sent to the scanning host
    ;now wait for the requested answer: 50
	;.....................................
    ;tx-off 
    call    rfm12_TXOFF
	ldi	r16,51
    jmp _errTX_REQ
    
	;.....................................
    ;max scan repeats reached...
	;check for limit...
    ;OR scan next host after the
    ;last wait state for requested answer:50
	;......................................
rfm12_117runNXTHOST:
	clr	r17
	sts	TXrepCNT,r17
	lds	r17,HscanCNT				;host counter
	cpi	r17,255				        ;
	breq	rfm12_117runEX  		;all hosts are scanned & exit
	inc	r17
	sts	HscanCNT,r17                ;for the next tx - job !
    ;.....................................
    ;data is sent to the scanning host
    ;now wait for the requested answer: 51
	;.....................................
    ;tx-off 
    call    rfm12_TXOFF
	ldi	r16,51
    jmp _errTX_REQ
  
	;........................
	;prepare for exit
	;........................
rfm12_117runEX:
    call    rfm12_TXOFFII
    ;b)network job ready display								
	call    _SYSmsgF
	;display header & rectangles
	call	_sys03IF
	;enable receive-mode
	call	rfm12_RXinit
    ;enable slave mode
    call    _SPIslaveINIT   ;including _ifSEL=input
    jmp _exTX







;..................................................................................................................
;9= TRANSMIT LHA-CHANGE MESSAGE
;the transmitting scan-frame-structure:
;<mode-instr.(9)>|<Shost-adress>|<Thost-adress>|<OLDShost-adress>|<checksum 8bit>
;ACKNOWLEDGE <51> requested !
;..................................................................................................................
;input (Thost_adr) needed !
;input (Shost_adr) needed !
rfm12_120runX:
	jmp	rfm12_120run

rfm12_119run:
    lds r16,STATEsend
	cpi	r16,9
	brne	rfm12_120runX
	;...............................
	;check tx-state
	;...............................
	lds	r16,COMTXstate
	sbrs	r16,RFMbufFILL				
	rjmp	rfm12_119runFILL			;0=fill rfm buffer first...
	sbrs	r16,RFMtxSTART			
	rjmp	rfm12_119runSTART			;0=rfm buffer filled -> transmit startsequence
	sbrs	r16,RFMtxRUN				
	rjmp	rfm12_119runACTIV			;0=nIRQ-svr based transmitting -> nothing else to do now!
	sbrs	r16,RFMtxSTOP				
	rjmp	rfm12_119runACTIV			
	jmp	rfm12_119runLOOPex			;1=nIRQ-svr already transmitted the tx-off instruction -> check further conditions

;......................................
;A) copy data to rfm_buffer
;......................................
rfm12_119runFILL:
	sbr	r16,1<<RFMbufFILL
	sts	COMTXstate,r16
	;clear checksum
	ldi	r17,0xaa
	sts	txCKsum,r17
	;adresspointer rfm-buffer start
	ldi	yl,low(rfm_buffer)
	ldi	yh,high(rfm_buffer)
	;*******************
	;set values
	;A) rfm12 instruction
	;*******************
	ldi	r18,9
	sts	rfm12_instr,r18		;mode instruction =1 (scan the network)
	st	y+,r18			;store to tx-buffer
	eor	r17,r18			;exclusive or (ck-sum)
	;*******************
	;B) load source adress 
	;*******************
   	lds	r22,Shost_adr
	st	y+,r22			;store to tx-buffer
	eor	r17,r22			;exclusive or (ck-sum)
	;*******************
	;C) target adress
	;*******************
    lds r19,HscanCNT
	sts	Thost_adr,r19   ;will be check by rx layer if equal with the scanned !!!!
	st	y+,r19			;store to tx-buffer
	eor	r17,r19
	;*******************
	;D) OLD source adress
	;*******************
    lds r19,OLDShost_adr
	st	y+,r19			;store to tx-buffer
	eor	r17,r19

	;*******************
	;E) store ck-sum
	;*******************
	st	y,r17			;store to tx-buffer
	sts	txCKsum,r17
	ret




;............................................
;B) init TX
;including preamble & sync !
;............................................
rfm12_119runSTART:
    ;spi free ?
    sbis    pinb,_ifSEL
	rjmp	rfm12_119runSTARTbk			;spi still busy with SPI
    ;free
 	sbr	r16,1<<RFMtxSTART
	sts	COMTXstate,r16
	call	rfm12_TXinit
	;.......................................
	;start is done !
	;-> tells th nIRQ-svr to do the further
	;   tx-job !
	;.......................................
	lds	r16,COMTXstate
	sbr	r16,1<<RFMtxRUN
	sts	COMTXstate,r16
 	;***************************
	;I) write data to buffer & tx
	;***************************
	;lade MSB
	ldi	yl,low(rfm_buffer)
	ldi	yh,high(rfm_buffer)
   ;manchester
    call    _TXmanSUB
 	ldi	r17,5
	sts	FRAME_siz,r17
    ldi r17,0
	sts	TRANSbyte,r17
	call rfm12_initSPI
    ;startsequence transmitted 
    jmp rfm12_119run
	
    ;*******************
    ;spi busy
    ;*******************
rfm12_119runSTARTbk:
	ret	
;............................................
;C) further tx will be done by nIRQfifo svr
;............................................
rfm12_119runACTIV:
	    jmp rfm12_119run


;.............................................
;D) tx is ready - check conditions 
;.............................................
rfm12_119runLOOPex:
	;.............................
	;check repeated tx
	;.............................
	lds	r17,TXrepCNT
	cpi	r17,TXrep					;max repeats
	breq	rfm12_119runNXTHOST
	inc	r17
	sts	TXrepCNT,r17
    ;.....................................
    ;rfm as a receiver & spi to slave
    ;.....................................
     call    rfm12_TXOFF
   ;.....................................
    ;data is sent to the scanning host
    ;now wait for the requested answer: 51
	;.....................................
	ldi	r16,51
    jmp _errTX_REQ
    
	;.....................................
    ;max scan repeats reached...
	;check for limit...
    ;OR scan next host after the
    ;last wait state for requested answer:50
	;......................................
rfm12_119runNXTHOST:
	clr	r17
	sts	TXrepCNT,r17
	lds	r17,HscanCNT				;host counter
	cpi	r17,255				        ;
	breq	rfm12_119runEX  		;all hosts are scanned & exit
	inc	r17
	sts	HscanCNT,r17                ;for the next tx - job !
   ;.....................................
    ;rfm as a receiver & spi to slave
    ;.....................................
    call    rfm12_TXOFF
    ;.....................................
    ;data is sent to the scanning host
    ;now wait for the requested answer: 51
	;.....................................
	ldi	r16,51
    jmp _errTX_REQ
  

	;........................
	;prepare for exit
	;........................
rfm12_119runEX:
    call    rfm12_TXOFFII
    ;.................
    ;network job ready 
	call    _SYSmsgF
	call	_sys03IF
    ;.................
	;enable receive-mode
	call	rfm12_RXinit
    ;enable slave mode
    call    _SPIslaveINIT   ;including _ifSEL=input
    jmp _exTX


 






;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;10 =TRANSMIT REMOTE DATA (MASTER/SLAVE)
;Decide if it's a:
;- Intruction without data          (REMtype=REMinstr) + (REMsize=8)
;- Instruction with controller data (REMtype=REMconDAT)+ (REMsize= [spiDATctSOLL]+8) --- set by spiIRQ
;- Instruction with display data    (REMtype=REMdisDAT)+ (REMsize=14) --- set by spiTX
;
;format:
;<m-instr><LAB-source><LAB-target><Master-adress><Client-adress><REMtype><REMsize>[data]<ck-sum>
;              |                           
;          this host       
;NOTE: activ remote connection uses channel 15          
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;jmp
rfm12_150runX:
    jmp rfm12_150run

rfm12_120run:
  	lds	r16,STATEsend		
	cpi	r16,10
	brne	rfm12_150runX
	;...............................
	;check tx-state
	;...............................
	lds	r16,COMTXstate
	sbrs	r16,RFMbufFILL				
	rjmp	rfm12_120runFILL			;0=fill rfm buffer first...
	sbrs	r16,RFMtxSTART			
	rjmp	rfm12_120runSTART			;0=rfm buffer filled -> transmit startsequence
	sbrs	r16,RFMtxRUN				
	rjmp	rfm12_120runACTIV			;0=nIRQ-svr based transmitting -> nothing else to do now!
	sbrs	r16,RFMtxSTOP				
	rjmp	rfm12_120runACTIV			
	jmp	rfm12_120runLOOPex			    ;1=nIRQ-svr already transmitted the tx-off instruction -> check further conditions


;......................................
;A) copy data to rfm_buffer
;NOTE: receiver is off
;......................................
rfm12_120runFILL:
	sbr	r16,1<<RFMbufFILL
	sts	COMTXstate,r16
    ;clear checksum
	ldi	r17,0xaa
	;adresspointer rfm-buffer start
	ldi	yl,low(rfm_buffer)
	ldi	yh,high(rfm_buffer)
	;*******************
	;set values
	;A) rfm12 instruction
	;*******************
	ldi	r18,10
	sts	rfm12_instr,r18		            ;mode instruction =10 (remote)
	st	y+,r18			                ;store to tx-buffer
	eor	r17,r18			                ;exclusive or (ck-sum)
	;******************************
	;B) load LAB source adress 
	;******************************
   	lds	r22,Shost_adr
	st	y+,r22			;store to tx-buffer
	eor	r17,r22			;exclusive or (ck-sum)
	;******************************
	;C) load LAB destination adress
	;******************************
    lds r22,REMif
    sbrs    r22,REMstate
    rjmp    rfm12_120runFILLSLAVE
    ;1=master
    lds r22,REMslave    ;to slave...
    rjmp    rfm12_120runFILLGO
rfm12_120runFILLSLAVE:
    ;0=slave
    lds r22,REMmaster   ;to master...
    ;goon...
rfm12_120runFILLGO:
	st	y+,r22			;LAB target adress to tx-buffer
	eor	r17,r22
    ;*****************************
    ;D) The master's remote adress
    ;*****************************
    lds r16,REMmaster
	st	y+,r16			;store to tx-buffer
	eor	r17,r16			;exclusive or (ck-sum)
    ;*****************************
    ;E) The client's remote adress
    ;*****************************
    lds r22,REMslave    ;to slave...
	st	y+,r22			;store to tx-buffer
	eor	r17,r22			;exclusive or (ck-sum)
    ;*******************
    ;F) remote type
    ;*******************
  	lds	r22,REMtype
	st	y+,r22			;store to tx-buffer
	eor	r17,r22			;exclusive or (ck-sum)
    ;*******************
    ;G)check REMtype
    ;*******************
  	lds	r22,REMtype
    cpi r22,REMinstr
    breq   rfm12_120runFILLinstr        ;A)instruction WITHOUT data
    cpi r22,REMdisDAT
    breq   rfm12_120runFILLdata         ;B)instruction WITH display-data
    cpi r22,REMconDAT
    breq   rfm12_120runFILLcon          ;C)instruction WITH control-data
    jmp rfm12_120runEX                  ;invalid....
    ;---------------------------------------
    ;G1) transmit instruction WITHOUT data
    ;---------------------------------------
rfm12_120runFILLinstr:
    clr r22
	st	y+,r22			;store to tx-buffer
	eor	r17,r22			;exclusive or (ck-sum)
	;*******************
	;A1) store ck-sum
	;*******************
	st	y,r17			;store to tx-buffer
	sts	txCKsum,r17
    ;*******************
    ;A2) store fr-size=8
    ;*******************
    ldi r22,8
    sts	FRAME_siz,r22
    ;back
	 ;back
    rjmp  rfm12_120runSTART  

    ;----------------------------------------------------------
    ;G2) slave => master transmit instruction WITH display-data
    ;----------------------------------------------------------
rfm12_120runFILLdata:
    lds r22,REMsize     ;set by spiIRQ
	st	y+,r22			;store to tx-buffer
	eor	r17,r22			;exclusive or (ck-sum)
    ;******************
    ;B1) calc fr-size
    ;******************
    ldi r16,8
    add r22,r16
    sts	FRAME_siz,r22
    ;******************
    ;B2)load & store 
    ;******************
    lds r22,REMsize     ;data size, set by spiIRQ
    ;comBUF startadresse
	ldi	zh,high(comBUF)
	ldi	zl,low(comBUF)
    ;fill loop
rfm12_120runFILLdataLOOP:
    ld  r16,z+
    st  y+,r16
    eor r17,r16
    ;size -1
    dec r22
    tst r22
    breq   rfm12_120runFILLdataGO
    rjmp   rfm12_120runFILLdataLOOP      
    ;ready stored
rfm12_120runFILLdataGO:
    ;*******************
	;B3) store ck-sum
	;*******************
	st	y,r17			;store to tx-buffer
	sts	txCKsum,r17
    ;back
 ;back
    rjmp  rfm12_120runSTART  

    ;----------------------------------------------------------
    ;G3) master => slave transmit instruction WITH control-data
    ;----------------------------------------------------------
rfm12_120runFILLcon:
    lds r22,REMsize     ;set by spiIRQ
	st	y+,r22			;store to tx-buffer
	eor	r17,r22			;exclusive or (ck-sum)
    ;******************
    ;C1) calc fr-size
    ;******************
    ldi r16,8
    add r22,r16
    sts	FRAME_siz,r22
    ;******************
    ;C2) load & store 
    ;******************
    ;index
	lds	r22,curINDEXte  
	st	y+,r22			;store to tx-buffer
	eor	r17,r22			;exclusive or (ck-sum)
    ;graph function
	lds		r22,paraFUNCte
	st	y+,r22			;store to tx-buffer
	eor	r17,r22			;exclusive or (ck-sum)
    ;para position
	lds		r22,paraPOSte
	st	y+,r22			;store to tx-buffer
	eor	r17,r22			;exclusive or (ck-sum)
    ;parameter
	lds		r22,paraAte
	st	y+,r22			;store to tx-buffer
	eor	r17,r22			;exclusive or (ck-sum)
    ;para limit lower
	lds		r22,paraMINte
	st	y+,r22			;store to tx-buffer
	eor	r17,r22			;exclusive or (ck-sum)
    ;para limit upper
	lds		r22,paraMAXte
	st	y+,r22			;store to tx-buffer
	eor	r17,r22			;exclusive or (ck-sum)
    ;*******************
	;C3) store ck-sum
	;*******************
	st	y,r17			;store to tx-buffer
	sts	txCKsum,r17
    ;clear flags
	lds	r16,comFLAG
    cbr r16,1<<newHIFtx
	cbr	r16,1<<spiBUSYhif	;1=spi hif-data ready transmitted 
;    sbr r16,1<<waitHIFdata  ;-> wait for concept-data... cleared after receive !
	sts	comFLAG,r16
    ;back
    rjmp  rfm12_120runSTART  
	;ret


;............................................
;H) init TX - including preamble & sync !
;............................................
rfm12_120runSTART:
    ;spi free ?
    sbis    pinb,_ifSEL
	rjmp	rfm12_120runSTARTbk			;spi still busy with SPI
    ;free
    lds r16,COMTXstate
 	sbr	r16,1<<RFMtxSTART
	sts	COMTXstate,r16
	call	rfm12_TXinit
	;.......................................
	;start is done !
	;-> tells th nIRQ-svr to do the further
	;   tx-job !
	;.......................................
	lds	r16,COMTXstate
	sbr	r16,1<<RFMtxRUN
	sts	COMTXstate,r16
 	;***************************
	;I) write data to buffer & tx
	;***************************
	;lade MSB
	ldi	yl,low(rfm_buffer)
	ldi	yh,high(rfm_buffer)
   ;manchester
    call    _TXmanSUB
   ldi r17,0
	sts	TRANSbyte,r17
	call rfm12_initSPI
    ;startsequence transmitted 
    jmp rfm12_120run
	
    ;*******************
    ;spi busy
    ;*******************
rfm12_120runSTARTbk:
	ret	
;............................................
;I) further tx will be done by nIRQfifo svr
;............................................
rfm12_120runACTIV:
	    jmp rfm12_120run
;.............................................
;J) tx is ready - check conditions 
;.............................................
rfm12_120runLOOPex:
    ;.....................................
    ;rfm as a receiver & spi to slave
    ;.....................................
    call    rfm12_TXOFF
    ;.....................................
    ;data is sent to the remoted host
    ;now wait for the requested answer: 55
	;.....................................
	ldi	r16,55
    jmp _errTX_REQ
    
;..................................
;K) prepare for exit
;..................................
rfm12_120runEX:
    ;enable slave mode
    call    _SPIslaveINIT   ;including _ifSEL=input
rfm12_120runEXX:
    jmp _exTX


 

















;..................................................................................................................
;50= TRANSMIT REQUESTED SCAN ACKNOWLEDGE 
;in order of a received scan frame(1)...;	    <rfm12-instr.(1)>|<source-adress>|<target-adress>|<checksum 8bit>
;the transmitting scan-frame-structure:
;<mode-instr.(50)>|<#meshed hosts>|<host-adress>|<scanned-adress>|*<host-quality>|<n*mesh-host>|<n*mesh-host quality>|<checksum 8bit>
;*don't care == will be calculated by receiver !!!
;explaining the shortform:
;#meshed hosts			<=>	count of direct hosts stored by the scanned host 		== this host
;scanned-adress		<=>	adress of the scanned host 						== this host
;host-quality			<=>	don't care, will be calculated by the receiver !
;n*mesh-host			<=>  adress of a meshed host stored by the scanned host 	== will be stored as meshed host by receiver
;n*mesh-host quality	<=>	meshed host data quality, stored by the scanned host	== will be stored as meshed hosts by receiver
;host-adress			<=>	the requesting device adress == is now the target adress !
;NEW: rebuild for dynamic process !
;..................................................................................................................
;input (Thost_adr) needed !
;input (Shost_adr) needed !
rfm12_151runX:
	jmp	rfm12_151run

rfm12_150run:
    lds r16,STATEsend
	cpi	r16,50
	brne	rfm12_151runX
	;...............................
	;check tx-state
	;...............................
	lds	r16,COMTXstate
	sbrs	r16,RFMbufFILL				
	rjmp	rfm12_150runFILL			;0=fill rfm buffer first...
	sbrs	r16,RFMtxSTART			
	rjmp	rfm12_150runSTART			;0=rfm buffer filled -> transmit startsequence
	sbrs	r16,RFMtxRUN				
	rjmp	rfm12_150runACTIV			;0=nIRQ-svr based transmitting -> nothing else to do now!
	sbrs	r16,RFMtxSTOP				
	rjmp	rfm12_150runACTIV			
	jmp	rfm12_150runLOOPex			;1=nIRQ-svr already transmitted the tx-off instruction -> check further conditions

;......................................
;A) copy data to rfm_buffer
;......................................
rfm12_150runFILL:
	sbr	r16,1<<RFMbufFILL
	sts	COMTXstate,r16	;check repeat counter
	;clear checksum
	ldi	r17,0xaa
	sts	txCKsum,r17
	;set rfm startadress
	ldi	yl,low(rfm_buffer)
	ldi	yh,high(rfm_buffer)
	;*******************
	;set values
	;A) rfm12 instruction
	;*******************
	ldi	r18,50
	sts	rfm12_instr,r18	;mode instruction =50 (ack scan the network)
	st	y,r18			;store to tx-buffer
	eor	r17,r18			;exclusive or (ck-sum)
	adiw	yh:yl,2
	;*******************
	;B) store target adress 
	;*******************
	lds	r22,Thost_adr
	st	y+,r22			;store to tx-buffer
	eor	r17,r22			;exclusive or (ck-sum)
	;*******************
	;C) target adress
	;*******************
	lds	r19,Shost_adr
	st	y+,r19			;store to tx-buffer
	eor	r17,r19
	;******************
	;dummy
	;******************
	st	y+,r19			;store to tx-buffer
	eor	r17,r19
	;*******************
	;D) store ck-sum
	;*******************
	sts	txCKsum,r17
	;**********************
	;store pointer
	;points to the first 
	;direct adressposition
	;**********************
	sts	TXbufPOINTl,yl
	sts	TXbufPOINTh,yh
	;**********************
	;now search for direct 
	;hosts
	;**********************
	call	scan_170HOtable
	ret



;............................................
;B) init TX
;including preamble & sync !
;............................................
rfm12_150runSTART:
    sbis    pinb,_ifSEL
	rjmp	rfm12_150runSTARTbk			;spi still busy with SPI
 	;free...
	sbr	r16,1<<RFMtxSTART
	sts	COMTXstate,r16
	call	rfm12_TXinit
	;.......................................
	;start is done !
	;-> tells th nIRQ-svr to do the further
	;   tx-job !
	;.......................................
	lds	r16,COMTXstate
	sbr	r16,1<<RFMtxRUN
	sts	COMTXstate,r16
 	;***************************
	;I) write data to buffer & tx
	;***************************
	;lade MSB
	ldi	yl,low(rfm_buffer)
	ldi	yh,high(rfm_buffer)
   ;manchester
    call    _TXmanSUB
    ldi r16,0
	sts	TRANSbyte,r16
	call rfm12_initSPI
	;startsequence transmitted
    jmp rfm12_150run
    ;*******************
    ;spi busy
    ;*******************
rfm12_150runSTARTbk:
	ret
;............................................
;C) further tx will be done by nIRQfifo svr
;............................................
rfm12_150runACTIV:
	   jmp rfm12_150run
;.............................................
;D) tx is ready - check conditions 
;.............................................
rfm12_150runLOOPex:
    ;tx-off & receive-mode
    call    rfm12_TXOFF
    jmp _exTXX                  ;check further: any other tx-job should ggon running?
	



















;..................................................................................................................
;51= TRANSMIT REQUESTED ,ALIVE-KILL-LHA Change' ACKNOWLEDGE IN ORDER OF RECEIVE=7/8/9
;acknowledge & NO quality-check !
;on order of a received frame...
;mi=7 received alive message,containing the meshed hosts of a direct hosts
;mi=8 received kill message,containing the meshed hosts of a direct hosts
;protocol: <mode-instr.>|<source-adress>|<target-adress>|<check-sum>
;don't change the rfm_buffer !!!
;NEW: rebuild for dynamic process !
;..................................................................................................................
;input (Thost_adr) needed !
;
rfm12_152runX:
	jmp	rfm12_152run

rfm12_151run:
	lds	r16,STATEsend		
    cpi	r16,51
	brne	rfm12_152runX
	;...............................
	;check tx-state
	;...............................
	lds	r16,COMTXstate
	sbrs	r16,RFMbufFILL				
	rjmp	rfm12_151runFILL			;0=fill rfm buffer first...
	sbrs	r16,RFMtxSTART			
	rjmp	rfm12_151runSTART			;0=rfm buffer filled -> transmit startsequence
	sbrs	r16,RFMtxRUN				
	rjmp	rfm12_151runACTIV			;0=nIRQ-svr based transmitting -> nothing else to do now!
	sbrs	r16,RFMtxSTOP				
	rjmp	rfm12_151runACTIV			
	jmp	rfm12_151runLOOPex			;1=nIRQ-svr already transmitted the tx-off instruction -> check further conditions

;......................................
;A) copy data to rfm_buffer
;.......................................
rfm12_151runFILL:
	sbr	r16,1<<RFMbufFILL
	sts	COMTXstate,r16	;check repeat counter
	;...........................
	;a) wait alive/kill mask
	call	_SYSmsgE
	;display header & rectangles
	call	_sys03IF
   

	;clear checksum
	ldi	r17,0xaa
	sts	txCKsum,r17
	;set rfm startadress
	ldi	yl,low(ack_buffer)
	ldi	yh,high(ack_buffer)
	;*******************
	;set values
	;A) rfm12 instruction
	;*******************
	ldi	r18,51
	sts	rfm12_instr,r18		;mode instruction =1 (scan the network)
	st	y+,r18			;store to tx-buffer
	eor	r17,r18			;exclusive or (ck-sum)
	;*******************
	;B) load source adress
	;*******************
	lds	r22,Shost_adr
	st	y+,r22			;store to tx-buffer
	eor	r17,r22			;exclusive or (ck-sum)
	;*******************
	;C) target adress
	;*******************
	lds	r19,Thost_adr
	st	y+,r19			;store to tx-buffer
	eor	r17,r19
	;*******************
	;D) store ck-sum
	;*******************
	st	y,r17			;store to tx-buffer
	sts	txCKsum,r17
	;*******************
	;E) set ack-frame size
	;*******************
	ldi	r17,4
	sts	FRAME_siz,r17
	ret



;............................................
;B) init TX
;including preamble & sync !
;............................................
rfm12_151runSTART:
    sbis    pinb,_ifSEL
	rjmp	rfm12_151runSTARTbk			;spi still busy with SPI
 	;free...
	sbr	r16,1<<RFMtxSTART
	sts	COMTXstate,r16
	call	rfm12_TXinit
   	;.......................................
	;start is done !
	;-> tells th nIRQ-svr to do the further
	;   tx-job !
	;.......................................
	lds	r16,COMTXstate
	sbr	r16,1<<RFMtxRUN
	sts	COMTXstate,r16
 	;***************************
	;I) write data to buffer & tx
	;***************************
	;lade MSB
	ldi	yl,low(ack_buffer)
	ldi	yh,high(ack_buffer)
   ;manchester
    call    _TXmanSUB
    ldi r16,0
	sts	TRANSbyte,r16
	call rfm12_initSPI
	;startsequence transmitted 
    jmp rfm12_151run
    ;*******************
    ;spi busy
    ;*******************
rfm12_151runSTARTbk:
	ret
;............................................
;C) further tx will be done by nIRQfifo svr
;............................................
rfm12_151runACTIV:
    jmp rfm12_151run
	;ret
	
;.............................................
;D) tx is ready - check conditions 
;.............................................
rfm12_151runLOOPex:
    ;tx-off
    call    rfm12_TXOFFII
    ;ALL DONE !
	;search for a equal direct host
    ;rebuild the table...
    ;............................                                
    call	scan_130HOtable	;130
    ;b)network job ready display								
	call    _SYSmsgF
	;display header & rectangles
	call	_sys03IF
  
	;enable receive-mode
	call	rfm12_RXinit
    ;enable slave mode
    call    _SPIslaveINIT   ;including _ifSEL=input
    ;back
    jmp _exTXX                  ;check further: any other tx-job should ggon running?









;..................................................................................................................
;52/53/54= TRANSMIT REQUESTED ACKNOWLEDGE IN ORDER OF RECEIVED=2/4/5/6
;acknowledge & NO quality-check !
;on order of a received frame...
;mi=7 received embedded data frame
;protocol: 	<mode-instr.52>|<source-adress>|<Dtarget-adress >|<---------->|<check-sum>
;			<mode-instr.53>|<Msource-adress>|<Dtarget-adress>|<target-adr>|<check-sum>						
;			<mode-instr.54>|<Dsource-adress>|<-------------->|<target-adr>|<check-sum>
;don't change the rfm_buffer !!!
;NEW: rebuild for dynamic process !
;..................................................................................................................
;input (Thost_adr) needed !
;
rfm12_153runX:
	jmp	rfm12_153run

rfm12_152run:
    lds r16,STATEsend
	cpi	r16,52
	brne	rfm12_152runA
	sts	rfm12_instr,r16
	jmp	rfm12_152runY
rfm12_152runA:
	cpi	r16,53
	brne	rfm12_152runB
	sts	rfm12_instr,r16
	jmp	rfm12_152runY
rfm12_152runB:
	cpi	r16,54
	brne	rfm12_153runX
	sts	rfm12_instr,r16
	jmp	rfm12_152runY	
		
	;..........................
	;check the entry-condition
	;pointer = start rfm12_buffer
	;..........................
rfm12_152runY:
	;...............................
	;check tx-state
	;...............................
	lds	r16,COMTXstate
	sbrs	r16,RFMbufFILL				
	rjmp	rfm12_152runFILL			;0=fill rfm buffer first...
	sbrs	r16,RFMtxSTART			
	rjmp	rfm12_152runSTART			;0=rfm buffer filled -> transmit startsequence
	sbrs	r16,RFMtxRUN				
	rjmp	rfm12_152runACTIV			;0=nIRQ-svr based transmitting -> nothing else to do now!
	sbrs	r16,RFMtxSTOP				
	rjmp	rfm12_152runACTIV			
	jmp	rfm12_152runLOOPex			;1=nIRQ-svr already transmitted the tx-off instruction -> check further conditions


;......................................
;A) copy data to rfm_buffer
;......................................
rfm12_152runFILL:
	sbr	r16,1<<RFMbufFILL
	sts	COMTXstate,r16	;check repeat counter
	;clear checksum
	ldi	r17,0xaa
	sts	txCKsum,r17
	;set rfm startadress
	ldi	yl,low(ack_buffer)
	ldi	yh,high(ack_buffer)
	;*******************
	;set values
	;A) rfm12 instruction
	;*******************
	lds	r18,rfm12_instr
	st	y+,r18			;store to tx-buffer
	eor	r17,r18			;exclusive or (ck-sum)
	;*******************
	;B) load source adress
	;*******************
	lds	r22,Shost_adr
	st	y+,r22			;store to tx-buffer
	eor	r17,r22			;exclusive or (ck-sum)
	;*******************
	;C) Dtarget adress
	;*******************
	lds	r19,Thost_adr
	st	y+,r19			;store to tx-buffer
	eor	r17,r19
	;*******************
	;C) Mtarget adress
	;*******************
	lds	r19,T2host_adr
	st	y+,r19			;store to tx-buffer
	eor	r17,r19

	;*******************
	;D) store ck-sum
	;*******************
	st	y,r17			;store to tx-buffer
	sts	txCKsum,r17
	;*******************
	;F) set ack-frame size
	;*******************
	ldi	r17,5
	sts	FRAME_siz,r17
	ret






;............................................
;B) init TX
;including preamble & sync !
;............................................
rfm12_152runSTART:
    sbis    pinb,_ifSEL
	rjmp	rfm12_152runSTARTbk			;spi still busy with SPI
	;free...
	sbr	r16,1<<RFMtxSTART
	sts	COMTXstate,r16
	call	rfm12_TXinit
	;.......................................
	;start is done !
	;-> tells th nIRQ-svr to do the further
	;   tx-job !
	;.......................................
	lds	r16,COMTXstate
	sbr	r16,1<<RFMtxRUN
	sts	COMTXstate,r16
 	;***************************
	;I) write data to buffer & tx
	;***************************
	;lade MSB
	ldi	yl,low(ack_buffer)
	ldi	yh,high(ack_buffer)
   ;manchester
    call    _TXmanSUB
    ldi r16,0
	sts	TRANSbyte,r16
	call rfm12_initSPI
	;startsequence transmitted
    jmp rfm12_152run
    ;*******************
    ;spi busy
    ;*******************
rfm12_152runSTARTbk:
	ret
;............................................
;C) further tx will be done by nIRQfifo svr
;............................................
rfm12_152runACTIV:
	   jmp rfm12_152run

;.............................................
;D) tx is ready - check conditions 
;.............................................
rfm12_152runLOOPex:
     ;tx-off & receive-mode
    call    rfm12_TXOFF
    jmp _exTXX                  ;check further: any other tx-job should goon running?
	





;NEW
;..................................................................................................................
;55= TRANSMIT REQUESTED ACKNOWLEDGE IN ORDER OF RECEIVED=10 (REMOTE)
;acknowledge & NO quality-check !
;on order of a received frame...
;protocol: 	<mode-instr.55>|<source-adress> |<Dtarget-adress>|<REMtype>|<check-sum>
;don't change the rfm_buffer !!!
;NOTE: activ remote connection uses channel 15
;..................................................................................................................

rfm12_154runX:
	jmp	rfm12_154run

rfm12_153run:
    lds r16,STATEsend
	cpi	r16,55
	brne	rfm12_154runX
	;...............................
	;check tx-state
	;...............................
	lds	r16,COMTXstate
	sbrs	r16,RFMbufFILL				
	rjmp	rfm12_153runFILL			;0=fill rfm buffer first...
	sbrs	r16,RFMtxSTART			
	rjmp	rfm12_153runSTART			;0=rfm buffer filled -> transmit startsequence
	sbrs	r16,RFMtxRUN				
	rjmp	rfm12_153runACTIV			;0=nIRQ-svr based transmitting -> nothing else to do now!
	sbrs	r16,RFMtxSTOP				
	rjmp	rfm12_153runACTIV			
	jmp	rfm12_153runLOOPex			;1=nIRQ-svr already transmitted the tx-off instruction -> check further conditions

;......................................
;A) copy data to rfm_buffer
;......................................
rfm12_153runFILL:
	sbr	r16,1<<RFMbufFILL
	sts	COMTXstate,r16	;check repeat counter
	;clear checksum
	ldi	r17,0xaa
	sts	txCKsum,r17
	;set rfm startadress
	ldi	yl,low(ack_buffer)
	ldi	yh,high(ack_buffer)
	;*******************
	;set values
	;A) rfm12 instruction
	;*******************
	ldi	r18,55
	st	y+,r18			;store to tx-buffer
	eor	r17,r18			;exclusive or (ck-sum)
	;*******************
	;B) load source adress
	;*******************
	lds	r22,Shost_adr
	st	y+,r22			;store to tx-buffer
	eor	r17,r22			;exclusive or (ck-sum)
	;*******************
	;C) Dtarget adress
	;*******************
	lds	r19,Thost_adr
	st	y+,r19			;store to tx-buffer
	eor	r17,r19
	;*******************
	;D) REMtype
	;*******************
	lds	r19,REMtypeTE
	st	y+,r19			;store to tx-buffer
	eor	r17,r19
	;*******************
	;D) store ck-sum
	;*******************
	st	y,r17			;store to tx-buffer
	sts	txCKsum,r17
	;*******************
	;F) set ack-frame size
	;*******************
	ldi	r17,5
	sts	FRAME_siz,r17
	;ret

;............................................
;B) init TX
;including preamble & sync !
;............................................
rfm12_153runSTART:
    sbis    pinb,_ifSEL
	rjmp	rfm12_153runSTARTbk			;spi still busy with SPI
	;free...
   	lds	r16,COMTXstate
	sbr	r16,1<<RFMtxSTART
	sts	COMTXstate,r16
	call	rfm12_TXinit
	;.......................................
	;start is done !
	;-> tells th nIRQ-svr to do the further
	;   tx-job !
	;.......................................
	lds	r16,COMTXstate
	sbr	r16,1<<RFMtxRUN
	sts	COMTXstate,r16
 	;***************************
	;I) write data to buffer & tx
	;***************************
	;lade MSB
	ldi	yl,low(ack_buffer)
	ldi	yh,high(ack_buffer)
   ;manchester
    call    _TXmanSUB
    ldi r16,0
	sts	TRANSbyte,r16
	call rfm12_initSPI
	;startsequence transmitted
    jmp rfm12_153run
    ;*******************
    ;spi busy
    ;*******************
rfm12_153runSTARTbk:
	ret
;............................................
;C) further tx will be done by nIRQfifo svr
;............................................
rfm12_153runACTIV:
	   jmp rfm12_153run

;.............................................
;D) tx is ready - check conditions 
;.............................................
rfm12_153runLOOPex:
    ;instr?
    lds r16,REMtypeTE
    cpi r16,REMinstr
    brne   rfm12_153runLOOPexX  ;no instr 
    ;check if it's the first ack of a client
    lds r16,REMif
    sbrc    r16,REMstate
    rjmp   rfm12_153runLOOPexMSTR  ;master 
    ;------------------------------------
    ;now client changes to remote channel
	;rx/tx-channel
	ldi	r16,15
	sts	RXch,r16
	sts	TXch,r16
    ;------------------------------------  
    ;tx-off & receive-mode
    call    rfm12_TXOFF
     ;remote activ text
    call    _SYSmsgK
    call	_sys03IF
    jmp _exTX   
  

rfm12_153runLOOPexMSTR:
    ;clear remote flags
    clr  r16                ;
    sts REMif,r16  
    ;------------------------------------
    ;now master changes to default channel
	;rx/tx-channel
	ldi	r16,10
	sts	RXch,r16
	sts	TXch,r16
    ;------------------------------------			
rfm12_153runLOOPexX:
   ;tx-off & receive-mode
    call    rfm12_TXOFF
    jmp _exTX                  ;











;..................................................................................................................
;56= TRANSMIT REQUESTED ACKNOWLEDGE IN ORDER OF RECEIVED=11
;acknowledge & NO quality-check !
;on order of a received frame...
;mi=7 received embedded data frame
;protocol: 	<mode-instr.56>|<source-adress>|<Dtarget-adress >|<check-sum>
;don't change the rfm_buffer !!!
;NEW: rebuild for dynamic process !
;..................................................................................................................
;input (Thost_adr) needed !
;
rfm12_155runX:
	jmp	rfm12_155run

rfm12_154run:
    lds r16,STATEsend
	cpi	r16,56
	brne	rfm12_155runX
    sts rfm12_instr,r16
	;...............................
	;check tx-state
	;...............................
	lds	r16,COMTXstate
	sbrs	r16,RFMbufFILL				
	rjmp	rfm12_154runFILL			;0=fill rfm buffer first...
	sbrs	r16,RFMtxSTART			
	rjmp	rfm12_154runSTART			;0=rfm buffer filled -> transmit startsequence
	sbrs	r16,RFMtxRUN				
	rjmp	rfm12_154runACTIV			;0=nIRQ-svr based transmitting -> nothing else to do now!
	sbrs	r16,RFMtxSTOP				
	rjmp	rfm12_154runACTIV			
	jmp	rfm12_154runLOOPex			;1=nIRQ-svr already transmitted the tx-off instruction -> check further conditions


;......................................
;A) copy data to rfm_buffer
;......................................
rfm12_154runFILL:
	sbr	r16,1<<RFMbufFILL
	sts	COMTXstate,r16	;check repeat counter
	;clear checksum
	ldi	r17,0xaa
	sts	txCKsum,r17
	;set rfm startadress
	ldi	yl,low(ack_buffer)
	ldi	yh,high(ack_buffer)
	;*******************
	;set values
	;A) rfm12 instruction
	;*******************
	lds	r18,rfm12_instr
	st	y+,r18			;store to tx-buffer
	eor	r17,r18			;exclusive or (ck-sum)
	;*******************
	;B) load source adress
	;*******************
	lds	r22,Shost_adr
	st	y+,r22			;store to tx-buffer
	eor	r17,r22			;exclusive or (ck-sum)
	;*******************
	;C) Dtarget adress
	;*******************
	lds	r19,Thost_adr
	st	y+,r19			;store to tx-buffer
	eor	r17,r19
	;*******************
	;D) store ck-sum
	;*******************
	st	y,r17			;store to tx-buffer
	sts	txCKsum,r17
	;*******************
	;F) set ack-frame size
	;*******************
	ldi	r17,4
	sts	FRAME_siz,r17
	ret

;............................................
;B) init TX
;including preamble & sync !
;............................................
rfm12_154runSTART:
    sbis    pinb,_ifSEL
	rjmp	rfm12_154runSTARTbk			;spi still busy with SPI
	;free...
	sbr	r16,1<<RFMtxSTART
	sts	COMTXstate,r16
	call	rfm12_TXinit
	;.......................................
	;start is done !
	;-> tells th nIRQ-svr to do the further
	;   tx-job !
	;.......................................
	lds	r16,COMTXstate
	sbr	r16,1<<RFMtxRUN
	sts	COMTXstate,r16
 	;***************************
	;I) write data to buffer & tx
	;***************************
	;lade MSB
	ldi	yl,low(ack_buffer)
	ldi	yh,high(ack_buffer)
   ;manchester
    call    _TXmanSUB
    ldi r16,0
	sts	TRANSbyte,r16
	call rfm12_initSPI
	;startsequence transmitted
    jmp rfm12_154run
    ;*******************
    ;spi busy
    ;*******************
rfm12_154runSTARTbk:
	ret
;............................................
;C) further tx will be done by nIRQfifo svr
;............................................
rfm12_154runACTIV:
	   jmp rfm12_154run

;.............................................
;D) tx is ready - check conditions 
;.............................................
rfm12_154runLOOPex:
   ;tx-off
    call    rfm12_TXOFFII
    ;application reset display								
	call    _SYSmsgM
	;display header & rectangles
	call	_sys03IF
	;enable receive-mode
	call	rfm12_RXinit
    ;enable slave mode
    call    _SPIslaveINIT   ;including _ifSEL=input


;.............................................
;reset system & bootloader entry
;.............................................
    ;buffer start
    ldi	zl,low(TX_buffer)
	ldi	zh,high(TX_buffer)				;startpoint TX_buffer
    ;B)mi: setup/to core
    ldi r16,0b01001000          
  	st  z+,r16
    ;C)4xdummy
    lds r16,Shost_adr
 	st  z+,r16
    ldi r16,0xaa
  	st  z+,r16
  	st  z+,r16
  	st  z+,r16
    ;D)size=7
    ldi r16,7                  
  	st  z+,r16
    ;E)type=99 (reset)
    ldi r16,99
  	st  z,r16
    ;enable tranmit to system
 	lds	r16,comNETstate
	sbr	r16,1<<comNETnewDATtx
	sts	comNETstate,r16
    call    _conceptSND             ;transmit to system
    ;disable interrupts
    cli
;.............................................
;reset interface & bootloader entry
;.............................................
    ldi zh,high(BOOTSTART)
    ldi zl,low(BOOTSTART)
    ijmp

























;-----------------------------------------------------------------------------
;Mehr gibt's nicht... also >56 clear
;-----------------------------------------------------------------------------
rfm12_155run:
    call    rfm12_RXinit
    jmp _exTX





;--------------------------------------------------------------------------
;exit jmp's
;--------------------------------------------------------------------------

   ;........................
    ;tx-mode <50
    ;transmit again...
    ;........................
_errTX_REQ:
	sts	STATEreq,r16
	;clear rx-state
	clr	r16
	sts	STATErec,r16
	;clear framesize
	sts	FRAME_siz,r16		
	;clear byte-counter			
	sts	TRANSbyte,r16
	;clear rfm-states
    ;but buffer secure still activ !
    sbr r16,1<<BUFsecure  
	sts	COMTXstate,r16
	ret	

    ;..........................
    ;- exit transmitting
    ;  & enable tx buffer
    ;  & clear states
    ;..........................
_exTX:
	clr	r16
	;clear rec-state
	sts	STATErec,r16
	;set transmit state=0
	sts	STATEsend,r16
	;set transmit state=0
	sts	STATEsendTE,r16
	;clear requested rec-state
	sts	STATEreq,r16
    ;all free
	sts	COMTXstate,r16
 	ret

    ;.....................................................
    ;- exit acknowledge transmitting mode:50/51/52/53/54
    ;  & enable tx buffer if STATEsendTE=0
    ;  & clear states
    ;.....................................................
_exTXX:
    lds r16,STATEsendTE
    tst r16
    brne  _exTXXX  
    ;=0
	clr	r16
	;clear rec-state
	sts	STATErec,r16
	;set transmit state=0
	sts	STATEsend,r16
	;clear requested rec-state
	sts	STATEreq,r16
    ;all free
	sts	COMTXstate,r16
 	ret
    ;>0
_exTXXX:
    ;restore transmit state
	sts	STATEsend,r16
    ;
	clr	r16
	;clear rec-state
	sts	STATErec,r16
	;clear requested rec-state
	sts	STATEreq,r16
	;set temp. transmit state=0
	sts	STATEsendTE,r16
  	;clear rfm-states
    ;but buffer secure still activ !
;    sbr r16,1<<BUFsecure  
	sts	COMTXstate,r16
 	ret
 


;MANsub
_TXmanSUB:
    ;*******************
	;store pointer
	;*******************
	sts	TXbufPOINTl,yl
	sts	TXbufPOINTh,yh
    ld  r16,y
    ;TEST
	swap	r16				;nibbles tauschen
	andi	r16,0b11110000	;getauschte nibbles freigeben
	mov		zl,r16
	call	_encodeMAN	;encoden ! (ergebnis yl)
	mov		r16,yl		;lower nibble
	;flag setzen
    lds     r17,MANstate
	sbr		r17,1<<MANnibble
	sts		MANstate,r17

	ldi	zh,high(_TXdata)		
	mov zl,r16				;data from tx-buffer
	sts	toMSB,zh
	sts	toLSB,zl
    ret
